/* calc_allbsmod
 * This program evaluates a velocity value for all kinds of
 * B-spline model (continuous, singly split, multiply split).
 * 
 * Input:
 * 	B-Spline Model ---- $$$$.XBS
 *	lat ---- latitude
 * 	lon ---- longitude
 *	r   ---- depth in km
 * Output:
 *	dv  ---- velocity perturbation dv/v with respect to PREM v
 * 
 * Remark:
 *   	to calculate many velocity values, simply go to main to make small
 * 	changes.
 */
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <assert.h>
#include "calc_S362D1.h"

/* -------------utility subroutines --------------------*/
/* -------------utility subroutines --------------------*/
/* -------------utility subroutines --------------------*/
float *vector(int nl, int nh);
int *ivector(int nl, int nh);
double *dvector(int nl, int nh);
float **matrix(int nrl, int nrh, int ncl, int nch);
double **dmatrix(int nrl, int nrh, int ncl, int nch);
int **imatrix(int nrl, int nrh, int ncl, int nch);
void free_vector(float *v, int nl, int nh);
void free_ivector(int *v, int nl, int nh);
void free_dvector(double *v, int nl, int nh);
void free_matrix(float **m, int nrl, int nrh, int ncl, int nch);
void free_dmatrix(double **m, int nrl, int nrh, int ncl, int nch);
void free_imatrix(int **m, int nrl, int nrh, int ncl, int nch);
int *IntVec(int nl, int nh);
void FreeIntVec(int *v, int nl, int nh);
SPHERE Geo2Sph(float lat, float lon, float r);
float *Vector(int nl, int nh);
FILE *OpenFl(char *name);
int GetLine(char *s, int lim);
int FGetLine(FILE *fp, char *s, int lim);
void nrerror(char *error_text);
void FreeVector(float *v, int nl, int nh);
void FreeMatrix(float **m, int nrl, int nrh, int ncl, int nch);

/* --------model evaluation routines------------------ */
/* --------model evaluation routines------------------ */
/* --------model evaluation routines------------------ */
int SelModel(MODEL *model);
void GetModel(MODEL *model, int flag);
void ReadSPHBSPL2(MODEL *model);
void ReadBSPSPLT(MODEL *model);
void ReadBSMULTISPLIT(MODEL *model);
void ReadBS1stLine(FILE *fp, int *nsplit, int *nnode,int *nlayer, int *nrad_um, int *nrad_lm);
void ReleaseModel(MODEL *model);
float fdelta(float delta, float h);
int BSplKer(float *B, float x, float *xnode, int N);
void SphBkernel(float**, SPHERE, float, int, float*, int*, int*);
float CalBSVal(MODEL model, float *rker,int irr, float *hker, int *iker,
		int nrad, int nnode);
float CalBSVal_LM(MODEL model, float *rker,int irr, float *hker, 
		int *iker,int nrad, int nnode);
float CalBSVal_UM(MODEL model, float *rker,int irr, float *hker, 
		int *iker,int nrad, int nnode);
float CalBSVal_MULTISPLIT(MODEL model, float *rker,int irr, float *hker, int *iker, int *nbslayer);
int find_region_split(float r, int nrad, MODEL model, int uplow);
int find_region(float r, int nrad, MODEL model);
int find_region_multisplit(float r, MODEL model, int *layernum);
float  ArcDist(SPHERE s1, SPHERE s2);
void Calc_S362C0(MODEL *model1,float r, float lat, float lon, float *dd);
void Calc_S362D1(MODEL *model1,float r, float lat, float lon, float *dd);
void Calc_S362MULT(MODEL *model1,float r, float lat, float lon, float *dd);
void slicer_gcpath(int *numpt, float *ddelta, float *xlat, float *xlon);
void slicer_by_pole(int *numpt, float *ddelta, float *xlat, float *xlon);

main(){
MODEL model;
FILE  *fp;
char str[MAXSTR],  outfile[MAXSTR];
int numpt, i, j;
float ddelta, delta0, delta, del;
float xlat[MAXPT_XSEC], xlon[MAXPT_XSEC];
float r, rr, lat, lon,dv, rmax;
float scale_r;
int ichoice;
	
  	GetModel(&model,TRUE);	/* read in spherical harmonic model */
	printf("choose: [1=regional xsection 2=full xsection:\n");	
	GetLine(str,MAXSTR);
	sscanf(str, "%d", &ichoice);
	if(ichoice>2 || ichoice<0) {
		printf("choice not found!\n");
		exit(-1);
	}
 	printf("output file name:\n");
  	GetLine(str, MAXSTR);
  	sscanf(str, "%s", outfile);
  	fp = fopen(outfile, "w");
	if(ichoice==1){ /* regional x-section */
		slicer_gcpath(&numpt, &ddelta, xlat, xlon);
		printf("radial scaling for regional xsection(1=normal 1.5=r*1.5):\n");
		scanf("%f", &scale_r);
		delta0=90-ddelta*(numpt-1)*0.5;
	}else{
		slicer_by_pole(&numpt,&ddelta,xlat,xlon);	
		scale_r=1.0;
		delta0=ddelta;
	}
/* plot cross-section */
	r=6371.-2860;
	rmax=6371.-30.;
	rr=r*scale_r;
	while(r<rmax){
		delta=delta0;
		for(i=0;i<numpt;i++){ 
		   if(model.mod==SPHBSPL2) Calc_S362C0(&model,r,xlat[i],xlon[i], &dv);
		   if(model.mod==BSPSPLT) Calc_S362D1(&model,r,xlat[i],xlon[i], &dv);
		   if(model.mod==BSMULTISPLIT) Calc_S362MULT(&model,r,xlat[i],xlon[i],&dv);
		   fprintf(fp, "%.3f %.3f %.6f\n", delta, rr, dv*100);
		   delta+=ddelta;
		}
		printf("processing depth = %f\n", 6371.-r);
		r+=20;
		rr=r*scale_r;  /* change aspect ratio of regional xsec */
 
	}
  	fclose(fp);
  	ReleaseModel(&model);
}

void Calc_S362C0(MODEL *model1,float r, float lat, float lon, float *dd)
{
float **au,**bu,*ar,*br;
float dr,rtop,rbot,B[4],*HB,hh,xx,*ha,**hker,**rker;
float vsp2,vpremp,vprems;
int i,j,k,nnode,nrad,ndata,*ia,*ihb,iht,mrpt;
int irr,nrdat,**iker;
SPHERE s;
PREM   *par;

  mrpt=1;			/* max points in r direction to cal kernels */
  
  nnode=model1->lmaxum;
  nrad=model1->kmaxum;
  au=model1->au;
  ar=model1->ar;
  printf("kmax=%d\n",nrad);
  printf("number of nodes=%d\n",nnode);
  printf("average distance between nodes = %12.4f\n",model1->rmid/RAD);
/*
 * calculate radial kernels
 */
  rker=matrix(1,mrpt,1,4);	/* to save kernels */
  
  BSplKer(B,r,ar,nrad);
  irr=find_region(r, nrad, *model1);
  printf(" r = %f, irr = %d\n",r, irr);
  for(i=0;i<4;i++)rker[1][i+1]=B[i];
  
/*
 * calculate horizontal kernel
 */
  hker=matrix(1,1,1,20);	/* to save horizontal kernels for each data point */
  iker=imatrix(1,1,0,20);	/* index */
  ihb=IntVec(1,20);
  HB=vector(1,20);
  hh=model1->rmid;
  s=Geo2Sph(lat,lon,1.0);
  SphBkernel(au,s,hh,nnode,HB,ihb,&iht);
  iker[1][0]=iht;
  for(j=1;j<=iht;j++){
          iker[1][j]=ihb[j];
          hker[1][j]=HB[j];
  }
/*
 * calculate model values at depths
 */
  s=Geo2Sph(lat,lon,1.0);
  *dd=CalBSVal(*model1, rker[1],irr,hker[1],iker[1],nrad,nnode);
/*
 * release allocated memories
 */  
  free_vector(HB,1,20);
  free_matrix(rker,1,mrpt,1,4);	/* to save kernels */
  free_imatrix(iker,1,1,0,20);
  free_matrix(hker,1,1,1,20);
  FreeIntVec(ihb,1,20);
}

void Calc_S362D1(MODEL *model1,float r, float lat, float lon, float *dd)
{
float **au,*ar,*br;
float **bu,**bl;
float dr,rtop,rbot,B[4],*HB,hh,xx,*ha,**hker,**rker;
float vsp2,vpremp,vprems;
int irr;
int i,j,k,l,nnode,nrad, nrad_um,nrad_lm;
int ndata,*ia,*ihb,iht,mrpt,*iar;
int numata,numatd,nrdat_um, nrdat_lm,nrdat;
int **iker;
SPHERE s;

  mrpt=1;			/* max radial pts to calc kernels */  
  nnode=model1->lmaxum;
  nrad_um=model1->kmaxum;
  nrad_lm=model1->kmaxlm;
  nrad = nrad_um+nrad_lm;
  au=model1->au;
  ar=model1->ar;
  br=model1->br;
/*
  printf("Total kmax=%d\n", nrad);
  printf("UM kmax=%d\n",nrad_um);
  printf("LM kmax=%d\n",nrad_lm);
  printf("number of nodes=%d\n",nnode);
*/

  hker=matrix(1,1,1,20);	/* save horizontal kernels for */
				/* each data point */
  iker=imatrix(1,1,0,20);	/* index */
  ihb=IntVec(1,20);
  HB=vector(1,20);
  hh=model1->rmid;

/*
 * cal kernels in horizontal direction
 */
  s=Geo2Sph(lat,lon,1.0);
  SphBkernel(au,s,hh,nnode,HB,ihb,&iht);  
  iker[1][0]=iht;   /* iker[i][0] stores the total number */
			/* of non-zero elements */
  for(j=1;j<=iht;j++){
	iker[1][j]=ihb[j];
	hker[1][j]=HB[j];
  }

/*
 * calculate kernels in radial direction
 */
  rker=matrix(1,mrpt,1,4);	/* to save kernels */

  irr=find_region_split(r,nrad_lm,*model1, 0);
  if(irr>0) {	/* lower mantle depth */
        BSplKer(B,r,br,nrad_lm);
  }else{		/* upper mantle depth */
        irr=find_region_split(r,nrad_um,*model1, 1);
	if(irr < 0) {
		printf("depth out of range, exit...\n");
		exit(-1);
	}
        BSplKer(B,r,ar,nrad_um);
	irr += nrad_lm;
  }
  for(i=0;i<4;i++)rker[1][i+1]=B[i]; 

/*
 * calculate model value
 */
  s=Geo2Sph(lat,lon,1.0);
  if(irr<=nrad_lm) {
	*dd=CalBSVal_LM(*model1, rker[1],irr,hker[1],iker[1],
			nrad_lm,nnode);     
  }else {
	*dd=CalBSVal_UM(*model1, rker[1],irr-nrad_lm,hker[1],
			iker[1],nrad_um,nnode);     
  }                   
/*
 * release allocated memories
 */  
  free_vector(HB,1,20);
  free_matrix(rker,1,mrpt,1,4);
  free_imatrix(iker,1,1,0,20);
  free_matrix(hker,1,1,1,20);
  FreeIntVec(ihb,1,20);
}

void Calc_S362MULT(MODEL *model1,float r, float lat, float lon, float *dd)
{
float **au,*ar,*br;
float **bu,**bl;
float dr,rtop,rbot,B[4],*HB,hh,xx,*ha,**hker,**rker;
float vsp2,vpremp,vprems;
float **rknot;
int irr, *ilayer;
int i,j,k,l,nnode,nrad, nrad_um,nrad_lm;
int ndata,*ia,*ihb,iht,mrpt,*iar;
int numata,numatd,nrdat_um, nrdat_lm,nrdat;
int **iker;
int nbslayer[10], templayer,ntotal;
SPHERE s;

  mrpt=1;			/* max radial pts to calc kernels */  
  nnode=model1->lmaxum;
  nrad_um=model1->kmaxum;
  nrad_lm=model1->kmaxlm;
  nrad = nrad_um+nrad_lm;
  au=model1->au;
  ar=model1->ar;
  br=model1->br;
  printf("Total kmax=%d\n", nrad);
  printf("UM kmax=%d\n",nrad_um);
  printf("LM kmax=%d\n",nrad_lm);
  printf("number of nodes=%d\n",nnode);

  hker=matrix(1,1,1,20);	/* save horizontal kernels for */
				/* each data point */
  iker=imatrix(1,1,0,20);	/* index */
  ihb=IntVec(1,20);
  HB=vector(1,20);
  hh=model1->rmid;
  ilayer =IntVec(1,mrpt);	/* designed to keep track of layer 
				   of the multiply split model */

/*
 * cal kernels in horizontal direction
 */
  s=Geo2Sph(lat,lon,1.0);
  SphBkernel(au,s,hh,nnode,HB,ihb,&iht);  
  iker[1][0]=iht;   /* iker[i][0] stores the total number */
			/* of non-zero elements */
  for(j=1;j<=iht;j++){
	iker[1][j]=ihb[j];
	hker[1][j]=HB[j];
  }

/*
 * calculate kernels in radial direction
 */
  rknot = model1->rknot;
  rker=matrix(1,mrpt,1,4);	/* to save kernels */

  irr=find_region_multisplit(r,*model1, &l);
  if(irr < 0) {
	printf("depth out of range, exit...\n");
	exit(-1);
  }else{
        BSplKer(B,r,rknot[l],model1->nbsperlay[l-1]);
  }
  for(i=0;i<4;i++)rker[1][i+1]=B[i]; 


/* 
 *  store accumulative layer info in nbslayer
 */
  ntotal=model1->nsplit[0]+model1->nsplit[1];
  printf("total number of splits = %d\n", ntotal);
  templayer=0;
  for(i=0;i<ntotal;i++){
	nbslayer[i]=model1->nbsperlay[i]+templayer;
	templayer=nbslayer[i];
  }

/*
 * calculate model value
 */
  s=Geo2Sph(lat,lon,1.0);
  *dd=CalBSVal_MULTISPLIT(*model1, rker[1],irr,hker[1],
			iker[1],nbslayer);     
/*
 * release allocated memories
 */  
  free_vector(HB,1,20);
  free_matrix(rker,1,mrpt,1,4);
  free_imatrix(iker,1,1,0,20);
  free_matrix(hker,1,1,1,20);
  FreeIntVec(ihb,1,20);
}


void SphBkernel(float **au, SPHERE s1, float hh, int nnode, float *HB, int *ihb, int *iht)
{
float h2=2.0*hh,delta,fac;
int i;
SPHERE s0;

   s0.r=1.0;
   *iht=0;
   for(i=1;i<=nnode;i++){
      s0.theta=au[1][i];
      s0.phi  =au[2][i];
      delta=ArcDist(s0,s1);
      if(delta<=h2){
         (*iht)++;
         HB[*iht]=fdelta(delta,hh);
         ihb[*iht]=i;
      }
   }
}

int SelModel(MODEL *model)
{
static char *defname[29]={ 
       "prem.dat",
       "m84a.dat",
       "m84c.dat",
       "l02_56.dat",
       "v3.dat",
       "sh425.2.dat",
       "sh8wm13.dat",
       "sh8u4l8.dat",
       "sh12wm13.dat",
       "ulmodel.dat",
       "wmmodel.dat",
       "surcom.dat",
       "mdlsh.dat",
       "zhangmodel.dat",
       "mbmodel.dat",
       "inoue_sph.dat",
       "SAF.dat",
       "gridm.dat",
       "richards_d.dat",
       "dtaniso.dat",
       "crust.dat",
       "grand.dat",
       "kappamu.dat",
       "ichetero.dat",
       "sphbspline.dat",
       "sphbsp2d.dat",
       "bspline.dat",
       "bspline_split.dat",
       "bspline_multisplit.dat"
        };
static char *mdldsc[29]={
       "PREM spherical Earth's model         ",
       "M84A,    upper mantle model          ",
       "M84C,    upper mantle model          ",
       "L02.56,  lower mantle model          ",
       "V3,      lower mantle model          ",
       "SH425.2, upper and lower mantle model",
       "SH8WM13, whole mantle model          ",
       "SH8U4L8, upper and lower mantle model",
       "S12_WM13, whole mantle model         ",
       "Harvard  upper-lower mantle model    ",
       "Harvard format whole mantle model    ",
       "Complex spherical  harmonics model   ",
       "Tanimoto's 11 layer model (MDLSH)    ",
       "Zhang & Tanimoto degree 36 model     ",
       "Masters & Bolton's model             ",
       "Inoue's model in spherical harmonics ",
       "SAF or SAT model   (Li, et al.)      ",
       "Grid value model                     ",
       "Mard Richard's density model         ",
       "Travel-time res due to IC anisotropy ",
       "Crust Model                          ",
       "Steve Grand's Model                  ",
       "Mu & Kappa model                     ",
       "Inner core heterogeneity model       ",
       "B-spline model (3D, spherical)       ",
       "B-spline model (2D, spherical)       ",
       "B-spline model (3D, cartesian)       ",
       "Split B-spline model (3D, spherical) ",
       "Multiply-Split B-spline model (3D)   "
       };

static char name1[MAXSTR];
char s[MAXSTR];
int i,sel=-1;

    printf("---------------------------------------------\n");
    printf("Input model type:\n");
    printf("-----------------------------------\n");
    printf("1. continuous model format \n");
    printf("2. single-split model format \n");
    printf("3. multiple-split model format\n");
    printf("-----------------------------------\n");
    printf("enter:\n");
    GetLine(s, MAXSTR);    
    printf("select  model:");
    sscanf(s, "%d",&sel);
    if(sel==1) sel=24;
    if(sel==2) sel=27;
    if(sel==3) sel=28;
          
    if(sel>=PREMDL  && sel<= BSMULTISPLIT){
       model->dsc = mdldsc[sel];
       printf("model description = %s\n",model->dsc);
       printf("the name of the model file (%s)\n",defname[sel]);
       printf("hit return to use default name:");
       GetLine(s,MAXSTR);
       if(s[0]=='\n'){
          model->name1=defname[sel];
          if(sel==PREMDL)model->name2=defname[sel];
       }else{
          sscanf(s,"%s",name1);
          model->name1=name1;
       }
    } else {
          return FALSE;
    }
    model->mod=sel;
/*
 * get file name for the PREM parameter
 */
    if(sel==SH12WM13 || sel==SH8WM13 || sel==WMMODEL || sel==L0256 ||
       sel==SH425    || sel==SH8U4L8 || sel==ULMODEL || sel==MRD   ||
       sel==M84C     || sel==M84A    || sel==WMMK){
/*       printf("the name of PREM parameter file (%s)\n",defname[PREMDL]);
       printf("hit return to use default name:");
       GetLine(s,MAXSTR); */
       s[0]='\n';
       if(s[0]=='\n'){
          model->name2=defname[PREMDL];
       }else {
          sscanf(s,"%s",name1);
          model->name2=name1;
       }
    }
    return TRUE;
}


void GetModel(MODEL *model, int flag)
{
   char str[MAXSTR];
   static char name1[MAXSTR];

    while(!SelModel(model));     /* select model                  */
    if(model->mod== SPHBSPL2){
       ReadSPHBSPL2(model);
       return;
    }else if(model->mod== BSPSPLT){
       ReadBSPSPLT(model);       /* space for the model only      */    
       return;
    }else if(model->mod== BSMULTISPLIT){
       ReadBSMULTISPLIT(model);      
       return;   /* everything is done in ReadBSPSPLT */
    }else {
       printf(" not complete yet\n");
       exit(1);
    }
}

void ReadSPHBSPL2(MODEL *model)
{
float **au,**bu,*ar,*br,lat,lon,coeff, **bsloc;
int  i,j,n,nb,np,nnode,nrad,**ajnode,dum[7];
char txt[132];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */

  FGetLine(fp,txt,132);		/* get the first line */
  sscanf(txt,"%d %d",&nnode,&nrad); /* # of nodes and # of radial nodes */
  model->mod=SPHBSPL2;
  model->lmaxum=nnode;			/* save number of nodes */
  model->kmaxum=nrad;			/* number of nodes in radial direction */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */

  if(nrad<=1){				/* model defined on surface */
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad>1){			/* defined as 3D model */
	printf("number of nodes in radial direction=%d\n",nrad);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	ar=model->ar=vector(-1,nrad+2);			/* for cooridinate of radial nodes */
      bu=model->bu=matrix(1,nrad,1,nnode);	/* for model coefficients */
  }else{
      printf("number of radial nodes has to be greater than four\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,132);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  
  if(nrad<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&br[i]);	/* model coefficients */
/*
     for(i=1;i<=nnode;i++){
         printf("%12.4e",br[i]);
         if(!(i%5))printf("\n");
     }
*/
  }else{
     for(j=1;j<=nrad;j++){
         fscanf(fp,"%*d %f",&ar[j]);				/* radial knots	*/
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[j][i]);	/* model coefficients */
     }
     model->rbot=RADIUS-ar[nrad];
     model->rtop=RADIUS-ar[1];
     ar[-1]=ar[0]=ar[1];
     ar[nrad+2]=ar[nrad+1]=ar[nrad];
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];				/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){				/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;  
  model->rmid=coeff;				/* save average distance */
  
  fclose(fp);
}

void ReadBSPSPLT(MODEL *model)
{
/* read split (discontinuous) B-spline model */
float **au,**bu, **bl, *ar,*br,lat,lon,coeff;
float **bsloc;
int  i,j,n,nb,np,nnode,nrad_lm, nrad_um,**ajnode,dum[7];
char typestr[50],txt[MAXSTR];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */

  FGetLine(fp,txt,MAXSTR);		/* get the first line */
  sscanf(txt,"%s %d %d %d",typestr,&nnode,&nrad_lm, &nrad_um); /* # of nodes and # of radial nodes */
  printf("%d %d %d\n", nnode, nrad_um, nrad_lm);
  model->mod=BSPSPLT;
  model->lmaxum=nnode;			/* save number of nodes */
  model->lmaxlm=nnode;
  model->kmaxum=nrad_um;		/* number of nodes in upper man */
  model->kmaxlm=nrad_lm;		/* number of nodes in lower man */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */

  if(nrad_um==1){			/* model defined on surface */
	printf("Check Split Model input!!\n");
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad_um>1){			/* defined as 3D model */
	printf("number of radial knots in lower mantle=%d\n",nrad_lm);
	printf("number of radial knots in upper mantle=%d\n",nrad_um);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	ar=model->ar=vector(-1,nrad_um+2);      /* for location of UM radial nodes */
	br=model->br=vector(-1,nrad_lm+2);      /* for location of LM radial nodes */
        bu=model->bu=matrix(1,nrad_um,1,nnode);	/* Upper Mantle coefficients */
        bl=model->bl=matrix(1,nrad_lm,1,nnode);	/* Lower Mantle coefficients */
  }else{
      printf("number of radial nodes has to be greater than 4\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,MAXSTR);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  
  if(nrad_lm<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[1][i]);	        /* model coefficients */
  }else{
     for(j=1;j<=nrad_lm;j++){
         fscanf(fp,"%*d %f",&br[j]);				/* radial knots	*/
	 printf( "lower mantle knot %d  %f\n", j, br[j]);
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bl[j][i]);	/* model coefficients */
     }
     for(j=1;j<=nrad_um;j++){
         fscanf(fp,"%*d %f",&ar[j]);				/* radial knots	*/
	 printf( "Upper mantle knot %d  %f\n", j, ar[j]);
         for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[j][i]);	/* model coefficients */
     }
     model->rbot=RADIUS-ar[nrad_um];    /* surface of the earth */
     model->rtop=RADIUS-br[1]; 		/* CMB */
     ar[-1]=ar[0]=ar[1];		/* to satisify boundary condition of B-splines*/
     br[-1]=br[0]=br[1];
     ar[nrad_um+2]=ar[nrad_um+1]=ar[nrad_um];
     br[nrad_lm+2]=br[nrad_lm+1]=br[nrad_lm];
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];		/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){			/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;
  
  printf("average distance between nodes = %12.4f\n",coeff/RAD);
  
  model->rmid=coeff;				/* save average distance */
  
  fclose(fp);
}

void ReadBSMULTISPLIT(MODEL *model)
{
/*  This is for the split (discontinuous) B-spline model.
 *  This version allows multiple splits in the model, it
 *  should also work for single split.
 *  implemented by Yu Gu, 1999.  
 */
float **au,**bu, **bl, *ar,*br,lat,lon,coeff;
float **rknot;
float **bsloc;
int  i,j,k,m,n,nb,np,nnode;
int  nrad_lm, nrad_um,**ajnode,dum[7], nlayer;
char txt[MAXSTR];
SPHERE s1,s2;
FILE *fp;

  fp=OpenFl(model->name1); 	/* open model file    */
  
  ReadBS1stLine(fp, model->nsplit, &nnode, model->nbsperlay, &nrad_um, &nrad_lm);
  model->mod=BSMULTISPLIT;
  model->lmaxum=nnode;			/* save number of nodes */
  model->lmaxlm=nnode;
  model->kmaxum=nrad_um;		/* number of nodes in upper man */
  model->kmaxlm=nrad_lm;		/* number of nodes in lower man */
  au=model->au=matrix(1,2,1,nnode);	/* au[1][*] --- lat, au[2][*] --- lon */
  bsloc=model->bsloc=matrix(1,2,1,nnode); /* bsloc[1][*] --- lat, bsloc[2][*] --- lon */
  model->ajnode=ajnode=imatrix(0,7,1,nnode);	/* ajacent nodes */

  if(nrad_um==1){			/* model defined on surface */
	printf("Check Split Model input!!\n");
	printf("number of nodes =%d\n",nnode);
	model->mantle=SURFACE;		/* defined on spherical surface		*/
	model->radfun=0;
	br=model->br=vector(1,nnode);	/* model coefficients */
	model->dim=2;
  }else if(nrad_um>1){			/* defined as 3D model */
	printf("number of radial knots in lower mantle=%d\n",nrad_lm);
	printf("number of radial knots in upper mantle=%d\n",nrad_um);
	printf("number of horizontal nodes for each radial node =%d\n",nnode);
	model->radfun=0;
	model->dim=3;
	nlayer = model->nsplit[0]+model->nsplit[1];
	printf("nlayer = %d\n", nlayer);
	rknot=model->rknot=matrix(1,nlayer,-1,nrad_um+nrad_lm+2);  /* for radial node loc */
        bu=model->bu=matrix(1,nrad_um,1,nnode);	/* Upper Mantle coefficients */
        bl=model->bl=matrix(1,nrad_lm,1,nnode);	/* Lower Mantle coefficients */
  }else{
      printf("number of radial nodes has to be greater than 4!\n");
      exit(1);
  }
  
  for(i=1;i<=nnode;i++){	/* read coor of horizontal nodes */
      FGetLine(fp,txt,132);
	sscanf(txt,"%*d %f %f %d %d %d %d %d %d %d",
	  &lat,&lon,&n,&dum[1],&dum[2],&dum[3],&dum[4],&dum[5],&dum[6]);
/*	printf("%f %f %d %d %d %d %d %d %d\n",
	  lat,lon,n,dum[1],dum[2],dum[3],dum[4],dum[5],dum[6]);
*/
	bsloc[1][i]=lat;
	bsloc[2][i]=lon;
	ajnode[0][i]=n;
	for(j=1;j<=n;j++)ajnode[j][i]=dum[j];
	s1=Geo2Sph(lat,lon,1.0);
	au[1][i]=s1.theta;
	au[2][i]=s1.phi;
  }
  n=1;
  if(nrad_lm<=1){
     for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[1][i]);	   /* model coefficients */
  }else{
     for(k=1;k<=nlayer;k++){
     	for(j=1;j<=model->nbsperlay[k-1];j++){
         	fscanf(fp,"%*d %f",&rknot[k][j]);	   /* radial knots	*/
		if(n<=nrad_lm){
         		for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bl[n][i]);   /* model coefficients */
	 		printf( "lower mantle knot k=%d, j=%d, %f\n", k,j,rknot[k][j]);
		}else{
			m=n-nrad_lm;
         		for(i=1;i<=nnode;i++)fscanf(fp,"%f",&bu[m][i]);   /* model coefficients */
	 		printf( "upper mantle knot k=%d, j=%d, %f\n", k,j,rknot[k][j]);
		}
		n++;
     	}
     }
     model->rbot=RADIUS-rknot[nlayer][model->nbsperlay[nlayer-1]];    /* surface of the earth */
     model->rtop=RADIUS-rknot[1][1]; 		/* CMB */
     printf("rtop = %f, rbot = %f\n", model->rtop, model->rbot);
     for(i=1;i<=nlayer;i++){
     	rknot[i][-1]=rknot[i][0]=rknot[i][1];		/* to satisify boundary condition of B-splines*/
     	rknot[i][model->nbsperlay[i-1]+2]=rknot[i][model->nbsperlay[i-1]];
	rknot[i][model->nbsperlay[i-1]+1]=rknot[i][model->nbsperlay[i-1]];
     }
  }
/*  
  for(i=1;i<=nnode;i++){
	printf("%d %f %f %d %d %d %d %d %d %d\n",
	i,au[1][i],au[2][i],ajnode[0][i],ajnode[1][i],ajnode[2][i],
	ajnode[3][i],ajnode[4][i],ajnode[5][i],ajnode[6][i]);
  }
*/
    
  for(n=0,coeff=0.0,i=1;i<=nnode;i++){	/* cal avg distance between nodes */
	np=ajnode[0][i];		/* total number of neighboring points */
	s1.theta=au[1][i];
	s1.phi=au[2][i];
	for(j=1;j<=np;j++){
	    nb=ajnode[j][i];
	    if(nb>i){			/* calculate only once */
		s2.theta=au[1][nb];
		s2.phi=au[2][nb];
		coeff+=ArcDist(s1,s2);
		n++;
	    }
	}
  }
  
  coeff/=(float)n;
  
  printf("average distance between nodes = %12.4f\n",coeff/RAD);
  
  model->rmid=coeff;				/* save average distance */
  
  fclose(fp);
}

void ReadBS1stLine(FILE *fp, int *nsplit, int *nnode,int *nlayer, int *nrad_um, int *nrad_lm)
{  /* this is for multiple splits in the B-spline model 
    * Correct format (separated by space(s)):
    *  	1.   MULTISPLITM  
    *   2.   number of lower mantle split layers
    *   3.   number of upper mantle split layers
    *   4.   number of horizontal splines
    *   5.   number of radial splines in lower mantle layer(s)
    *   6.   number of radial splines in lower mantle layer(s)
    *   sample input:   (split at 670, 1 layer in LM, 2 in UM)
			MULTISPLITM 1 2 362 7 3 4
    */
char txt[MAXSTR], typestr[50];
int ntotal, i,j;

  FGetLine(fp,txt,MAXSTR);		/* get the first line */
  sscanf(txt, "%s %d %d", typestr, &nsplit[0], &nsplit[1]);
  printf("# lower mantle split  = %d, upper mantle = %d\n", nsplit[0], nsplit[1]);
  ntotal = nsplit[0]+nsplit[1];
  switch(ntotal) {
	case 1:
		sscanf(txt, "%s %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0]);
		printf("%s %d %d %d %d\n", typestr, &nsplit[0], &nsplit[1], *nnode, nlayer[0]);
		break;
	case 2:
		sscanf(txt, "%s %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1]);
		printf("%s %d %d %d %d %d\n", typestr, &nsplit[0], &nsplit[1], *nnode, nlayer[0], nlayer[1]);
		break;
	case 3:
		sscanf(txt, "%s %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1],
			&nlayer[2]);
		printf("%s %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2]);
		break;
	case 4:
		sscanf(txt, "%s %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], &nlayer[1],
			&nlayer[2], &nlayer[3]);
		printf("%s %d %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2], nlayer[3]);
		break;
	case 5:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4]);
		printf("%s %d %d %d %d %d %d %d %d\n", typestr, nsplit[0], nsplit[1], *nnode, nlayer[0], nlayer[1], nlayer[2], nlayer[3], nlayer[3]);
		break;
	case 6:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4], &nlayer[5]);
		break;
	case 7:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode, &nlayer[0], 
			&nlayer[1],&nlayer[2], &nlayer[3], &nlayer[4], &nlayer[5], &nlayer[6]);
		break;
	case 8:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,&nlayer[0], 
			&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],&nlayer[6],
			&nlayer[7]);
		break;
	case 9:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,
			&nlayer[0],&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],
			&nlayer[6],&nlayer[7],&nlayer[8]);
		break;
	case 10:
		sscanf(txt, "%s %d %d %d %d %d %d %d %d %d %d %d %d %d %d", typestr,&nsplit[0], &nsplit[1],nnode,
			&nlayer[0],&nlayer[1],&nlayer[2],&nlayer[3],&nlayer[4],&nlayer[5],
			&nlayer[6],&nlayer[7],&nlayer[8],&nlayer[9]);
		break;
	default:
		printf("First line of model not reasonable!\n");
		break;
  }
  *nrad_lm=0;
  *nrad_um=0;
  for(i=0;i<nsplit[0];i++){
	*nrad_lm=*nrad_lm+nlayer[i];
	printf("lower mantle, splines=%d\n", nlayer[i]);
  }
  for(i=0;i<nsplit[1];i++){
	*nrad_um=*nrad_um+nlayer[i+nsplit[0]];
	printf("upper mantle, splines=%d\n", nlayer[i+nsplit[0]]);
  }
}
/* --------------MODEL CALCULATION ROUTINES ------------------- */
/* ------------------------------------------------------------ */
/* ------------------------------------------------------------ */

/*
 * to convert geographic latitude and longitude (in degree)
 * to spherical coordinate
 */
SPHERE Geo2Sph(float lat, float lon, float r)
{
SPHERE s;

   s.theta=(90.0-lat)*RAD;
   if(lon<0.0)lon+=360.0;
   s.phi  = lon*RAD;
   s.r    = r;
   
   return s;
}


void ReleaseModel(MODEL *model)
{
int num,lmaxum,lmaxlm,kmaxum,kmaxlm,lmax,llmax,nnode,nrad;
   
    if(model->mod==BSPL3){   /* for B-spline model */
       num=model->ixyz[0]*model->ixyz[1]*model->ixyz[2];
       FreeVector(model->ar,1,num);
       return;
    }else if(model->mod==SPHBSPL2){	 /* for spherical spline model */
       nnode=model->lmaxum;			/* number of nodes */
       nrad=model->kmaxum;			/* number of nodes in radial direction */
       FreeMatrix(model->au,1,2,1,nnode);	/* node coordinates */

       if(nrad==0){
          FreeVector(model->br,1,nnode);	/* model coefficients */
       }else{
	    FreeVector(model->ar,-1,nrad+2);		/* for cooridinate of radial nodes */
          FreeMatrix(model->bu,1,nrad,1,nnode);		/* for model coefficients */
       }  
       return;
    }
/* 
 * free the allocated memory for upper- and whole mantle
 */   
    if(model->mantle==UPPERMANTLE ||  model->mantle==UPPERLOWERM ||
       model->mantle==WHOLEMANTLE ){
       lmaxum=model->lmaxum;
       kmaxum=model->kmaxum;
       llmax =(lmaxum+2)*(lmaxum+1)/2-1;       
       FreeMatrix(model->au,0,kmaxum,0,llmax);   
       FreeMatrix(model->bu,0,kmaxum,0,llmax);
       FreeVector(model->ac,0,llmax);   
       FreeVector(model->bc,0,llmax);
       FreeVector(model->a670,0,llmax);
       FreeVector(model->b670,0,llmax);
    }
/* 
 * free allocated memory for lower mantle
 */ 
    if(model->mantle==LOWERMANTLE || model->mantle==UPPERLOWERM){
       lmaxlm=model->lmaxlm;
       kmaxlm=model->kmaxlm;
       llmax =(lmaxlm+2)*(lmaxlm+1)/2-1;       
       FreeMatrix(model->al,0,kmaxlm,0,llmax);   
       FreeMatrix(model->bl,0,kmaxlm,0,llmax);
    }
    if(model->mod != BSMULTISPLIT){
    	lmax  = max(lmaxum,lmaxlm);
    	llmax = (lmax+2)*(lmax+1)/2-1;       
   
    	FreeVector(model->ar,0,llmax);
    	FreeVector(model->br,0,llmax);
    }
}

float fdelta(float delta, float h)
{ /* calculates horizontal spline kernel */
float deltil,value;

  deltil=delta/h;
  
  if(deltil>2.0){
     return 0.0;
  }else if(deltil<=1.0){
     value=(4.0+3.0*deltil*deltil*(deltil-2.0));
  }else{
     value=2.0-deltil;
     value*=value*value;
  }
   return value/4.0;
}

int BSplKer(float *B, float x, float *xnode, int N)
{ /* calculate radial spline kernels */
float  *xn,temp,temp1,X1,X2,X3,X4;
int i,ib,ie;
static float A1,A2,A3,A4;
static int i0=0;

  if(N<=0){		/* initialize, A1, A2 etc will be recalculated */
     i0=0;
     return 0;
  }
  if(x<xnode[1] || x>xnode[N])return 0;
  ib=1;
  ie=N;

  while(ie-ib>1){
	i=ib+((ie-ib)>>1);
	if(x<xnode[i])ie=i;
	else ib=i;
  }
  
  i=ib;
  xn=&xnode[i-2];
  if(i!=i0){
	temp=1.0/(xn[3]-xn[2]);
	A1=temp/(xn[3]-xn[1]);
	A3=temp/(xn[4]-xn[2]);
	A4=A3/(xn[5]-xn[2]);
	temp=xn[4]-xn[1];
	A2=A1/temp;
	A3/=temp;
	A1/=xn[3]-xn[0];
	i0=i;
  }
  
  X1=x-xn[1];
  X2=x-xn[2];
  X3=x-xn[3];
  X4=x-xn[4];
  
  temp=X3*X3*A1;
  temp1=X3*X1*A2+X2*X4*A3;
  B[0]=-temp*X3;
  B[1]=(x-xn[0])*temp+X4*temp1;
  temp=X2*X2*A4;
  B[3]=X2*temp;
  B[2]=-X1*temp1-(x-xn[5])*temp;
/*
 * for 2nd derivative = 0
 */

  if(i==1){
     B[2]+=B[1]/3.0;
     B[1]=B[1]/1.5+B[0];
     B[0]=0.0;
  }
  if(i==2){
     B[1]+=B[0]/3.0;
     B[0]/=1.5;
  }
  if(i==N-1){
     B[1]+=B[2]/3.0;
     B[2]=B[2]/1.5+B[3];
     B[3]=0.0;
  }
	
  if(i==N-2){
     B[2]+=B[3]/3.0;
     B[3]/=1.5;
  }
 
  return i;
}

float CalBSVal(MODEL model, float *rker,int irr, float *hker, int *iker,
		int nrad, int nnode)
{ /* calculate model value for a continuous B-spline model */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){
          val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      }
   }
   return val;
}


float CalBSVal_LM(MODEL model, float *rker,int irr, float *hker, int *iker,int nrad, int nnode)
{ /* calculate model value for a Split B-spline model in LM */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){ /* iker[0] is num of nonzero elements */
          val +=rker[i]*hker[j]*model.bl[m][iker[j]];
      }
   }
   return val;
}


float CalBSVal_UM(MODEL model, float *rker,int irr, float *hker, int *iker,int nrad, int nnode)
{ /* calculate model value for a Split B-spline model in UM */
int i,i0,i1,j,k,m,ii,jj;
float rowi;
float val;

   i0=1;
   i1=4;
   if(irr==1)i0=2;
   if(irr==nrad-1)i1=3;

   val = 0.0;
   for(i=i0;i<=i1;i++){
      m=(irr-2+i);
      for(j=1;j<=iker[0];j++){
          val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      }
   }
   return val;
}

float CalBSVal_MULTISPLIT(MODEL model, float *rker,int irr, float *hker, int *iker, int *nbslayer)
{ /* calculate model value for a Multiply Split B-spline model, YG 99. */
int i,i0,i1,j,k,m,ii,jj;
int itemp,nsplit;
float rowi;
float val;

   nsplit=model.nsplit[0]+model.nsplit[1];
/*
   printf("irr=%d  nsplit=%d\n", irr, nsplit);
*/
   i0=1;
   i1=4;
   if(irr==1)i0=2;
   
   for(i=0;i<nsplit;i++){
	if(irr==nbslayer[i]-1){
/*
		printf("set i1=3 due to top layer!\n");
*/
		i1=3;
		break;
   	}
   	if(irr==nbslayer[i]+1){
/*		printf("set i0=2 due to bottom layer!\n"); */
		i0=2;
		break;
   	}
   }
   val = 0.0;
   if(irr>model.kmaxlm) {
	itemp=irr-model.kmaxlm;
   	for(i=i0;i<=i1;i++){
/*		printf("i=%d rker=%f\n", i, rker[i]); */
      		m=(itemp-2+i);
      		for(j=1;j<=iker[0];j++){ /* iker[0] is number of nonzero elements */
          		val +=rker[i]*hker[j]*model.bu[m][iker[j]];
      		}
   	}
   }else{
	itemp=irr;
   	for(i=i0;i<=i1;i++){
/*		printf("i=%d rker=%f\n", i, rker[i]); */
      		m=(itemp-2+i);
      		for(j=1;j<=iker[0];j++){ /* iker[0] is number of nonzero elements */
          		val +=rker[i]*hker[j]*model.bl[m][iker[j]];
      		}
   	}
   }
   return val;
}

int find_region_split(float r, int nrad, MODEL model, int uplow)
{ /* locate between which knots the given radius fall, split model */
  /* uplow = 0 (lower mantle knots), uplow = 1 (upper mantle) */
 int i, ireg;
	
 ireg = -1;	
 if(model.mod!=BSPSPLT){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 if(uplow ==0) { /* lower mantle */
 	for(i=1;i<=nrad;i++){
		if((r+0.01)>=model.br[i] && (r+0.01)<=model.br[i+1]) {
			ireg = i;
			break;
		}
	}
 }else {	/* upper mantle */
 	for(i=1;i<=nrad;i++){
		if((r+0.01)>=model.ar[i] && (r+0.01)<=model.ar[i+1]) {
			ireg = i;
			break;
		}
	}

 }
 return ireg;
}

int find_region(float r, int nrad, MODEL model)
{ /* locate between which knots the given radius fall */
 int i, ireg;
	
 if(model.mod!=SPHBSPL2){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 assert((r+0.01)>=model.ar[1] && (r+0.01)<=model.ar[nrad]);

 ireg = -1;	
 for(i=1;i<=nrad;i++){
	if((r+0.01)>=model.ar[i] && (r+0.01)<=model.ar[i+1]) {
		ireg = i;
		break;
	}
 }
	printf("ireg = %d\n", ireg);
 return ireg;
}

int find_region_multisplit(float r, MODEL model, int *layernum)
{ /* locate between which knots the given radius fall, multiply
   * split model, YG, 1999.
   */
 int i,j,k,l,m,ireg;
 int ntotal;
 float **rknot;
 float rtop, rbot;
 int iffound;

 ntotal=model.nsplit[0]+model.nsplit[1];
 printf("ntotal = %d  r = %f\n", ntotal,r);	
 ireg = -1;	
 if(model.mod!=BSMULTISPLIT){
	printf("model inconsistent!  exit...\n");
	exit(-1);
 }
 rknot=model.rknot;
 iffound=0;
 for(k=0,l=1;l<=ntotal;l++){
    	for(j=1;j<=model.nbsperlay[l-1];j++){ /* loop for every region */
		k++;
		rtop=rknot[l][j+1];
		rbot=rknot[l][j];
		if((r+0.001)>=rbot && (r+0.001)<rtop) {
			ireg=j ;
			iffound=1;
			*layernum=l;
			printf("ireg=%d  rbot=%f rtop=%f\n", ireg, rbot,rtop);
			break;
		}
	}
	if(iffound==1){
		for(m=0;m<l-1;m++){ /* irr is accumulative from 1 to 13 */
        		ireg=ireg+model.nbsperlay[m];
		}
		break;
	}
 }

 return ireg;
}


/* arctwo.c 
 * To calculate the minor arc distance between two points on a sphere
 * input       : s1,s2 -- the coordinates of the two points 
 * return value: the arc length between two poin (in radian)
 */
float  ArcDist(SPHERE s1, SPHERE s2)
{
float arclen;

  arclen = cos((double)s1.theta)*cos((double)s2.theta)+
           sin((double)s1.theta)*sin((double)s2.theta)*
           cos((double)(s1.phi-s2.phi));
  return   acos((double)arclen);
}


/* ------------------ Utility routines ---------------------------
   ------------------ Utility routines ---------------------------
   ------------------ Utility routines ---------------------------
   ------------------ Utility routines ---------------------------
*/

float *vector(int nl, int nh)
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

int *ivector(int nl, int nh)
{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl;
}

double *dvector(int nl, int nh)
{
	double *v;

	v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl;
}



float **matrix(int nrl, int nrh, int ncl, int nch)
{
	int i;
	float **m;

	m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}

double **dmatrix(int nrl, int nrh, int ncl, int nch)
{
	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in dmatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
		m[i] -= ncl;
	}
	return m;
}

int **imatrix(int nrl, int nrh, int ncl, int nch)
{
	int i,**m;

	m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
	if (!m) nrerror("allocation failure 1 in imatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
		if (!m[i]) nrerror("allocation failure 2 in imatrix()");
		m[i] -= ncl;
	}
	return m;
}

void free_vector(float *v, int nl, int nh)
{
	free((char*) (v+nl));
}

void free_ivector(int *v, int nl, int nh)
{
	free((char*) (v+nl));
}

void free_dvector(double *v, int nl, int nh)
{
	free((char*) (v+nl));
}



void free_matrix(float **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_dmatrix(double **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_imatrix(int **m, int nrl, int nrh, int ncl, int nch)
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

int *IntVec(int nl, int nh)
{
  int *v;

  v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
  if (!v) printf("allocation failure in IntVec()\n");
  return v-nl;
}

void FreeIntVec(int *v, int nl, int nh)
{
  free((char*) (v+nl));
}

float *Vector(int nl, int nh)
{
float *v;

  v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
  if (!v) printf("allocation failure in Vector()");
  return v-nl;
}

FILE *OpenFl(char *name)
{
FILE *fp;

  if((fp=fopen(name,"r"))==NULL){
      printf("Cannot open file %s\n",name);
      exit(1);              
  }
  return fp;
}

int GetLine(char *s, int lim)
{
int c, i;

    for(i=0; i<lim-1 && (c=getchar())!=EOF && c!='\n'; ++i)
        s[i]=c;
        
    if(c=='\n' && i==0){
       s[i] =c;
       ++i; 
    }
    s[i] = '\0';
    return i;
}


int FGetLine(FILE *fp, char *s, int lim)
{
char c;
int i;

    for(i=0; i<lim-1 && (fscanf(fp,"%c",&c))!=EOF && c!='\n'; ++i)
        s[i]=c;
    if(c=='\n'){
       s[i] =c;
       ++i;
    }
    s[i] = '\0';
    return i;
}

void FreeVector(float *v, int nl, int nh)
{
  free((char*) (v+nl));
}

void FreeMatrix(float **m, int nrl, int nrh, int ncl, int nch)
{
  int i;

  for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
      free((char*) (m+nrl));
}


void nrerror(char *error_text)
{
/*	void exit(); */

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}

