#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "draw3dlib.h"
extern short *ReadCoast(int flag, int size);


void DepthRange(float rr[], int *n, float *dmin, float *dmax)
{ /* read in a range of depths */
  int i, nbad;
  float depmin, depmax, dep, drr;
 
  printf("depth range (min, max) and number of points:\n");
  scanf("%f %f %d", &depmin, &depmax, n);
  printf("Earth radius = %f\n", *n, RADIUS);
  if((depmax > depmin) && (*n>0)) {
	dep = depmin;
	drr = (depmax-depmin)/(float)(*n-1);
	nbad=0;
	for(i=1;i<=*n;i++){
		rr[i] = RADIUS - dep;
		dep= dep + drr;
		if(rr[i] >= RMOHO || rr[i] <= RCMB) {
			printf("depth not in mantle, ignore...\n");
			nbad = nbad+1;
			continue;
		}
	}
	*n = *n - nbad;
  }else {
	printf("depth range unreasonable, check again!!!\n");
	exit(-1);
  }
  *dmin = depmin;
  *dmax = depmax;
   printf("depmin = %f, depmax = %f, total number of depth = %d\n", *dmin, *dmax, *n);
}

void DepthRange_reverse(float rr[], int *n, float *dmin, float *dmax)
{ /* read in a range of depths, then reverse the order for right-handed
     coordinate system */
  int i, nbad;
  float depmin, depmax, dep, drr;
 
  printf("depth range (min, max) and number of points:\n");
  scanf("%f %f %d", &depmin, &depmax, n);
  if((depmax > depmin) && (*n>0)) {
	dep = depmax;
	drr = (depmax-depmin)/(float)(*n-1);
	nbad=0;
	for(i=1;i<=*n;i++){
		rr[i] = RADIUS - dep;
		dep= dep - drr;
		if(rr[i] >= RMOHO || rr[i] <= RCMB) {
			printf("depth not in mantle, ignore...\n");
			nbad = nbad+1;
			continue;
		}
	}
	*n = *n - nbad;
  }else {
	printf("depth range unreasonable, check again!!!\n");
	exit(-1);
  }
  *dmin = RADIUS - depmax;
  *dmax = RADIUS - depmin;
   printf("depmin = %f, depmax = %f, total number of depth = %d\n", *dmin, *dmax, *n);
}

void GetGrid_draw(float xlat[], float xlon[], float *latmin, float *latmax, int *nlat, float *lonmin, float *lonmax, int *nlon, int *n)
{
  int i,j,k;
  float dlat, dlon, lat, lon;
	printf("enter latitude range (min, max) and number of points:\n");
	scanf("%f %f %d", latmin, latmax, nlat);
	dlat = (*latmax-*latmin)/(*nlat-1);
	printf("enter co-longitude range (min, max) and number of points:\n");
	scanf("%f %f %d", lonmin, lonmax, nlon);
	dlon = (*lonmax-*lonmin)/(*nlon-1);
	lat = *latmin;
	k = 1;
	for(i=1; i<= *nlat;i++){
		lon = *lonmin;
		for(j=1; j<= *nlon;j++){
			xlon[k] = lon;
			while(xlon[k]>180.0) xlon[k]=xlon[k]-360.0;
			xlat[k] = lat;
			lon = lon + dlon;
			k ++;
		}
		lat = lat + dlat;
	}
	*n = k-1;
}


/*
*  # AVS field file 
*  # this is a header file for a field to be 
*  # used in conjunction with the  build a field module of AVS
*  #
*  ndim = 3
*  dim1 = 64
*  dim2 = 32
*  dim3 = 64
*  nspace = 3
*  veclen = 1
*  data = byte
*  field = uniform
*  variable 1 file=sphere.dat filetype=binary skip=3
*/

void output_3dfield(char *outfl, int ndim, int dim1, int dim2, int dim3,int nspace, int veclen, char *data,char *field, int nvar, char *filetype, int nskip)
{
FILE *fp;
char tempstr[128],tempstr1[128],tempstr2[128], tempstr3[128];

   sprintf(tempstr,outfl);
   strcat(tempstr, ".fld");
   printf("%s\n", tempstr); 
   fp = fopen(tempstr, "w");                                                     
   sprintf(tempstr1,outfl);
   strcat(tempstr1, ".x");
   printf("%s\n", tempstr1); 
   sprintf(tempstr2,outfl);
   strcat(tempstr2, ".y");
   printf("%s\n", tempstr1); 
   sprintf(tempstr3,outfl);
   strcat(tempstr3, ".z");
   printf("%s\n", tempstr3); 


   fprintf(fp, "# AVS field file\n#\n");
   fprintf(fp, "ndim = %d\n", ndim);
   fprintf(fp, "dim1 = %d\n", dim1);
   fprintf(fp, "dim2 = %d\n", dim2);
   fprintf(fp, "dim3 = %d\n", dim3);
   fprintf(fp, "coord 1 file=%s  filetype=ascii\n", tempstr1);
   fprintf(fp, "coord 2 file=%s  filetype=ascii\n", tempstr2);
   fprintf(fp, "coord 3 file=%s  filetype=ascii\n", tempstr3);
   fprintf(fp, "nspace = %d\n", nspace);
   fprintf(fp, "veclen = %d\n", veclen);
   fprintf(fp, "data = %s\n", data);
   fprintf(fp, "field = %s\n\n", field);
   fprintf(fp, "variable %d file=%s filetype=%s skip=%d\n", 
            nvar,outfl,filetype,nskip);
   fclose(fp);
   printf("Output field file %s created...\n", tempstr); 
}

/*
void output_3dfield(char *outfl, int ndim, int dim1, int dim2, int dim3,int nspace, int veclen, char *data,char *field, int nvar, char *filetype, int nskip)
{
FILE *fp;
char tempstr[128],tempstr1[128],tempstr2[128], tempstr3[128];

   sprintf(tempstr,outfl);
   strcat(tempstr, ".fld");
   printf("%s\n", tempstr); 
   fp = fopen(tempstr, "w");                                                     
   sprintf(tempstr1,outfl);
   strcat(tempstr1, ".x");
   printf("%s\n", tempstr1); 
   sprintf(tempstr2,outfl);
   strcat(tempstr2, ".y");
   printf("%s\n", tempstr1); 
   sprintf(tempstr3,outfl);
   strcat(tempstr3, ".z");
   printf("%s\n", tempstr3); 


   fprintf(fp, "# AVS field file\n#\n");
   fprintf(fp, "ndim = %d\n", ndim);
   fprintf(fp, "dim1 = %d\n", dim1);
   fprintf(fp, "dim2 = %d\n", dim2);
   fprintf(fp, "dim3 = %d\n", dim3);
   fprintf(fp, "coord 1 file=%s  filetype=ascii\n", tempstr1);
   fprintf(fp, "coord 2 file=%s  filetype=ascii\n", tempstr2);
   fprintf(fp, "coord 3 file=%s  filetype=ascii\n", tempstr3);
   fprintf(fp, "nspace = %d\n", nspace);
   fprintf(fp, "veclen = %d\n", veclen);
   fprintf(fp, "data = %s\n", data);
   fprintf(fp, "field = %s\n\n", field);
   fprintf(fp, "variable %d file=%s filetype=%s skip=%d\n", 
            nvar,outfl,filetype,nskip);
   fclose(fp);
   printf("Output field file %s created...\n", tempstr); 
}
*/

void out_3dlimit(char *outfl, float xmin, float xmax, float ymin, float ymax, float zmin, float zmax) 
{ 
  int i, j;
  FILE *fp;
  char tempstr1[128], tempstr2[128], tempstr3[128];
   sprintf(tempstr1,outfl);
   strcat(tempstr1, ".x");
   printf("%s\n", tempstr1); 
   fp = fopen(tempstr1, "w");
   fprintf(fp, "%f  %f\n",xmin, xmax);    
   fclose(fp);
   sprintf(tempstr2,outfl);
   strcat(tempstr2, ".y");
   printf("%s\n", tempstr2); 
   fp = fopen(tempstr2, "w");
   fprintf(fp, "%f  %f\n",ymin, ymax);    
   fclose(fp);
   sprintf(tempstr3,outfl);
   strcat(tempstr3, ".z");
   printf("%s\n", tempstr3); 
   fp = fopen(tempstr3, "w");
   fprintf(fp, "%f  %f\n",zmin, zmax);    
   fclose(fp);
   

}

void out_coast(char *outfl,float xmin, float xmax, float ymin, float ymax, float zmin, float zmax, float coastpos, int ichoice)
{ /* output coastline file
   * ichoice = 1 plot in xy plane, 2=xz, 3=yz 
   */
   char tempstr[128], tempstr1[128];
   short *dpt;
   int size, count, platenum, number, i, j;
   int npt;
   float lat, lon;
   FILE *fp;

   sprintf(tempstr,outfl);
   strcat(tempstr, ".coast");
   printf("%s\n", tempstr); 
   fp = fopen(tempstr, "w");
   size = COASTSIZE;
   dpt=ReadCoast(COAST,size);
   count=platenum=npt=0;
   while(count<size){
        number=(int)(*(dpt++)/2);
        for(j=0;j<number;j++){
            lon=(float)(*(dpt++))*0.1;	/* longitude (-180 -- 180 ) */
            if(lon<0.0)lon+=360.0;	/* longitude ( 0 -- 360 ) */
            lat=(float)(*(dpt++))*0.1;	/* latitude  (-90  --  90 ) */
	    if(lon>=xmin && lon<=xmax && lat>=ymin && lat<=ymax) {
	     if(ichoice==2) {
	       fprintf(fp,"%f %f %f %f\n", lon,coastpos,lat,2.0);
 	     }else if(ichoice==3){
	       fprintf(fp,"%f %f %f %f\n", coastpos,lon,lat,2.0);
	     }else {
	       fprintf(fp,"%f %f %f %f\n", lon,lat,coastpos, 2.0);
	     }
	     npt = npt+1;
	    }
        }
        count+=2*number+1;
   }
   fclose(fp);
/* below output field file for coastline */
   sprintf(tempstr1,outfl);
   strcat(tempstr1, ".coast.fld");
   printf("%s\n", tempstr1); 
   fp = fopen(tempstr1, "w");
   fprintf(fp, "# AVS field file\n#\n");
   fprintf(fp, "ndim = 1\n");
   fprintf(fp, "dim1 = %d\n", npt);
   fprintf(fp, "nspace = 3\n");
   fprintf(fp, "veclen = 1\n");
   fprintf(fp, "data = float\n");
   fprintf(fp, "field = irregular\n");
   fprintf(fp, "coord 1 file=%s  filetype=ascii offset=0 stride=4\n", tempstr);
   fprintf(fp, "coord 2 file=%s  filetype=ascii offset=1 stride=4\n", tempstr);
   fprintf(fp, "coord 3 file=%s  filetype=ascii offset=2 stride=4\n", tempstr);
   fprintf(fp, "variable 1 file=%s filetype=ascii offset=3 stride=4 skip=0\n", tempstr);   
   fclose(fp);
}

void out_platebound(char *outfl,float xmin, float xmax, float ymin, float ymax, float zmin, float zmax, float boundpos, int ichoice)
{ /* output plate boundary file
   * ichoice = 1 plot in xy plane, 2=xz, 3=yz
   * the position of the plane is at plan 
   */
   char tempstr[128], tempstr1[128];
   short *dpt;
   int size, count, platenum, number, i, j;
   int npt;
   float lat, lon;
   FILE *fp;

   sprintf(tempstr,outfl);
   strcat(tempstr, ".plate");
   printf("%s\n", tempstr); 
   fp = fopen(tempstr, "w");
   size = BOUNDARYSIZE;
   dpt=ReadCoast(BOUNDARY,size);
   count=npt=platenum=0;
   while(count<size){
        number=(int)(*(dpt++)/2);
        for(j=0;j<number;j++){
            lon=(float)(*(dpt++))*0.1;	/* longitude (-180 -- 180 ) */
            if(lon<0.0)lon+=360.0;	/* longitude ( 0 -- 360 ) */
            lat=(float)(*(dpt++))*0.1;	/* latitude  (-90  --  90 ) */
	    if(lon>=xmin && lon<=xmax && lat>=ymin && lat<=ymax) {
	      if(ichoice==2) {
	        fprintf(fp,"%f %f %f %f\n", lon,boundpos,lat, 1.0);
 	      }else if(ichoice==3){
	        fprintf(fp,"%f %f %f %f\n", boundpos,lon,lat,1.0);
	      }else {
	        fprintf(fp,"%f %f %f %f\n", lon,lat,boundpos, 1.0);
	      }
	      npt = npt+1;
	    }
        }
        count+=2*number+1;
   }
   fclose(fp);
/* below output field file for plate boundary */
   sprintf(tempstr1,outfl);
   strcat(tempstr1, ".plate.fld");
   printf("%s\n", tempstr1); 
   fp = fopen(tempstr1, "w");
   fprintf(fp, "# AVS field file\n#\n");
   fprintf(fp, "ndim = 1\n");
   fprintf(fp, "dim1 = %d\n", npt);
   fprintf(fp, "nspace = 3\n");
   fprintf(fp, "veclen = 1\n");
   fprintf(fp, "data = float\n");
   fprintf(fp, "field = irregular\n");
   fprintf(fp, "coord 1 file=%s  filetype=ascii offset=0 stride=4\n", tempstr);
   fprintf(fp, "coord 2 file=%s  filetype=ascii offset=1 stride=4\n", tempstr);
   fprintf(fp, "coord 3 file=%s  filetype=ascii offset=2 stride=4\n", tempstr);
   fprintf(fp, "variable 1 file=%s filetype=ascii offset=3 stride=4 skip=0\n",tempstr);   
   fclose(fp);

}

void conv_to_byte(float xmin, float xmax, float *aspratio, int ndata,float *xarray, unsigned char *uarray, int *ifirst)
{
int i;

  if(*ifirst == 0) { 
	printf("xmax = %f, xmin= %f, ndata = %d\n", xmax, xmin, ndata);
	*aspratio = 255./(float)(xmax-xmin);
	*ifirst = 1;
  }
  for(i=1;i<=ndata;i++){
   	uarray[i]= (unsigned char)((xarray[i]-xmin) * (*aspratio));
        printf("y = %u\n", uarray[i]);
  	if(uarray[i]>255) uarray[i]=255;
  	if(uarray[i]<0) uarray[i]=0;
  }	
}

unsigned char byteval(float xmin, float xmax, float *aspratio, float x, int *ifirst, int choice)
{
int i;
float temp;
unsigned char y;

  if(*ifirst == 0) { 
	printf("xmax = %f, xmin= %f\n", xmax, xmin);
	*aspratio = 255./(xmax-xmin);
	*ifirst = 1;
  }
  if(choice==1){ /* velocity models */
	temp = xmax -x;
  }else{
	temp = x-xmin;
  }
  y = (unsigned char)(temp * (*aspratio));
  if(y > 255) y = 255;
  if(y < 0) y = 0;
  return y;	
}

void edit_coastline(char *infile, float dep)
{ /* to change the coastline depth */
 FILE *fp;
 int i, j;
 float lat[10000], lon[10000], depth, value[10000];

 fp = fopen(infile, "r");
 i=0;
 while(fscanf(fp, "%f %f %f %f\n",&lon[i],&lat[i],&depth,&value[i]) != EOF){
	i++;
 }
 fclose(fp);
 fp = fopen(infile,"w");
 for(j=0;j<i;j++){
	fprintf(fp, "%f %f %f %f\n",lon[j],lat[j],dep,value[j]);
 }
} 



void write_layer(int fd, float *layer, int numhnode) 
{ /* writes a layer of model values into a binary file for 3D model conversion */
   int n;
   printf("numhnode = %d, layer[0] = %f\n", numhnode, layer[0]);
   n = write(fd, layer, numhnode*sizeof(float));
   printf("number of bytes written = %d\n", n);
}

void write_modelinfo_c(char *filename, int *fd, float *theta, float *phi, float *radlist, int numhnode, int numrnode) 
{ /* this program outputs a binary file containing a 3D model
   * values as layers of values.  Called from fortran programs
   */ 
   int i, j, n;
   float theta_phi[2];

   *fd = creat(filename, 0666);
   printf("*fd = %d\n", *fd);
   printf("numhnode = %d, numrnode = %d\n", numhnode, numrnode);
   n = write(*fd, &numhnode, sizeof(int));
   printf("number of bytes written = %d\n", n);
   n = write(*fd, &numrnode, sizeof(int));
   printf("number of bytes written = %d\n", n);
   for(i=0; i<numhnode; i++){
	theta_phi[0] = theta[i];
	theta_phi[1] = phi[i];
	write(*fd, theta_phi, sizeof(theta_phi));
   }
   write(*fd, radlist, numrnode*sizeof(float));
}


void calc_1d_prem(float DEPTH, int ichoice, float *ALPHA, float *BETA, float *RHO, 
			float *QKP, float *QMU)
{	
/*  ichoice = 1 Isotropic PREM, 2 Anisotropic PREM SV, 3. Anisotropic PREM sh */
	float X;
	float radius=6371.;
	float temp;


	X = (6371.-DEPTH)/radius;
	if(DEPTH<0) {
		printf("depth less than 0, quit!!!\n");
		exit(-1);
	}
	if((DEPTH <= 3.0))  {
		*ALPHA = 1.45;
		*BETA = 0.0;
		*RHO = 1.02;
		*QMU = 9999999.;
		*QKP = 57823.;

	}
	if(DEPTH>3. && DEPTH<=15.) {
		*ALPHA = 5.800;
		*BETA = 3.200;
		*RHO = 2.600;
		*QMU = 600.;
		*QKP = 57823.;
	}
	if(DEPTH>15.&& DEPTH<=24.3711) {
		*ALPHA = 6.800;
		*BETA = 3.900;
		*RHO = 2.900;
		*QMU = 600.;
		*QKP = 57823.0;
	}
	if(DEPTH>24.4 && DEPTH <= 80.0) {
		if(ichoice==1) {
	   		*ALPHA=4.1875+3.9382*X;
	   		*BETA = 2.1519+2.3481*X;
		}
		if(ichoice==2) {
	    		*ALPHA =0.8317+7.2180*X;
	    		*BETA = 5.8582-1.4678*X;
		}
		if(ichoice==3) {
	    		*ALPHA = 3.5908+4.6172*X;
	    		*BETA  = -1.0839+5.7176*X;
		}
		*RHO = 2.691+0.6924*X;
		*QMU = 600.0;
		*QKP = 57823.0;
	}
	if(DEPTH>80. && DEPTH<=220.) {
	 	if(ichoice==1) {
	       		*ALPHA=4.1875+3.9382*X;
	       		*BETA = 2.1519+2.3481*X;
	 	}
		if(ichoice==2) {
	    		*ALPHA =0.8317+7.2180*X;
	    		*BETA = 5.8582-1.4678*X;
		}
		if(ichoice==3) {	 
	       		*ALPHA=3.5908+4.6172*X;
	       		*BETA = -1.0839+5.7176*X;
		}
	 	*RHO = 2.691+0.6924*X;	      
	 	*QMU = 80.0;
	 	*QKP = 57823.0;
	}
	if(DEPTH>220. && DEPTH<=400.) {
		*ALPHA = 20.3926-12.2569*(radius-DEPTH)/radius;
		*BETA =  8.9496-4.4597*(radius-DEPTH)/radius;
		*RHO = 7.1089-3.8045*(radius-DEPTH)/radius;
		*QMU = 143.0;
		*QKP = 57823.0;

	}
	if(DEPTH>400. && DEPTH<=600.) {
		*ALPHA = 39.7027-32.6166*(radius-DEPTH)/radius;
		*BETA = 22.3512-18.5856*(radius-DEPTH)/radius;
		*RHO = 11.2494-8.0298*(radius-DEPTH)/radius;
		*QMU = 143.0;
		*QKP = 57823.0; 

	}
	if(DEPTH>600. && DEPTH<=670.) {
		*ALPHA = 19.0957-9.8672*(radius-DEPTH)/radius;
		*BETA = 9.9839-4.9324*(radius-DEPTH)/radius;
		*RHO = 5.3197-1.4836*(radius-DEPTH)/radius;
		*QMU = 143.;
		*QKP = 57823.;

	}
	if(DEPTH>670. && DEPTH<=771.) {
		*ALPHA = 29.2766-23.6027*X+5.5242*X*X-2.5514*X*X*X;
		*BETA = 22.3459-17.2473*X-2.0834*X*X+0.9783*X*X*X;
		*RHO = 7.9565-6.4761*X+5.5283*X*X-3.0807*X*X*X;
		*QMU = 312.;
		*QKP = 57823;

	}
	if(DEPTH>771. && DEPTH<=2741.) {
		*ALPHA = 24.9520-40.4673*X+51.4832*X*X-26.6419*X*X*X;
		*BETA = 11.1671-13.7818*X+17.4575*X*X-9.2777*X*X*X;
		*RHO = 7.9565-6.4761*X+5.5283*X*X-3.0807*X*X*X;
		*QMU = 312.0;
		*QKP = 57823.0;

	}
/*      the above calculates P,S, rho values for 2741km deep or less. */
	temp = ((*BETA)* (*BETA))/((*ALPHA)* (*ALPHA));
	*QKP = (1-4*temp/3)/(*QKP) + 4*temp/(3* (*QMU));
	*QKP=1/(*QKP);
	return;
}


