#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
/*
 * geo-constants
 */
#define RADIUS   6371.0              /* radius of the earth          */
#define RMOHO    6346.6              /* radius of the moho           */
#define R670     5701.0              /* radius of the 670 discon     */
#define RCMB     3479.96             /* radius of the CMB            */
#define RICB     1221.49072          /* radius of inner core boundary*/
#define LAYER    222                 /* # of layers in PREM model    */
/*
 * math-constants
 */
#define PI          3.1415926535897932
#define HPI		  1.5707963267948966
#define RAD         0.017453292519943   /* (pi/180.0)                */
#define FALSE       0
#define TRUE        1
#define MAXSTR      132
#define y00         0.28209479177388    /*  sqrt(1.0/fpi)            */
#define y10         0.48860251190292    /*  sqrt(3.0/fpi)            */
/*
 * model constants
 */
#define RADFUNONE		0		/* no radial functions */
#define LEGENDRE		1
#define CHEBYSHEV		2
#define SHELL		3
#define BSPLINE		4
#define OTHERFUNC		5

#define SURFACE     0
#define UPPERMANTLE 1
#define WHOLEMANTLE 2
#define LOWERMANTLE 3
#define UPPERLOWERM 4
#define WHOLEEARTH  5
#define REGIONAL    6
#define INNERCORE   7
#define OUTERCORE   8
#define MOHOSURF    9

#define DVFORMAT    1
#define DVOVFORMAT  2
#define DD          3         /* density perturbation */
#define PREMFORMAT  4
#define DT          5
#define DH          6
#define DV2         7
#define DEM         8         /* elastic modulus */
#define OTHERFT     9

#define PMODEL      1
#define SMODEL      2
#define DMODEL      3
#define PREMODEL    4
#define ANIDTM      5
#define UNKNOWN     6

#define OPENED	0
#define SKELETAL	1
#define STARRED	2
#define FILLED	3
#define POLYGON	4

#define CALPOWER 1
#define CALRMS	 2
#define CALRMS_L 3

typedef enum{PREMDL,M84A,M84C,L0256,V3,SH425,SH8WM13,SH8U4L8,SH12WM13,
             ULMODEL,WMMODEL,SURCOM,MDLSH,ZT36,MB92,INOUE,SAF,
             GRDM,MRD,ICANI,CRUSTMD,GRAND,WMMK,ICHET,SPHBSPL2,SPHBSPL3,BSPL3,BSPSPLT,
             UCSD} ModIndex;
typedef enum{HRVFMT=1,HRVPUBFMT,L0256FMT,CMPFMT,ZTFMT,CSTFMT,
             POWER,SCALEP,GEOMAPFMT,CRUST} FileFormat;
typedef enum{P=1,S} WaveType;

#define COAST       1
#define BOUNDARY    2

#define COASTSIZE     10263       /* # of caostline data points      */
#define BOUNDARYSIZE   7725       /* # of plate boundary data points */

#define PERMS 0666    /* RW for owner, group, others */

#define PMODE  0644

void DepthRange(float rr[], int *n, float *dmin, float *max);
void DepthRange_reverse(float rr[], int *n, float *dmin, float *max);
void output_3dfield(char *outfl, int ndim, int dim1, int dim2, int dim3, 
		   int nspace, int veclen, char *data,
	char *field, int nvar, char *filetype, int nskip);
void GetGrid(float *xlat, float *xlon, float *latmin, float *latmax, int *nlat, float *lonmin, float *lonmax, int *nlon,int *n);
void conv_to_byte(float xmin,float xmax,float *aspratio,int ndata,float *xarray, unsigned char *uarray, int *ifirst);
unsigned char byteval(float xmin, float xmax, float *aspratio, float x, int *ifirst, int choice);
void out_3dlimit(char *outfl, float xmin, float xmax, float ymin, float ymax, float zmin, float zmax);
void edit_depth(char *infile, float dep);
void write_modelinfo_c(char *filename, int *fd, float *theta, float *phi, float *radlist, int numhnode, int numrnode);
void write_layer(int fd, float *layer, int numhnode);
