c this is a fortran library routine for model evaluations and
c plotting, Jeff Gu, 1999.

c  The subroutine rot_euler rotates coordinate frames based on the Euler
c  angles alpha, beta and gamma (see Edmonds page 7 and page 53).
c  The input theta and phi are the coordinates in the geographic
c  coordinate system. The angle zeta is measured counterclockwise
c  from due east. The returned values of theta, phi, and zeta are
c  in the equatorial coordinate system.
c  The returned value of theta is between 0 and pi, and the returned
c  value of phi is between 0 and 2*pi.
c  The inverse operation, i.e. from the rotated (equatorial)
c  frame back to the original (geographic) frame is performed
c  by replacing alpha by -gamma, beta by -beta, and gamma by -alpha
c  in the call to the subroutine rot_euler.

c The following are called by the rot_euler subroutine
	function icheck_if_even(i)
	n=(i-2*((i)/2))
	if(n.eq.0) icheck_if_even=1
	else icheck_if_even=0
	return
	end

	function icheck_if_odd(i)
	n=(i-2*((i)/2))
	if(n.eq.0) icheck_if_odd=1
	else icheck_if_odd=0
	return
	end

	subroutine collon2euler(scol,slon,rcol,rlon,delta,alpha,beta,gamma)
	real scol, slon, rcol, rlon, delta
	real alpha, beta, gamma
	real az,sina,cosa,eta,mu,ep
	real tiny
 	parameter(PI=3.141592653579)
 	parameter(PI_2=PI/2.0)	
	tiny=1e-6
 	
	print*, 'scol, slon, rcol, rlon', scol, slon, rcol, rlon
	delta=acos(cos(scol)*cos(rcol)+sin(scol)*sin(rcol)*cos(rlon-slon))
	sina=sin(rlon-slon)*sin(rcol)/sin(delta)
	cosa=(cos(rcol)-cos(scol)*cos(delta))/(sin(scol)*sin(delta))
  	az=atan2(sina,cosa)
  	sina=sin(az)
	cosa=cos(az)
  	if(cosa.eq.0) then
      		if(cos(scol).gt.0) then
			eta=PI_2
      		else 
			eta=-PI_2
		endif
 	else
      		eta=atan(cos(scol)/(cosa*sin(scol)))
	endif
  	if(eta.lt.0.0) eta=eta+PI
  	gamma=PI_2+eta
  	mu=atan2(sina*sin(scol),cos(scol)/sin(eta))
  	beta=PI_2-mu
  	ep=atan2(sina*sin(eta),cos(eta)/sin(scol))
  	alpha=-PI_2+slon-ep
	return
	end

	subroutine reduce(pth, pph)
c force theta between 0 and pi, and phi between 0 and 2*pi
 	parameter(PI=3.141592653579)
 	parameter(PI_2=PI/2.0)
	real pth, pph	
	real th,ph,PI2

	PI2=PI*2
  	th=pth
	ph=pph
  	i=abs(ifix(ph/PI2))
	if(ph.lt.0.0) then
		ph=ph+(i+1)*PI2
  	else
		if(ph.gt.PI2) ph=ph-i*PI2
	endif
  	pph=ph
  	if(th.lt.0.0.or.th.gt.PI) then
    		i=ifix(th/PI)
    		if(th.gt.0.0) then
      			if(icheck_if_odd(i).eq.1) then
        			th=(i+1)*PI-th
				if(ph.lt.PI) then
					ph=ph+PI
				else
					ph=ph-PI
				endif
      			else 
				th=th-i*PI
			endif
    		else
      			if(icheck_if_even(i).eq.1) then
        			th=-th+i*PI
				if(ph.lt.PI) then
					ph=ph+PI
				else
					ph=ph-PI
				endif
      			else
     		 		th=th-i*PI
			endif
  		endif
    		pth=th
		pph=ph
  	endif
	return
	end


	subroutine rot_euler(alpha,beta,gamma,theta,phi,zeta)
	real alpha, beta, gamma,  theta, phi, zeta
  	real r_hat(0:3),rp_hat(0:3),k_hat(0:3),kp_hat(0:3),rotation(0:3,0:3)
  	real*8 sint,cost,sinp,cosp,sina,cosa,sinb,cosb,sing,cosg,sinz,cosz

  	sint=sin(theta)
	cost=cos(theta)
  	sinp=sin(phi)   
	cosp=cos(phi)
  	sina=sin(alpha)
	cosa=cos(alpha)
  	sinb=sin(beta)
	cosb=cos(beta)
  	sing=sin(gamma)
  	cosg=cos(gamma)
  	sinz=sin(zeta)
	cosz=cos(zeta)
	r_hat(0)=sint*cosp
	r_hat(1)=sint*sinp
	r_hat(2)=cost
	k_hat(0)=-(sinz*cost*cosp+cosz*sinp)
  	k_hat(1)=cosz*cosp-sinz*cost*sinp
  	k_hat(2)=sinz*sint
  	rotation(0,0)=cosg*cosb*cosa-sing*sina
  	rotation(0,1)=cosg*cosb*sina+sing*cosa
  	rotation(0,2)=-cosg*sinb
  	rotation(1,0)=-sing*cosb*cosa-cosg*sina
  	rotation(1,1)=-sing*cosb*sina+cosg*cosa
  	rotation(1,2)=sing*sinb
  	rotation(2,0)=sinb*cosa
  	rotation(2,1)=sinb*sina
  	rotation(2,2)=cosb
  	do i=0, 2
    		rp_hat(i)=0.0
    		kp_hat(i)=0.0
    		do j=0, 2
      			rp_hat(i)=rp_hat(i)+rotation(i,j)*r_hat(j)
      			kp_hat(i)=kp_hat(i)+rotation(i,j)*k_hat(j)
    		enddo
  	enddo
  	theta=atan2(sqrt(rp_hat(0)*rp_hat(0)+rp_hat(1)*rp_hat(1)),rp_hat(2))
  	phi=atan2(rp_hat(1),rp_hat(0))
  	call reduce(theta,phi)
  	sint=sin(theta)
	cost=cos(theta)
  	sinp=sin(phi)
	cosp=cos(phi)
  	sinz=-(kp_hat(0)*cost*cosp+kp_hat(1)*cost*sinp-kp_hat(2)*sint)
  	cosz=-kp_hat(0)*sinp+kp_hat(1)*cosp
	zeta=atan2(sinz,cosz)
	return
	end

	subroutine slicer_gcpath(numpt,ddelta,xlat,xlon)
c
c allows user to input an eq-station pair and return great-circle path
c
	integer numpt
	real ddelta, xlat(*), xlon(*)
  	real  PPI
  	real  RRAD
  	real  lat0, lon0, lat1, lon1, delta
  	real  theta, phi, gamma, zeta, ddel
  	real  alpha, beta
  	real  scol, rcol

  	PPI=3.1415926535897932
  	RRAD=0.017453292519943
  	write(*, "('enter beginning lat, lon:')")
  	read(*,*) lat1, lon1
  	write(*, "('enter ending lat, lon:')")
  	read(*,*) lat0, lon0
  	lat0=lat0*RRAD
  	lon0=lon0*RRAD
  	lat1=lat1*RRAD
  	lon1=lon1*RRAD
  	scol=PPI/2.0-lat0
  	rcol=PPI/2.0-lat1
	if(lon0.lt.0.0) lon0=lon0+PPI*2.0
	if(lon1.lt.0.0) lon1=lon1+PPI*2.0
  	call collon2euler(scol,lon0,rcol,lon1,delta,alpha,beta,gamma)
  	write(*,*) 'delta,alpha,beta,gamma', delta, alpha,beta,gamma
  	write(*, "('enter delta increment [in degrees]:')")
  	read(*,*) ddel
	ddelta=ddel
  	ddel=ddel*RRAD
  	numpt=ifix(delta/ddel+1)
  	write(*,*) 'total number of points:', numpt
  	theta=scol
  	phi=lat0
  	do i=1, numpt
		call rot_euler(alpha, beta,gamma,theta,phi,zeta)
		theta=PPI/2.0
		phi=real(i-1)*ddel
     		call rot_euler(-gamma, -beta, -alpha, theta, phi, zeta)
     		xlat(i) = (PPI/2-theta)/RRAD
     		phi = phi/RRAD
     		if(phi.ge.180.0) then
        		xlon(i) = phi-360.0
     		else
       			xlon(i) = phi
     		endif
	enddo
	return
	end

	subroutine select_neargcpath(numpt, xlat, xlon, delta, ngood, 
     c			indexgood, xoutarray, iopt)
c
c designed to use in conjunction with plotcmts.f
c allows user to input an eq-station pair and return great-circle path
c
	integer numpt, ngood
	real  xlat(1), xlon(1), xoutarray(1)
	integer  indexgood(1), iopt
  	real  PPI, HPPI, deltamax
  	real  RRAD, delta
  	real  lat0, lon0, lat1, lon1
  	real  theta, phi, gamma, zeta, ddel
  	real  alpha, beta
  	real  scol, rcol
	
  	PPI = 3.1415926535897932
  	HPPI = PPI/2.0
  	RRAD=0.017453292519943
  	write(*, "('enter beginning lat, lon of gc path:')")
  	read(*,*) lat1, lon1
  	write(*, "('enter ending lat, lon of gcpath:')")
  	read(*,*) lat0, lon0
  	lat0=lat0*RRAD
  	lon0=lon0*RRAD
  	lat1=lat1*RRAD
  	lon1=lon1*RRAD
  	scol=HPPI-lat0
  	rcol=HPPI-lat1
  	call collon2euler(scol,lon0,rcol,lon1,delta,alpha,beta,gamma)
  	write(*,*) 'delta,alpha,beta,gamma', delta, alpha,beta,gamma
	write(*,*)
  	write(*, "('Half-width of the band along great circle path (deg):')")
	read(*,*) deltamax
	deltamax = deltamax * RRAD
	j = 1
	ngood = 0
  	do i=1, numpt
		theta = (90.0 - xlat(i)) * RRAD
		phi = xlon(i)
		if(phi.lt.0.0) phi = phi + 360.0
		phi = phi * RRAD
		call rot_euler(alpha, beta,gamma,theta,phi,zeta)
c		print*, phi-HPPI, deltamax
		if(iopt.eq.1) then
c  minor arc only
		    if((abs(theta-HPPI).le.deltamax).and.
     c			(phi.ge.0.0).and.(phi.le.delta)) then
			indexgood(j) = i
			xoutarray(j) = delta/RRAD - phi/RRAD
			j = j + 1
		    endif
		else
c full great circle
		    if(abs(theta-HPPI).le.deltamax) then
				indexgood(j) = i
				xoutarray(j) = delta/RRAD - phi/RRAD
				j = j + 1
		    endif
		endif 
	enddo
	print*, 'selected events = ', j - 1
	ngood = j - 1
	delta = delta/RRAD
	return
	end
	
	subroutine slicer_by_pole(numpt, ddelta, xlat, xlon)
c this subrouine will compute a global cross-section path
c from a given pole location.
	integer numpt
	real xlat(1), xlon(1)
 	parameter(PI=3.141592653579)
 	parameter(PRAD=0.017453292519943)
	print*, 'pole of the great circle [lat, lon]'
	read(*,*) pcolat, plon
	print*, 'delta increment [in degrees]:'
	read(*,*) ddel
	numpt=ifix(360.0/ddel)
	print*, 'total number of points = ', numpt
	ddelta=ddel
	ddel=ddel*PRAD
	pcolat=90.0-pcolat
	pcolat=pcolat*PRAD
	plon=plon*PRAD
c find a number of equidistant points on a great circle defined by
c its pole.
c The formulae employed here can be found by means of a double rotation
c from the reference frame whose pole is the geographical north to
c the one whose pole is the pole of the great circle.
	sf=sin(plon)
	cf=cos(plon)
	ctsf=cos(pcolat)*sf
	ctcf=cos(pcolat)*cf
	st=sin(pcolat)
	PI_2=PI*0.5
	do i=1, numpt
c--by summing 90. to i*rincr in the next line I set the azimuth to 90.,
c--i.e. in the output file the 0-point on the great circle corresponds
c--to longitude 90 in the rotated system (the system whose pole is the
c--pole of the great circle).
           delta=PI_2+(i-1)*ddel
           theta=abs(acos(-cos(delta)*st))
           sphi=(ctsf*cos(delta)+cf*sin(delta))/sin(theta)
           cphi=(ctcf*cos(delta)-sf*sin(delta))/sin(theta)
           phi=atan2(sphi,cphi)
           xlat(i)=90.0-theta/PRAD
	   if(xlat(i).le.-90.0) xlat(i)=-89.999999
	   if(xlat(i).ge.90.0) xlat(i)=89.999999
           xlon(i)=phi/PRAD
	   if(xlon(i).le.-180.0) xlon(i)=-179.99999
	   if(xlon(i).ge.180.0) xlon(i)=179.99999
	enddo
	return
	end


	subroutine GetGrid(xxlat, xxlon,dlat,dlon,xlatmin,xlatmax,xlonmin,xlonmax,ndata)
	real xxlat(1), xxlon(1)
	real dlat,dlon,xlatmin,xlatmax
	real xlonmin, xlonmax
	real xlon, xlat, xlon1, xlat1
	integer ndata, i, j, k, nlat, nlon
	character*128 gridfile
	
	print*, 'surface point:'
	print*, '----------------------------------'
	print*, '1. enter by keyboard'
	print*, '2. enter from a file'
	print*, '3. enter a range and increment'
	print*, 'enter:'
	read(*,*) optdata
	if(optdata.eq.1) then
		print*, 'enter longitude and latidude:'
		read(*,*) xlon, xlat
		xxlat(1)=xlat
		xxlon(1)=xlon
		ndata=1
	else if(optdata.eq.2) then
		print*, 'enter longitude-latitude grid file:'
		read(*,"(a)") gridfile
		open(333, file=gridfile, type='old')
		i=1
110		read(333, *, end=120) xlon, xlat
		xxlat(i)=xlat
		xxlon(i)=xlon
		if(i.gt.100000) stop 'too many points!'
		i=i+1
		goto 110
120		ndata=i-1
    		print*, 'total surface node points = ', ndata
		close(333)
	else if(optdata.eq.3) then
		print*, 'latitude range [min, max, #points]:'
		read(*,*) xlatmin, xlatmax, nlat
		print*, 'colongitude range [min, max, #points]:'
		read(*,*) xlonmin, xlonmax, nlon
		k=1
		xlat1=xlatmin
		dlat=(xlatmax-xlatmin)/(nlat-1)
		dlon=(xlonmax-xlonmin)/(nlon-1)
		do i=1, nlat
			xlon1=xlonmin
			do j=1, nlon
				xxlon(k)=xlon1
				xxlat(k)=xlat1
				k=k+1
				xlon1=xlon1+dlon
			enddo
			xlat1=xlat1+dlat
		enddo
		ndata=k-1
    		print*, 'total surface node points = ', ndata
	else
		stop 'input option not reasonable!!'
	endif
	return
	end

      subroutine delazc(eplat,eplong,stlat,stlong,delta,azep,azst)
      data geoco,hpi,twopi,rad,reprad/.993277,1.5707963,
     16.2831853,.0174533,57.29578/
      tan(x)=sin(x)/cos(x)
      arcos(x)=atan2(sqrt(1.-x*x),x)
      if(abs(eplat).ne.90.0)then
          el=atan(geoco*tan(eplat*rad))
      else
          el=eplat*rad
      endif
      el=hpi-el
      if(abs(stlat).ne.90.0)then
         stl=atan(geoco*tan(stlat*rad))
      else
         stl=stlat*rad
      endif
      stl=hpi-stl
      elon=eplong*rad
      slon=stlong*rad
      as=cos(stl)
      bs=sin(stl)
      cs=cos(slon)
      ds=sin(slon)
      a=cos(el)
      b=sin(el)
      c=cos(elon)
      d=sin(elon)
      cdel=a*as+b*bs*(c*cs+d*ds)
      if(abs(cdel).gt.1.) cdel=sign(1.,cdel)
      delt=arcos(cdel)
      delta=delt*reprad
      sdel=sin(delt)
      caze=(as-a*cdel)/(sdel*b)
      if(abs(caze).gt.1.) caze=sign(1.,caze)
      aze=arcos(caze)
      if(bs.gt.0.) cazs=(a-as*cdel)/(bs*sdel)
      if(bs.eq.0.) cazs=sign(1.,cazs)
      if(abs(cazs).gt.1.) cazs=sign(1.,cazs)
      azs=arcos(cazs)
      dif=ds*c-cs*d
      if(dif.lt.0.) aze=twopi-aze
      azep=reprad*aze
      if(dif.gt.0.) azs=twopi-azs
      azst=reprad*azs
      return
      end

	subroutine initialize1_int(iarr1,npt,ival)
	integer iarr1(1)
	integer npt, ival
c
c  intialize 1 integer array at a time for N-points
c
	do i=1, npt
		iarr1(i)=ival
	enddo
	return
	end

	subroutine initialize2_int(iarr1,iarr2,npt,ival)
	integer iarr1(1),iarr2(1)
	integer npt, ival
c
c  intialize 1 integer array at a time for N-points
c
	do i=1, npt
		iarr1(i)=ival
		iarr2(i)=ival
	enddo
	return
	end

	subroutine initialize3_int(iarr1,iarr2,iarr3,npt,ival)
	integer iarr1(1),iarr2(1),iarr3(1)
	integer npt, ival
c
c  intialize 1 integer array at a time for N-points
c
	do i=1, npt
		iarr1(i)=ival
		iarr2(i)=ival
		iarr3(i)=ival
	enddo
	return
	end

	subroutine initialize1(arr1,npt,val)
	dimension arr1(1)
	real val
	integer npt
	
c
c  intialize 1 array for N-points
c
c real type
	do i=1, npt
		arr1(i)=val
	enddo
	return
	end

      function ff(x, nf)
c
c horizontal B-spline function
c x = distance in arc radians
c nf = frequency of B-splines, 6 = 362 nodes
c
      real x
      real rim, h, rad, c72
      integer  nf
      data conv/6371./
      
      pi=3.1415926536
      rad=180.0/pi
      c72=cos(72./rad)
      base=acos(c72/(1.-c72))
      h=base/float(nf)*1.09
      rim = 2.0*h

      ff=0.
      r=2.*x/rim
      if(r.gt.2.) return
      dif=r-1.
      if(dif.le.0.) ff=(0.75*r-1.5)*(r**2)+1.
      if(dif.gt.0) ff=((-0.25*dif+0.75)*dif-0.75)*dif+0.25
      return
      end

      function spl(x, h)
c
c horizontal B-spline function with arbitray bucket length
c x = distance to center
c h = bell size
c
      real x
      real rim, h, rad, c72
      
      rim = 2.0*h
      spl=0.
      r=2.*x/rim
      if(r.gt.2.) return
      dif=r-1.
      if(dif.le.0.) spl=(0.75*r-1.5)*(r**2)+1.
      if(dif.gt.0) spl=((-0.25*dif+0.75)*dif-0.75)*dif+0.25
      return
      end

	subroutine initialize2(arr1,arr2,npt,val)
	dimension arr1(1),arr2(1)
	real val
	integer npt	
c
c  intialize 2 array at a time for N-points
c
c real type
	do i=1, npt
		arr1(i)=val
		arr2(i)=val
	enddo
	return
	end

	subroutine initialize3(arr1,arr2,arr3,npt,val)
	dimension arr1(1),arr2(1),arr3(1)
	real val
	integer npt	
c
c  intialize 3 array at a time for N-points
c
c real type
	do i=1, npt
		arr1(i)=val
		arr2(i)=val
		arr3(i)=val
	enddo
	return
	end

c  below is numerical recipes sorting routine
      SUBROUTINE INDEXX(N,ARRIN,INDX)
      DIMENSION ARRIN(N),INDX(N)
      DO 11 J=1,N
        INDX(J)=J
11    CONTINUE
      L=N/2+1
      IR=N
10    CONTINUE
        IF(L.GT.1)THEN
          L=L-1
          INDXT=INDX(L)
          Q=ARRIN(INDXT)
        ELSE
          INDXT=INDX(IR)
          Q=ARRIN(INDXT)
          INDX(IR)=INDX(1)
          IR=IR-1
          IF(IR.EQ.1)THEN
            INDX(1)=INDXT
            RETURN
          ENDIF
        ENDIF
        I=L
        J=L+L
20      IF(J.LE.IR)THEN
          IF(J.LT.IR)THEN
            IF(ARRIN(INDX(J)).LT.ARRIN(INDX(J+1)))J=J+1
          ENDIF
          IF(Q.LT.ARRIN(INDX(J)))THEN
            INDX(I)=INDX(J)
            I=J
            J=J+J
          ELSE
            J=IR+1
          ENDIF
        GO TO 20
        ENDIF
        INDX(I)=INDXT
      GO TO 10
      END

	subroutine read_oceancrust(ifl,filenm,th,vp,vs,rho,nl)
c
c read oceanic crust in the format shown in Nishimura & Forsyth, 89.
c
	character filenm*(*)
	dimension  th(1), vp(1), vs(1), rho(1)
	integer nl, ifl, i
	character*128 str
	
	open(ifl,file=filenm(1:lnblnk(filenm)),type='old')
	read(ifl,"(a)") str
	print*, str
	read(ifl,"(a)") str
	nl = 1
	i=1
121	read(ifl, *, end=122) th(i), rho(i), vp(i), vs(i)
	print*, th(i), vp(i), vs(i), rho(i)
	i=i+1
	goto 121
122	nl=i-1
	print*, 'total number of layers = ', nl
	close(ifl)
	return
	end

	subroutine read_cara(ifl,dep,vp,vs,rho,nl)
c
c read Cara's P7 oceanic model
c
	character*128 str
	dimension dep(1), vp(1), vs(1), rho(1)
	integer nl, i
	
	open(ifl,file='/data/herbie/gu/dbase/nishimura_forsyth/cara_p7.mod',type='old')
	read(ifl,*) str
	read(ifl,*) str
	i=1
131	read(ifl,*,end=132) dep(i), rho(i), vp(i), vs(i)
	i=i+1
	goto 131
132	nl=i-1
	print*, 'total number of layers = ', nl
	close(ifl)
	return
	end

	subroutine read_oceanmantle(ifl,filenm,dep,vph,vpv,vsh,vsv,nl)
c
c read oceanic mantle in the format of Nishimura & Forsyth, 89.
c
	character filenm*(*)
	dimension  dep(1), vph(1), vpv(1), vsh(1), vsv(1)
	real	vph_std, vsv_std
	real	zeta, phi, eta
	real 	zeta_std, phi_std, eta_std
	integer nl, ifl, i
	character*7 str
	
	open(ifl,file=filenm(1:lnblnk(filenm)),type='old')
	read(ifl,*) str
	read(ifl,*) str
	i=1
131	read(ifl,*,end=132) dep(i),vph(i),vph_std,vsv(i),vsv_std,
     +			zeta,zeta_std,phi,phi_std,eta,eta_std
	vpv(i)=vph(i)*sqrt(zeta)
	vsh(i)=vsv(i)*sqrt(phi)
	i=i+1
	goto 131
132	nl=i-1
	print*, 'total number of layers = ', nl
	close(ifl)
	return
	end

	subroutine read_xu_wiens(ifl, filenm, th, vs, sigma, qb, nl)
c
c  read Xu & Wiens 1997 models
c
	character filenm*(*)
	dimension th(1), vs(1), sigma(1), qb(1)
	integer nl
	character*128 str
	
	open(ifl,file=filenm(1:lnblnk(filenm)),type='old')
	read(ifl, *) str
	i=1
133	read(ifl, *, end=134) th(i), vs(i), sigma(i), qb(i)
	i=i+1
	goto 133
134	nl=i-1
	close(ifl)
	return
	end

	subroutine read_sna_tna(ifl, filenm, dep, vp, vs, rho, nl)
c
c  read Grand & Helmberger's SNA and TNA models
c
	character filenm*(*)
	dimension dep(1), vp(1), vs(1), rho(1)
	real tempqa, tempqb
	integer nl
	character*128 str
	
	open(ifl,file=filenm(1:lnblnk(filenm)),type='old')
	read(ifl, *) str
	read(ifl, *) nl
	do i=1, nl
		read(ifl, *) itemp, vp(i), vs(i), rho(i), dep(i), tempqa, tempqb
	enddo
	close(ifl)
	return
	end

	subroutine read_iaspei(ifl, dep, vp, vs, rho, nl, disc, ndisc)
c
c  read iaspei91 model (4 columns)
c
	dimension dep(1), vp(1), vs(1), rho(1), disc(1)
	integer nl, ndisc
	character*128 str
	
	open(ifl,file='/data/herbie/gu/dbase/model1D/iaspei140.dat',type='old')
	i=1
135	read(ifl, *, end=136) dep(i), vp(i), vs(i), rho(i)
	i=i+1
	goto 135
136	nl=i-1
	j=0
	do i=2, nl
		if(abs(dep(i)-dep(i-1)).lt.1.0) then
c  discontinuity found	
			j=j+1
			disc(j)=ifix(dep(i))
		endif
	enddo
	do i=1, j
		print*, 'discontinuity ', i, '   dep=', disc(i)
	enddo
	close(ifl)
	return
	end

	subroutine premmodel(depth,ichoice,alpha,beta,rho,
     %		qai,qbi,reg,nreg)
c
c	This subroutine calculates the values for the P wave
c 	and S wave velocity and density of the first
c	1000 km from the Earth surface to the center by using 
c	the PREM model, see Dziewonski and Anderson, 1981.
c	INPUT:
c		depth ---   depth to be evaluated, in km
c		ichoice ---  anisotropic option between 1 and 3
c			1 = Isotropic velocity
c			2 = anisotropic velocity SV or PV
c			3 = anisotropic velocity SH or PH
c	OUTPUT:
c		alpha --- compressional velocity
c		beta  --- shear velocity
c		rho   --- density
c		qai   -- quality factor for compression
c		qbi   -- quality factor for shear
c		reg  -- array containing discontinuity depths
c		nreg  -- number of discontinuities 
c
c
	real depth, alpha, beta, rho
	real x, temp, qmu, qkp, qai, qbi
	real reg(*)
	integer ichoice, nreg
c
	nreg = 13
	reg(1)=3.0
	reg(2)=15.0
	reg(3)=24.4
	reg(4)=80.0
	reg(5)=220.0
	reg(6)=400.0
	reg(7)=600.0
	reg(8)=670.0
	reg(9)=771.0
	reg(10)=2741.0
	reg(11)=2891.0
	reg(12)=5149.5
	reg(12)=6371.0
	X = (6371.0-DEPTH)/6371.0  ! X= r/a
	IF(DEPTH.LT.0.OR.DEPTH.GT.6371.) THEN
		print*, 'depth = ', depth
		stop 'Impossible for the earth, quit!'
	ENDIF
	if(ichoice.lt.1.or.ichoice.gt.3) then
		stop 'Anisotropic option not valid!!'
	endif
	IF((DEPTH.LE.3.0))  THEN
		ALPHA = 1.45
		BETA = 0.0
		RHO = 1.02
		QMU = 9999999
		QKP = 57823
	END IF
C
	IF(DEPTH.GT.3.AND.DEPTH.LE.15) THEN
		ALPHA = 5.800
		BETA = 3.200
		RHO = 2.600
		QMU = 600
		QKP = 57823
	END IF
C
	IF(DEPTH.GT.15.AND.DEPTH.LE.24.4) THEN
		ALPHA = 6.800
		BETA = 3.900
		RHO = 2.900
		QMU = 600
		QKP = 57823
	END IF
C
	IF(DEPTH.GT.24.4.AND.DEPTH.LE.80.0) THEN
	   if(ichoice.eq.1) then
	   	ALPHA=4.1875+3.9382*X
	   	BETA = 2.1519+2.3481*X
	   else if(ichoice.eq.2) then
	    	ALPHA =0.8317+7.2180*X
	    	BETA = 5.8582-1.4678*X
	   else if(ichoice.eq.3) then
	    	ALPHA = 3.5908+4.6172*X   ! THIS IS PH
	    	BETA  = -1.0839+5.7176*X  ! THIS IS SH
	   endif
	   RHO = 2.691+0.6924*X
	   QMU = 600 
	   QKP = 57823
	END IF
c
	IF(DEPTH.GT.80.AND.DEPTH.LE.220) THEN
	 if(ichoice.eq.1) then
	 	ALPHA=4.1875+3.9382*X
	 	BETA = 2.1519+2.3481*X
	 else if(ichoice.eq.2) then
	    	ALPHA =0.8317+7.2180*X
	 	BETA = 5.8582-1.4678*X
	 else if(ichoice.eq.3) then	 
	  	ALPHA=3.5908+4.6172*X
	 	BETA = -1.0839+5.7176*X
	 endif
	 RHO = 2.691+0.6924*X	      
	 QMU = 80
	 QKP = 57823
	END IF
C
	IF(DEPTH.GT.221.AND.DEPTH.LE.400) THEN
		ALPHA = 20.3926-12.2569*(6371.0-DEPTH)/6371.0
		BETA =  8.9496-4.4597*(6371.0-DEPTH)/6371.0
		RHO = 7.1089-3.8045*(6371.0-DEPTH)/6371.0
		QMU = 143
		QKP = 57823
	END IF
C
	IF(DEPTH.GT.400.AND.DEPTH.LE.600) THEN
		ALPHA = 39.7027-32.6166*(6371.0-DEPTH)/6371.0
		BETA = 22.3512-18.5856*(6371.0-DEPTH)/6371.0
		RHO = 11.2494-8.0298*(6371.0-DEPTH)/6371.0
		QMU = 143
		QKP = 57823 
	END IF
C
	IF(DEPTH.GT.600.AND.DEPTH.LE.670) THEN
		ALPHA = 19.0957-9.8672*(6371.0-DEPTH)/6371.0
		BETA = 9.9839-4.9324*(6371.0-DEPTH)/6371.0
		RHO = 5.3197-1.4836*(6371.0-DEPTH)/6371.0
		QMU = 143
		QKP = 57823
	END IF
C
	IF(DEPTH.GT.670.AND.DEPTH.LE.771) THEN
		ALPHA = 29.2766-23.6027*X+5.5242*X**2-2.5514*X**3
		BETA = 22.3459-17.2473*X-2.0834*X**2+0.9783*X**3
		RHO = 7.9565-6.4761*X+5.5283*X**2-3.0807*X**3
		QMU = 312
		QKP = 57823
	END IF
c
c lower mantle
c
	IF(DEPTH.GT.771.AND.DEPTH.LE.2741) THEN
		ALPHA = 24.9520-40.4673*X+51.4832*X**2-26.6419*X**3
		BETA = 11.1671-13.7818*X+17.4575*X**2-9.2777*X**3
		RHO = 7.9565-6.4761*X+5.5283*X**2-3.0807*X**3
		QMU = 312
		QKP = 57823
	END IF

	IF(DEPTH.GT.2741.AND.DEPTH.LE.2891) THEN
		ALPHA = 15.3891-5.3181*X+5.5242*X**2-2.5514*X**3
		BETA = 6.9254+1.4672*X-2.0834*X**2+0.9783*X**3
		RHO = 7.9565-6.4761*X+5.5283*X**2-3.0807*X**3
		QMU = 312
		QKP = 57823
	END IF
c
c outer core
c
	IF(DEPTH.GT.2891.AND.DEPTH.LE.5149.5) THEN
		ALPHA = 11.0487-4.0362*X+4.8023*X**2-13.5732*X**3
		BETA = 0.0
		RHO = 12.5815-1.2638*X-3.6426*X**2-5.5281*X**3
		QMU = 1.0e30
		QKP = 57823
	END IF
c
c inner core
c
	IF(DEPTH.GT.5149.5.AND.DEPTH.LE.6371.0) THEN
		ALPHA = 11.2622-6.3640*X**2
		BETA = 3.6678-4.4475*X**2
		RHO = 13.0885-8.8381*X**2
		QMU = 84.6
		QKP = 1327.7
	END IF
C      the above calculates P,S, rho values PREM.
c	TYPE*, 'THE VALUES FOR ALPHA, BETA, DENSITY, QAI and QBI ARE:'
c  The following conversions are taken from page 300 of Don's "Theory
c  of the Earth"
c
	QBI = QMU
	temp = (4.0*(beta/alpha)**2)/3.0
	QAI = (1.0-temp)/QKP + temp/QMU
	QAI = 1.0/QAI
c	TYPE*, ALPHA, BETA, RHO, QAI, QBI
1000	close(1)
	RETURN 
	END


      subroutine flatten(th,c,s,d,qa,qb,jo)
c  Earth flattening transformation.
c  This is modified from "curay".
c    For flattening transformation, goto Shearer (Page 49),
c	v_flat = (a/r) v_orig
c	depth_flat = -a ln(r/a)  (not used here, ???)
c	where a = 6371.0
c  Input:
c	th   ---  thickness of the layers
c	c    ---  avg compressional velocities for the layers
c       s    ---  avg shear velocities
c 	d    ---  density layers
c 	qa   ---  Q for compressional velocity
c 	qb   ---  Q for shear velocity
c	jo   ---  number of layers
c  Output:
c	th, c, s, d, qa, qb (modified arrays)
c
      parameter (nl=10000)
      integer jo
      dimension th(1),c(1),s(1),d(1),qa(1),qb(1)
      dimension depth(nl)
      real*4   q, q1
      depth(1)=th(1)/2.0
      do j = 2,jo
		depth(j)=depth(j-1)+(th(j)+th(j-1))/2.
      enddo
      do j = 1,jo
      	q = 6371.0 / (6371.0-depth(j))
      	c(j)=c(j)*q 
      	s(j)=s(j)*q
      	d(j)=d(j)*q
      	th(j)=th(j)*q
      enddo
      return
      end

      subroutine flatten_depth(dd,c,s,d,qa,qb,jo)
c  Earth flattening transformation.
c  This is modified from "curay".
c    For flattening transformation, goto Shearer (Page 49),
c	v_flat = (a/r) v_orig
c	depth_flat = -a ln(r/a)  (not used here, ???)
c	where a = 6371.0
c  Input:
c	dd   ---  depth of the layers
c	c    ---  avg compressional velocities for the layers
c       s    ---  avg shear velocities
c 	d    ---  density layers
c 	qa   ---  Q for compressional velocity
c 	qb   ---  Q for shear velocity
c	jo   ---  number of layers
c  Output:
c	dd, c, s, d, qa, qb (modified arrays)
c
      parameter (nl=10000)
      integer jo
      dimension dd(1),c(1),s(1),d(1),qa(1),qb(1)
      dimension depth(nl), th(nl)
      real*4   q, q1, dep
      th(1)=dd(1)
      do j = 2,jo
		th(j)=dd(j)-dd(j-1)
      enddo
      do j = 1,jo
      	q = 6371.0 / (6371.0-dd(j))
      	c(j)=c(j)*q 
      	s(j)=s(j)*q
      	d(j)=d(j)*q
      	th(j)=th(j)*q
      enddo
      dep=0.0
      do j=1, jo
	dep=dep+th(j)
	dd(j)=dep
      enddo
      return
      end

      subroutine unflatten(th,c,s,d,qa,qb,jo)
c  UNDO Earth flattening transformation.
c  This is modified from "curay".
c    For flattening transformation, goto Shearer (Page 49),
c	v_flat = (a/r) v_orig
c	depth_flat = -a ln(r/a)  (not used here, ???)
c	where a = 6371.0
c  Input:
c	th   ---  thickness of the layers
c	c    ---  avg compressional velocities for the layers
c       s    ---  avg shear velocities
c 	d    ---  density layers
c 	qa   ---  Q for compressional velocity
c 	qb   ---  Q for shear velocity
c	jo   ---  number of layers
c  Output:
c	th, c, s, d, qa, qb (modified arrays)
c
      parameter (nl=10000)
      integer jo
      dimension th(1),c(1),s(1),d(1),qa(1),qb(1)
      dimension depth(nl)
      real*4   q, q1
      depth(1)=th(1)/2.0
      do j = 2,jo
		depth(j)=depth(j-1)+(th(j)+th(j-1))/2.
      enddo
      do j = 1,jo
      	q = (6371.0-depth(j))/6371.0
      	c(j)=c(j)*q 
      	s(j)=s(j)*q
      	d(j)=d(j)*q
      	th(j)=th(j)*q
      enddo
      return
      end

      subroutine unflatten_depth(dd,c,s,d,qa,qb,jo)
c  Earth flattening transformation.
c  This is modified from "curay".
c    For flattening transformation, goto Shearer (Page 49),
c	v_flat = (a/r) v_orig
c	depth_flat = -a ln(r/a)  (not used here, ???)
c	where a = 6371.0
c  Input:
c	dd   ---  depth of the layers
c	c    ---  avg compressional velocities for the layers
c       s    ---  avg shear velocities
c 	d    ---  density layers
c 	qa   ---  Q for compressional velocity
c 	qb   ---  Q for shear velocity
c	jo   ---  number of layers
c  Output:
c	dd, c, s, d, qa, qb (modified arrays)
c
      parameter (nl=10000)
      integer jo
      dimension dd(1),c(1),s(1),d(1),qa(1),qb(1)
      dimension depth(nl), th(nl)
      real*4   q, q1, dep
      th(1)=dd(1)
      do j = 2,jo
		th(j)=dd(j)-dd(j-1)
      enddo
      do j = 1,jo
      	q = (6371.0-dd(j))/6371.0
      	c(j)=c(j)*q 
      	s(j)=s(j)*q
      	d(j)=d(j)*q
      	th(j)=th(j)*q
      enddo
      dep=0.0
      do j=1, jo
	dep=dep+th(j)
	dd(j)=dep
      enddo
      return
      end

      subroutine rdwr_reflmodel(th,al,be,rho,qal,qbe,nlyrs,ifl,
     %		ifl1,ifrd)
c
c read and write a reflectivity model
c ****PARAMETERS:
c	th  ---  []array of flattened thickness
c	al  ---  []array of   "  "    vp
c	be  ---  []array of   "  "    vs
c      rho  ---  []array of   "  "    density
c      qal  ---  []array of   "  "    Qp
c      qbe  ---  []array of   "  "    Qs
c    nlyrs  ---  number of total layers
c      ifl  ---- input model file identifier
c     ifl1  ---- output model file identifier
c     ifrd  ---- 1=read in a model file  2= generate a model file
c
c ****Other variables
c
c     pmin  ---  minimum ray slowness
c      plc  ---  slowness of lower crust
c      puc  ---  slowness of upper crust
c     delp  ---  increment in slowness
c     fmin  ---  minimum frequency
c     fmax  ---  maximum frequency
c     fny   ---  nyquist frequency
c     nft   ---  number of frequency points
c     nsou  ---  number of sources
c     nsta  ---- [] number of stations for each source
c   sdepth  ---- [] array of source depths for multiple sources
c   recdep  ---- [][] 2-D array of receiver depths for multiple receivers
c       rh  ---- [][] 2-D array of epicentral distance
c     azis  ---- [][] 2-D array of receiver azimuth
c      mxx  ---- [] cartesian moment tensor components for each source
c      mxy  ---- [] cartesian moment tensor components
c      mxz  ---- [] cartesian moment tensor components
c      myy  ---- [] cartesian moment tensor components
c      myz  ---- [] cartesian moment tensor components
c      mzz  ---- [] cartesian moment tensor components
c iperturb  ---- perturbation option
c	  1 =  Computing Partials for P wave velocity'
c	  2 =  Computing Partials for S wave velocity'
c	  3 =  Computing Partials for P wave velocity, S and Rho tied to P'
c	  4 =  Computing Partials for S wave velocity, P and Rho tied to S'
c
c
      parameter (maxlayer=500)
      parameter (maxsource=100)
      parameter (maxsta=100)
      parameter (FF=2049)
      parameter (RR=200)
      character*128 fname
      dimension th(1), al(1), be(1), rho(1), qal(1), qbe(1)
      dimension sdepth(maxsource),recdep(maxsta,maxsource) 
      dimension nsta(maxsource)
      dimension rh(maxsta,maxsource), azis(maxsta,maxsource)
      real 	deptop(maxlayer)
      real      mxx(maxsource),mxy(maxsource),mxz(maxsource)
      real 	myy(maxsource),myz(maxsource),mzz(maxsource)
      integer   nlyrs,nft,nsou,nsta,ifl,ifl1,ifrd

      
      
c
c  decide if read or write a model file for reflectivity
c
      if(ifrd.ne.1)  goto 5050
      print*, 'enter reflectivity input file...'
      read(*,"(a)") fname
      open(ifl,file=fname,type='old')
      print*, 'choose to output:'
      print*, '1.  Vp vs. depth'
      print*, '2.  Vs vs. depth'
      print*, '3.  rho vs. depth'
      print*, '4.  Qalpha vs. depth'
      print*, '5.  Qbeta vs. depth'
      print*, '6.  no print'
      read(*,*) ioutopt
      if(ioutopt.gt.6.or.ioutopt.lt.1) stop 'unreasonable choice'
      read(ifl,*) nlyrs
      print*, 'number of layers = ', nlyrs
      dep0=0.0
      dep1=0.0
      do i=1, nlyrs
      	 read(ifl,*) th(i),al(i),be(i),rho(i),qal(i),qbe(i)
	 write(*, 11110) th(i), al(i), be(i), rho(i), qal(i), qbe(i)
	 dep1=dep1+th(i)
	 if(ioutopt.eq.1) then
		write(ifl1,*) al(i),dep0
		write(ifl1,*) al(i),dep1
	 else if(ioutopt.eq.2) then
		write(ifl1,*) be(i),dep0
		write(ifl1,*) be(i),dep1
	 else if(ioutopt.eq.3) then
		write(ifl1,*) rho(i),dep0
		write(ifl1,*) rho(i),dep1
	 else if(ioutopt.eq.4) then
		write(ifl1,*) qal(i),dep0
		write(ifl1,*) qal(i),dep1
	 else if(ioutopt.eq.5) then
		write(ifl1,*) qbe(i),dep0
		write(ifl1,*) qbe(i),dep1
	 endif
	 dep0=dep1
      enddo
      do 6000 l = 1, nlyrs
         if ( l .eq. 1 ) then
	    deptop(1) = 0.0
	 else
	    deptop(l) = deptop(l-1) + th(l-1)
	 endif
	 if (  (th(l) .le. 0.) .and. (l .lt. nlyrs) ) then
	    write(iounit,*) ' Input Error:  layer ',l,' th <= 0 '
	    stop
	 endif
	 if (  al(l) .le. 0. ) then
	    write(iounit,*) ' Input Error:  layer ',l,' al0 <= 0 '
	    stop
	 endif
	 if (  be(l) .lt. 0. ) then
	    write(iounit,*) ' Input Error:  layer ',l,' be0 < 0 '
	    stop
	 endif
	 if (  rho(l) .le. 0. ) then
	    write(iounit,*) ' Input Error:  layer ',l,' rho <= 0 '
	    stop
	 endif
	 if (  qal(l) .le. 0. ) then
	    write(iounit,*) ' Input Error:  layer ',l,' qal <= 0 '
	    stop
	 endif
	 if (  qbe(l) .le. 0. ) then
	    write(iounit,*) ' Input Error:  layer ',l,' qbe <= 0 '
	    stop
	 endif
 6000 continue
      read(ifl,*) pmin,plc,puc,pmax,delp
      if ( pmin .ge. pmax ) then
	write(*,*) ' Input Error: pmin >= pmax : ', pmin, pmax
	stop
      endif
      if ( plc .ge. puc ) then
	write(*,*) ' Input Error: plc >= puc : ', plc, puc
	stop
      endif
      if ( ( plc .ge. pmax ) .or. ( puc .le. pmin ) ) then
	write(*,*) ' Input Error: plc >= pmax OR puc <= pmin : ', pmin, plc, puc, pmax
	stop
      endif
      if ( delp .le. 0.0 ) then
	write(*,*) ' Input Error: delp <= 0.0 : ', delp
	stop
      endif
      write(*,*) ' range of p (slowness) values: ', pmin,plc,puc,pmax
      read(ifl,*) fmin,fmax,fny,nft
      if ( fmin .ge. fmax ) then
	write(*,*) ' Input Error: fmin >= fmax : ', fmin, fmax
	stop
      endif
      if ( fmax .gt. fny ) then
	write(*,*) ' Input Error: fmax >= fny : ', fmax, fny
	stop
      endif
      if ( ( nft/2 + 1 ) .gt. FF ) then
	write(*,*) ' Input Error: nft/2 + 1 >= FF : ', (nft/2 + 1), FF
	stop
      endif
c
c     Compute slowness and frequency parameters
c
      pc1 = ( pmin + plc ) * 0.5
      pr1 = pi / ( plc - pmin )
      pc2 = ( pmax + puc ) * 0.5
      pr2 = pi / ( pmax - puc )
      dt = 1/(2.*fny)
      t = nft * dt
      delf = 2. * fny / nft
      nfpts = nft/2 + 1
      ifmin = fmin / delf
      ifmax = fmax / delf
      nffpts = ifmax - ifmin + 1
c
      write(*,*) ' '
      write(*,*) ' Frequency Domain and Time Domain parameters '
      write(*,*) ' fmin,fmax,fny,nft :' , fmin,fmax,fny,nft
      write(*,*) ' Sampling rate (dt) and total trace length (T) : ', dt, t
      write(*,*) ' Number of Frequencies values computed : ',nffpts
c
c     delp - delta p
c      delp = 1./(fmax * 2.*pfac * distmx)
c set delp = constant
      npmin = pmin / delp
      npmax = pmax / delp
      nppts = npmax - npmin + 1
c
      write(*,*) ' '
      write(*,*) ' Slowness integration parameters '
      write(*,*) ' range of p (slowness) values: ', pmin,plc,puc,pmax
      write(*,*) ' range of c (phase velocity) values: ', 1/pmax,1/puc,1/plc,1/pmin
      write(*,*) ' Number of slownesses and dp: ', nppts, delp
      
      read(ifl,*)nsou
      do isource=1,nsou
       read(ifl,*) nsta(isource)
       if ( nsta(isource) .gt. RR ) then
	  write(*,*) ' Input Error: nr > RR : ', nsta(isource), RR
	  stop
       endif
       do i=1,nsta(isource)
       		read(ifl,*) rh(i,isource),azis(i,isource)
       enddo
       read(ifl,*) sdepth(isource)
       print*, 'seismograms calculated at following distances (km)'
      write(*,*) ' '
      write(*,*) ' Sources and Receiver Depths '
      write(*,*) ' source depth :', sdepth(isource)
      do i=1,nsta(isource)
       read(ifl,*) recdep(i,isource)
      enddo
c---------------------------------------------------------------------------------
c     source representation follows Aki and Richards (1980) Box 4.4
c     and Kennett (1983) (4.59 - 4.60)
c     Note: if the sources are changed, calls to srcvec may have
c           to change to get proper order numbers and construction
c           of u0,w0,tn arrays must also change to reflect order
c
c     Use simple Mij terms, and use a postprocessor to recombine for
c         an arbitrary orientation Moment Tensor ala A&R Box 4.4
c


c
c     Mij General moment Tensor
c
        read(ifl,*) mxx(isource),mxy(isource),mxz(isource),
     *	myy(isource),myz(isource),mzz(isource)
      	write(*,*) ' '
	write(*,*) ' Calculating Moment Tensor: Mxx, Mxy, Mxz, Myy, Myz, Mzz '
	write(*,*) mxx(isource),mxy(isource),mxz(isource),myy(isource),myz(isource),mzz(isource)

	enddo
        read(ifl,*) iperturb
	if (( iperturb .lt. 1 ) .or. ( iperturb .gt. 4 )) then
	  write(*,*) ' Input Error: illegal iperturbation code: ', iperturb
	  write(*,*) ' Perturbation code must be between 1 and 4 '
	  stop
	endif
	if ( iperturb .eq. 1 ) then
	  write(*,*) ' Computing Partials for P wave velocity'
	  perta = 1.002
	  pertb = 1.00
	  pertr = 1.00
	endif
	if ( iperturb .eq. 2 ) then
	  write(*,*) ' Computing Partials for S wave velocity'
	  perta = 1.00
c	  pertb = 1.0005
	  pertb = 0.9995
	  pertr = 1.00
	endif
	if ( iperturb .eq. 3 ) then
	  write(*,*) ' Computing Partials for P wave velocity, S and Rho tied to P'
	  perta = 1.001
	  pertb = 1.001
	  pertr = 1.01
	endif
	if ( iperturb .eq. 4 ) then
	  write(*,*) ' Computing Partials for P wave velocity, P and Rho tied to S'
	  perta = 1.001
	  pertb = 1.001
	  pertr = 1.001
	endif
5050   close(ifl)
       if(ifrd.ne.1) then
c------------------------------------------------------------------------------------------
c generate reflectivity input file
c
c
c first print out all the layers
	write(ifl1, *) nlyrs+1 	! add 1 to output the 0 km thick half space
	do i=1, nlyrs
		write(ifl1, 11110) th(i),al(i),be(i),rho(i),qal(i),qbe(i)
	enddo
	write(ifl1, 11110) 0.0, al(nlyrs)+(al(nlyrs)-al(nlyrs-1)), be(nlyrs)+(be(nlyrs)-be(nlyrs-1)),
     +		rho(nlyrs)+(rho(nlyrs)-rho(nlyrs-1)), qal(nlyrs)+(qal(nlyrs)-qal(nlyrs-1)),
     +		qbe(nlyrs)+(qbe(nlyrs)-qbe(nlyrs-1))
	print*, 'slowness minimum, left_corner, right_corner, maximum, and increment:'
      	read(*,*) pmin,plc,puc,pmax,delp
      	if ( pmin .ge. pmax ) then
		write(*,*) ' Input Error: pmin >= pmax : ', pmin, pmax
		stop
      	endif
      	if ( plc .ge. puc ) then
		write(*,*) ' Input Error: plc >= puc : ', plc, puc
		stop
      	endif
      	if ( ( plc .ge. pmax ) .or. ( puc .le. pmin ) ) then
		write(*,*) ' Input Error: plc >= pmax OR puc <= pmin : ', pmin, plc, puc, pmax
		stop
      	endif
      	if ( delp .le. 0.0 ) then
		write(*,*) ' Input Error: delp <= 0.0 : ', delp
		stop
      	endif	
	write(ifl1, *) pmin,plc,puc,pmax,delp
      	write(*,*) ' range of p (slowness) values: ', pmin,plc,puc,pmax
	print*, ' '
	print*, 'enter frequency ranges: min, max, nyquist, number_of_freq:'
      	read(*,*) fmin,fmax,fny,nft
      	if ( fmin .ge. fmax ) then
		write(*,*) ' Input Error: fmin >= fmax : ', fmin, fmax
		stop
      	endif
      	if ( fmax .gt. fny ) then
		write(*,*) ' Input Error: fmax >= fny : ', fmax, fny
		stop
      	endif
      	if ( ( nft/2 + 1 ) .gt. FF ) then
		write(*,*) ' Input Error: nft/2 + 1 >= FF : ', (nft/2 + 1), FF
		stop
      	endif
	write(ifl1,*) fmin,fmax,fny,nft
      	t = nft * dt
      	delf = 2. * fny / nft
     	 nfpts = nft/2 + 1
      	ifmin = fmin / delf
      	ifmax = fmax / delf
      	nffpts = ifmax - ifmin + 1
c
      	write(*,*) ' '
      	write(*,*) ' Frequency Domain and Time Domain parameters '
      	write(*,*) ' fmin,fmax,fny,nft :' , fmin,fmax,fny,nft
      	write(*,*) ' Sampling rate (dt) and total trace length (T) : ', dt, t
      	write(*,*) ' Number of Frequencies values computed : ',nffpts
	print*, ' '
	print*, 'number of earthquake sources:'
      	read(*,*)nsou
	write(ifl1,*) nsou
      	do isource=1,nsou
		print*, 'number of stations for source', isource, ':'
       		read(*,*) nsta(isource)
       		if ( nsta(isource) .gt. RR ) then
	  		write(*,*) ' Input Error: nr > RR : ', nsta(isource), RR
	  		stop
       		endif
		write(ifl1,*) nsta(isource)
       		do i=1,nsta(isource)
			print*, 'distance and azimuth of station: ', i
       			read(*,*) rh(i,isource),azis(i,isource)
			write(ifl1,*) rh(i,isource),azis(i,isource)
      		enddo
      		print*, 'enter source depth of ', isource
      		read(*,*) sdepth(isource)
		write(ifl1,*) sdepth(isource)
      		print*, 'enter receiver depths: total=', nsta(isource)
      		do kk=1,nsta(isource)
       			read(*,*) recdep(kk,isource)
       			write(ifl1,*) recdep(kk,isource)
      		enddo
		print*, 'enter Harvard Moment Tensor Solution:'
		print*, 'Note: input should be Mrr, Mtt, Mpp, Mrt, Mrp, Mtp:'
		print*, '      automatically convert to Mxx, Mxy, Mxz, Myy, Myz, Mzz:'
		read(*,*) rrm,ttm,ppm,rtm,rpm,tpm
c
c conversion to cartesian
c
        	mxx(isource)=ttm
		mxy(isource)=-tpm
		mxz(isource)=rtm
     		myy(isource)=ppm
		myz(isource)=-rpm
		mzz(isource)=rrm
      		write(*,*) ' '
		write(*,*) ' Calculating Moment Tensor: Mxx, Mxy, Mxz, Myy, Myz, Mzz '
		write(*,*) mxx(isource),mxy(isource),mxz(isource),myy(isource),myz(isource),mzz(isource)
		write(ifl1,*) mxx(isource),mxy(isource),mxz(isource),myy(isource),myz(isource),mzz(isource)
	enddo
	print*, 'choose perturbations:'
	print*, '----------------------------------'
	print*, ' Computing Partials for P wave velocity'
	print*, ' Computing Partials for S wave velocity'
	print*, ' Computing Partials for P wave velocity, S and Rho tied to P'
	print*, ' Computing Partials for S wave velocity, P and Rho tied to S'
        read(*,*) iperturb
	if (( iperturb .lt. 1 ) .or. ( iperturb .gt. 4 )) then
	  write(*,*) ' Input Error: illegal iperturbation code: ', iperturb
	  write(*,*) ' Perturbation code must be between 1 and 4 '
	  stop
	endif
	write(ifl1, *) iperturb
      endif
11110 format(4f10.3, 2e10.2)
      return
      end

      subroutine read_reflmodel(th,al,be,rho,qal,qbe,nlyrs,ifl,fname)
c
c read velocity layering from a reflectivity file, written by Jeff Gu
c 2005.
c ****PARAMETERS:
c	th  ---  []array of flattened thickness
c	al  ---  []array of   "  "    vp
c	be  ---  []array of   "  "    vs
c      rho  ---  []array of   "  "    density
c      qal  ---  []array of   "  "    Qp
c      qbe  ---  []array of   "  "    Qs
c    nlyrs  ---  number of total layers
c      ifl  ---- input model file identifier
c Output:
c     fname ----  name of the reflectivity file
c
c
c
      parameter (maxlayer=500)
      parameter (maxsource=100)
      parameter (maxsta=100)
      parameter (FF=2049)
      parameter (RR=200)
      character(*) fname
      dimension th(*), al(*), be(*), rho(*), qal(*), qbe(*)
      integer   nlyrs,ifl

	print*, 'I am here!'
      print*, 'enter the reflectivity input file:'
      read(*,"(a)") fname
      open(ifl, file=fname, type='old')
      read(ifl, *) nlyrs
      do i=1, nlyrs
	read(ifl, *) th(i), al(i), be(i), rho(i), qal(i), qbe(i)
      enddo
      close(ifl)
      return
      end

      subroutine edit_reflmodel(th,al,be,rho,qal,qbe,nlyrs,ifl,namein,
	1    iflout)
c
c read a reflectivity file, but replace the velocity and Q structure
c with a new model.  Everything else, like frequency and source-stations
c all stay the same, Jeff Gu, 2005.
c ****PARAMETERS:
c	th  ---  []array of flattened thickness
c	al  ---  []array of   "  "    vp
c	be  ---  []array of   "  "    vs
c      rho  ---  []array of   "  "    density
c      qal  ---  []array of   "  "    Qp
c      qbe  ---  []array of   "  "    Qs
c    nlyrs  ---  number of total layers
c      ifl  ---- input model file identifier
c   namein  ---- name of existing reflectivity inupt file
c   iflout  ---- file pointer for output reflectivity input file
c
c ****Other variables
c
c     pmin  ---  minimum ray slowness
c      plc  ---  slowness of lower crust
c      puc  ---  slowness of upper crust
c     delp  ---  increment in slowness
c     fmin  ---  minimum frequency
c     fmax  ---  maximum frequency
c     fny   ---  nyquist frequency
c     nft   ---  number of frequency points
c     nsou  ---  number of sources
c     nsta  ---- [] number of stations for each source
c   sdepth  ---- [] array of source depths for multiple sources
c   recdep  ---- [][] 2-D array of receiver depths for multiple receivers
c       rh  ---- [][] 2-D array of epicentral distance
c     azis  ---- [][] 2-D array of receiver azimuth
c      mxx  ---- [] cartesian moment tensor components for each source
c      mxy  ---- [] cartesian moment tensor components
c      mxz  ---- [] cartesian moment tensor components
c      myy  ---- [] cartesian moment tensor components
c      myz  ---- [] cartesian moment tensor components
c      mzz  ---- [] cartesian moment tensor components
c iperturb  ---- perturbation option
c	  1 =  Computing Partials for P wave velocity'
c	  2 =  Computing Partials for S wave velocity'
c	  3 =  Computing Partials for P wave velocity, S and Rho tied to P'
c	  4 =  Computing Partials for S wave velocity, P and Rho tied to S'
c
c
      parameter (maxlayer=500)
      parameter (maxsource=100)
      parameter (maxsta=100)
      parameter (FF=2049)
      parameter (RR=200)
      character(*) namein
      dimension th(1), al(1), be(1), rho(1), qal(1), qbe(1)
      dimension sdepth(maxsource),recdep(maxsta,maxsource) 
      dimension nsta(maxsource)
      dimension rh(maxsta,maxsource), azis(maxsta,maxsource)
      real 	deptop(maxlayer)
      real      tt, aa, bb, rro, qqa, qqb
      real      mxx(maxsource),mxy(maxsource),mxz(maxsource)
      real 	myy(maxsource),myz(maxsource),mzz(maxsource)
      integer   nlyrs,nft,nsou,nsta,ifl,iflout,ifl1,ifrd

      call write_reflmodel(iflout,th,al,be,rho,qal,qbe,nlyrs)
c
c  reads reflectivity input file, assume it is open
c
      open(ifl, file=namein, type='old')
      read(ifl,*) nlyrs1
      print*, 'number of layers, old = ', nlyrs1, '  new = ', nlyrs
c
c  read old models and do not save
c
      do i=1, nlyrs1
      	 read(ifl,*) tt, aa, bb, rro, qqa, qqb
      enddo

      read(ifl,*) pmin,plc,puc,pmax,delp
      write(iflout,*) pmin, plc, puc, pmax, delp
      read(ifl,*) fmin,fmax,fny,nft
      write(iflout,"(f8.3 f8.3 f8.3 i6)") fmin,fmax,fny,nft
      
      read(ifl,*)nsou
      write(iflout,"(i3)") nsou
      do isource=1,nsou
	 read(ifl,*) nsta(isource)
	 write(iflout,"(i3)") nsta(isource)
	 do i=1,nsta(isource)
	    read(ifl,*) rh(i,isource),azis(i,isource)       	
	    write(iflout,"(f10.3 f8.3)") rh(i,isource),azis(i,isource)
	 enddo
	 read(ifl,*) sdepth(isource)
	 write(iflout, "(f8.3)") sdepth(isource)

	 do i=1,nsta(isource)
	    read(ifl,*) recdep(i,isource)
	    write(iflout,"(f8.3)") recdep(i,isource)
	 enddo
c
c moment tensor
c
        read(ifl,*) mxx(isource),mxy(isource),mxz(isource),
     *	myy(isource),myz(isource),mzz(isource)
        write(iflout,*) mxx(isource),mxy(isource),mxz(isource),
     *	myy(isource),myz(isource),mzz(isource)

      enddo
      read(ifl,*) iperturb
      write(iflout, *) iperturb
5050  close(ifl)
      close(iflout)
      return
      end

      subroutine write_reflmodel(ifl,th,a,b,rho,qa,qb,nl)
c
c  write the velocity model portion of the reflectivity input file
c
      dimension th(*), a(*), b(*), rho(*), qa(*), qb(*)
      integer nl

      write(ifl,"(i3)") nl
      do i=1, nl
	 write(ifl,"(f8.3 f8.3, f8.3, f8.3 f10.3 f10.3)") th(i),
     &				a(i),b(i),rho(i),qa(i),qb(i)
      enddo
      end

      subroutine xlinearint(x,x0,x1,y0,y1,xx)
c
c Linear interpolation, good enough for the purpose here, though cubic
c interpolation may be more accurate.  But this is time saving.
c
      implicit double precision (a-h, o-z)

      if(x0.eq.x1) then
	 xx=0.5d0*(y0+y1)
      else
	 xx=y0+(y1-y0)*(x-x0)/(x1-x0)
      endif
      return
      end

      subroutine xlinearint_float(x,x0,x1,y0,y1,xx)
c
c Linear interpolation, good enough for the purpose here, though cubic
c interpolation may be more accurate.  But this is time saving.
c
      real x, x0, x1, y0, y1, xx

      if(x0.eq.x1) then
	 xx=0.5d0*(y0+y1)
      else
	 xx=y0+(y1-y0)*(x-x0)/(x1-x0)
      endif
      return
      end

      subroutine depth_to_thickness(dep, th, n, iopt, ifmidpt)
c
c converts depth to thickness and vise versa
c opt=1 (depth to thickness)
c opt=2 (thickness to depth)
c
      dimension dep(1), th(1)
      integer n, iopt, i
      if(iopt.eq.1) then
	 th(1)=dep(1)
	 do i=2, n
		th(i)=dep(i)-dep(i-1)
	 enddo
      else if(iopt.eq.2) then
c
c  use mid points of layers to represent the true depths
c
	 if(ifmidpt.ne.0) then
		dep(1)=th(1)*0.5
	 else
		dep(1)=th(1)
	 endif
	 do i=2, n
		dep(i)=dep(i-1)+th(i)
	 enddo
      else
	 stop 'unreasonable choice!'
      endif
      return
      end

      subroutine submean_1array(a,na)
c
c remove the mean
c
      dimension a(1)
      integer na
      real xmean

      xmean = 0.0
      do i=1, na
	 xmean = xmean + a(i)
      enddo
      xmean = xmean / na
c      print*, 'mean = ', xmean
      do i=1, na
	 a(i)=a(i)-xmean
      enddo
      return
      end

      subroutine submean(a,b,na)
c
c remove the mean
c
      dimension a(1), b(1)
      integer na
      real xmean

      xmean = 0.0
      do i=1, na
	 b(i) = a(i)
	 xmean = xmean + a(i)
      enddo
      xmean = xmean / na
      print*, 'mean = ', xmean
      do i=1, na
	 b(i)=b(i)-xmean
      enddo
      return
      end


      subroutine diff2array(a,b,na,val,iopt)
c
c find difference between two arrays
c iopt:  1=sum of simple difference
c	 2=sum of absolute difference
c	 3=sum of difference squared
c	 4=root-mean-square per data
c
      dimension a(*), b(*)
      integer na, iopt
      real val, diff

      if(iopt.lt.1.and.iopt.gt.4) stop 'diff2array option not found'
      val = 0.0
      do i=1, na
	 diff=a(i)-b(i)
	 if(iopt.eq.1)  val = val + diff
	 if(iopt.eq.2)  val = val + abs(diff)
	 if(iopt.gt.3)  val = val + diff*diff
	 if(iopt.gt.4)  val = val + diff*diff
      enddo
      if(iopt.eq.4) val=sqrt(val/real(na))
      return
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c    routine to invert a matrix by Gaussian elimination
c     Ainv=inverse(A)    A(n,n)   Ainv(n,n)
c     Note: sizes maxed at 64 x 64 - re dimension for larger matrices;
c           D matrix is extended.
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
       subroutine invert( A, Ainv, n, maxlyr)
       dimension A(maxlyr,maxlyr),Ainv(maxlyr,maxlyr)
       dimension D(maxlyr,maxlyr*2)

c  initialize the reduction matrix
       n2 = 2*n
       do 1 i = 1,n
        do 2 j = 1,n
         D(i,j) = A(i,j)
         d(i,n+j) = 0.
 2      continue
        D(i,n+i) = 1.
 1     continue

c  do the reduction 
       do 3 i = 1,n
        alpha = D(i,i)
        if(alpha .eq. 0.) go to 300
        do 4 j = 1,n2
         D(i,j) = D(i,j)/alpha
 4      continue
        do 5 k = 1,n
         if((k-i).eq.0) go to 5
         beta = D(k,i)
         do 6 j = 1,n2
          D(k,j) = D(k,j) - beta*D(i,j)
 6       continue
 5      continue
 3     continue

c  copy result into output matrix
       do 7 i = 1,n
        do 8 j = 1,n
         Ainv(i,j) = D(i,j+n)
 8      continue
 7     continue

       return
 300   print *,'*** ERROR: Singular matrix ***'
       return
       end
      subroutine choles(a,g,b,y,x,n,nono)

      implicit real*4 (a-h, o-z)
      real*4 a(1),g(1)
      real*4 b(1),y(1),x(1)
c
c        a= row-wise p.d. symm. system  n*(n+1)/2
c        g= cholesky storage
c        b= r.h.s. vector               n
c        y= temp. vector
c        x= answer vector
c        n= system dimension
c        nono .gt. 0 is the level at which p.d. failed
c
c        (a,g) and (b,y,x) may be equivalenced.
c
c----------------------------------------------------------
c-----
c     first compute cholesky decomposition

      nono=0
      
      if(a(1).le.0.) then
      nono=1
      return
      endif
      
      g(1)=sqrt(a(1))
      y(1)=b(1)/g(1)

      do 400 i=2,n
      
      kz=(i*(i-1))/2
      g(kz+1)=a(kz+1)/g(1)
      sg=g(kz+1)**2
      y(i)=b(i)-g(kz+1)*y(1)
      
      if(i.gt.2) then
      
      jmax=i-1
      
      do 200 j=2,jmax
      
      gkz=a(kz+j)
      kj=(j*(j-1))/2
      kmax=j-1
      
      do 100 k=1,kmax
      gkz=gkz-g(kz+k)*g(kj+k)
100   continue

      g(kz+j)=gkz/g(kj+j)
      y(i)=y(i)-g(kz+j)*y(j)
      sg=sg+g(kz+j)**2
      
200   continue

      endif
      
      gkz=a(kz+i)-sg
      
      if(gkz.le.0.) then
      nono=i
      return
      endif
      
      g(kz+i)=sqrt(gkz)
      y(i)=y(i)/g(kz+i)
      
400   continue

      kz=(n*(n-1))/2
      x(n)=y(n)/g(kz+n)
      if(n.le.1) return

c-----
c     compute solution for particular rhs
      
      do 600 k=2,n
      
      i=n+1-k
      x(i)=y(i)
      jmin=i+1
      
      do 500 j=jmin,n
      kj=(j*(j-1))/2
      x(i)=x(i)-g(kj+i)*x(j)
500   continue

      kz=(i*(i+1))/2
      x(i)=x(i)/g(kz)
      
600   continue

      return
      end

      subroutine read_2coldat(ifile, filename, x, y, ndata)
c
c  read a 2-col file and save into x, y, ndata
c
      character*(*) filename
      dimension x(*), y(*)
      integer ndata, ifile
	
      open(ifile, name=filename, iostat=ios)
      if(ios.ne.0) then
	 stop 'Error opening file, check again...'
      endif
      i=1
10    read(ifile, *, end=20) x(i), y(i)
      i=i+1
      goto 10
20    ndata=i-1
      close(ifile)
      return
      end

      subroutine read_1coldat(ifile, x, maxd, ndata)
c
c  read a 2-col file and save into x, y, ndata
c
      character*128 filename
      dimension x(1)
      integer ndata, ifile, maxd

      print*, 'enter input file:'
      read(*,"(a)") filename
      open(ifile, name=filename, iostat=ios)
      if(ios.ne.0) then
	 stop 'Error opening file, check again...'
      endif
      i=1
10    read(ifile, *, end=20) x(i)
      if(i.gt.maxd) stop 'too many data points!'
      i=i+1
      goto 10
20    ndata=i-1
      close(ifile)
      print*, 'number of data read = ', ndata
      return
      end

      subroutine find_min_max_mean(x, ndata, xmin, xmax, xabsmax, xmean)
      dimension x(*)
      integer ndata
      real    xmin, xmax, xabsmax, xmean

      xmin=1.0e19      
      xmax=-1.0e-19      
      xabsmax=-1.0e-19     
      xmean=0.0
      do i=1, ndata
	if(xmin.gt.x(i)) xmin=x(i)
	if(xmax.lt.x(i)) xmax=x(i)
	if(xabsmax.lt.abs(x(i))) xabsmax=abs(x(i))
	xmean=xmean+x(i)
      enddo
      xmean=xmean/ndata
      print*, 'min=', xmin, '  max=', xmax, '  mean=', xmean
      return
      end

      subroutine find_stat_rangelimited(x, y, ndata, xmin, xmax, 
     %		ymin, ymax, yabsmax, ymean)
      dimension x(1), y(1)
      integer ndata, ndata_good
      real    ymin, ymax, yabsmax, ymean

      ymin=1.0e19      
      ymax=-1.0e-19      
      yabsmax=-1.0e-19     
      ymean=0.0
      do i=1, ndata
	if(x(i).ge.xmin.and.x(i).le.xmax) then
		if(ymin.gt.y(i)) ymin=y(i)
		if(ymax.lt.y(i)) ymax=y(i)
		if(yabsmax.lt.abs(y(i))) yabsmax=abs(y(i))
		ymean=ymean+y(i)
		ndata_good = ndata_good + 1
	endif
      enddo
      ymean=ymean/ndata_good
      print*, 'min=', ymin, '  max=', ymax, '  mean=', ymean
      return
      end

      subroutine get_density(depth, n, ifsame, rho1)
c
c    dep0   ----   starting depth to use a given model
c    depth  ----   list of depths
c    n      ----   number of original depths
c    n      ----   number of original depths
c
      parameter(maxlyr=1000)
      dimension depth(1)
      dimension disclyr(1)
      dimension depth1(1), rho1(1), rho2(maxlyr)
      real 	deptmp(maxlyr), rhotmp(maxlyr), vstmp(maxlyr), vptmp(maxlyr)
      real 	deptmp1(maxlyr), depthnew(maxlyr)
      real	disc(maxlyr)
      real     x0, x1, y0, y1
      integer i, j, ndisc, ninsert, ntmp

      print*, 'enter density model:'
      print*, '1. PREM'
      print*, '2. Cara97 model + PREM for deeper depths, if any'
      print*, '3. IASPEI91 model'
      print*, '4. Cara97 model + IASPEI91 for deeper depths, if any'
      print*, 'enter:'
      read(*,*) ioprho
      if(ioprho.lt.1.or.ioprho.gt.4) then
		stop 'unresonable choice for Density!'
      endif
      ntmp=0
      depmin=0.0
      istart=1
      n1=0
      if(ioprho.eq.2.or.ioprho.eq.4) then
	call read_cara(445454,deptmp,vptmp,vstmp,rhotmp,ntmp)
	call get_depval1(depth,n,deptmp,rhotmp,ntmp,0,ifsame,0.0,
     +			deptmp(ntmp),1,depthnew,rho1,ntmp1)
	print*, 'ntmp=', ntmp,   '   deptmp =', deptmp(ntmp)
		depmin=deptmp(ntmp)
		istart=ntmp1+1
		n1=ntmp1
      endif
      if(ioprho.gt.2) then
	call read_iaspei(11,deptmp,vptmp,vstmp,
     %		rhotmp,n2,disc,ndisc)
	print*, ifsame, depmin, istart, dep
	call get_depval1(depth,n,deptmp,rhotmp,n2,0,ifsame,depmin,
     +			6371.0,istart,deptmp1,rho1,ntmp)
		print*, 'n1=', n1,  '  n=', n, '  ntmp=', ntmp
		do i=1, n
			print*, i, depth(i), '  rho=', rho1(i)
		enddo
      else
		do i=n1+1, n
		  call premmodel(depth(i),1,va,vb,rho1(i),
     %		qa,qb,disc,ndisc)
		enddo
      endif
      return
      end

      subroutine get_Q(depth, n, ifsame, qa, qb)
      parameter(maxlyr=1000)
      dimension depth(1), qa(1), qb(1)
      dimension thm(maxlyr), vs(maxlyr), sigma(maxlyr)
      dimension deptmp(maxlyr), tmpqa(maxlyr), tmpqb(maxlyr)
      dimension depthnew(maxlyr)
      dimension disc(maxlyr)
      character*128 modname
      integer iop, i
c
c  get the Qa and Qb for a number of values
c
      print*, 'enter Q model:'
      print*, '1. PREM'
      print*, '2. Ding&Grand Qs in EPR + PREM'
      print*, '3. Xu and Wiens Lau Basin + PREM'
      print*, '4. Xu and Wiens North Fiji + PREM'
      print*, '5. Xu and Wiens South Fiji + PREM'
      print*, '6. Xu and Wiens Pacific Plate + PREM'
      print*, 'enter:'
      read(*,*) iop
      if(iop.lt.1.or.iop.gt.6) stop 'unresonable choice for Q model!'
      ntmp=0
      depmin=0.0
      istart=1
      n1=0
      if(iop.eq.2) then
		call read_ding_grand(121221, deptmp, tmpqb, ntmp)
      endif
      if(iop.ge.3) then
		if(iop.eq.3) modname='/data/herbie/gu/dbase/model1D/Xu_Wiens_Lau.mod'
		if(iop.eq.4) modname='/data/herbie/gu/dbase/model1D/Xu_Wiens_NFiji.mod'
		if(iop.eq.5) modname='/data/herbie/gu/dbase/model1D/Xu_Wiens_SFiji.mod'
		if(iop.eq.6) modname='/data/herbie/gu/dbase/model1D/Xu_Wiens_Pacific.mod'
		call read_xu_wiens(4454,modname,thm,vs,sigma,tmpqb,ntmp)
		call depth_to_thickness(deptmp, thm, ntmp, 2, 1)
      endif
      if(iop.ne.1) then
		print*, 'scaling factor for P (suggested 1.75 or 2.25):'
		read(*,*) scfac
		do i=1, ntmp
			tmpqa(i)=tmpqb(i)*scfac
		enddo
		call get_depval(depth,n,deptmp,tmpqa,tmpqb,ntmp,0,ifsame,0.0,
     +			deptmp(ntmp),1,depthnew,qa,qb,ntmp1)
		istart=ntmp1+1
      endif

      do i=istart, n
	call premmodel(depth(i),1,va,vb,rho,qa(i),qb(i),disc,ndisc)
      enddo
	do i=1, n
		print*, i, depth(i), qa(i), qb(i)
	enddo
      return
      end

	subroutine read_ding_grand(ifl, depth, qs, nl)
c
c  read Ding and Grand, 97 paper
c
	dimension depth(1), qs(1)
	integer ifl, nl

	open(ifl, file='/data/herbie/gu/dbase/model1D/Ding_QTNA_S.mod', 
     %			type='old')
	i=1
10	read(ifl, *, end=20) depth(i), qs(i)
	i=i+1
	goto 10
20	nl=i-1
	close(ifl)
	return
	end
	
	subroutine get_crustmod(icrust, crustmod)
	integer icrust
	character crustmod*(*)
	print*, 'choose crust model:'
	print*, '--------------------------------------'
	print*, '1. skip'
	print*, '2. oceanic crust 0-4 myr old'
	print*, '3. oceanic crust 4-20 myr old'
	print*, '4. oceanic crust 20-52 myr old'
	print*, '5. oceanic crust 52-110 myr old'
	print*, '6. oceanic crust 110+ myr old'
	print*, '--------------------------------------'
	print*, 'enter:'
	read(*,*) icrust
	if(icrust.lt.1.or.icrust.gt.6) stop 'unreasonable choice for crust!'
	if(icrust.eq.2) then
	   crustmod='/data/herbie/gu/dbase/nishimura_forsyth/crust_0-4my.mod'
	else if(icrust.eq.3) then
	   crustmod='/data/herbie/gu/dbase/nishimura_forsyth/crust_4-20my.mod'
	else if(icrust.eq.4) then
	   crustmod='/data/herbie/gu/dbase/nishimura_forsyth/crust_20-52my.mod'
	else if(icrust.eq.5) then
	   crustmod='/data/herbie/gu/dbase/nishimura_forsyth/crust_52-110my.mod'
	else if(icrust.eq.6) then		
	   crustmod='/data/herbie/gu/dbase/nishimura_forsyth/crust_110+my.mod'
	endif
	return
	end

	subroutine get_mantlemod(imantle, mantlemod)
c
c get mantle model option
c
	integer imantle
	character mantlemod*(*)

	print*, 'choose mantle/core velocity model:'
	print*, '---------------------------------------------'
	print*, '1. no mantle model'
	print*, '2. Nishimura&Forsyth mantle 0-4 myr old'
	print*, '3. Nishimura&Forsyth mantle 4-20 myr old'
	print*, '4. Nishimura&Forsyth mantle 20-52 myr old'
	print*, '5. Nishimura&Forsyth mantle 52-110 myr old'
	print*, '6. Nishimura&Forsyth mantle 110+ myr old'
	print*, '7. Cara P7 model (see N&F 89)'
	print*, '8. PREM mantle'
	print*, '9. IASPEI mantle'
	print*, '10. SNA mantle'
	print*, '11. TNA mantle'
	print*, '12. read from a reflectivity input file'
	print*, '13. Xu and Wiens Lau Basin'
	print*, '14. Xu and Wiens North Fiji'
	print*, '15. Xu and Wiens South Fiji'
	print*, '16. Xu and Wiens Pacific Plate'
	print*, '---------------------------------------------'
	print*, 'enter:'
	read(*,*) imantle
	if(imantle.eq.2) then
		mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/nishimura89_0-4my.mod'
	else if(imantle.eq.3) then
		mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/nishimura89_4-20my.mod'
	else if(imantle.eq.4) then
	   	mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/nishimura89_20-52my.mod'
	else if(imantle.eq.5) then
		mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/nishimura89_52-110my.mod'
	else if(imantle.eq.6) then		
		mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/nishimura89_110+my.mod'
	else if(imantle.eq.7) then		
		mantlemod='/data/herbie/gu/dbase/nishimura_forsyth/cara_p7.mod'
	else if(imantle.eq.9) then
		mantlemod='/data/herbie/gu/dbase/model1D/iaspei140.dat'
	else if(imantle.eq.10) then
		mantlemod='/data/herbie/gu/dbase/model1D/sna.b'
	else if(imantle.eq.11) then
		mantlemod='/data/herbie/gu/dbase/model1D/tna.b'
	else if(imantle.eq.13) then
		mantlemod='/data/herbie/gu/dbase/model1D/Xu_Wiens_Lau.mod'
	else if(imantle.eq.14) then
		mantlemod='/data/herbie/gu/dbase/model1D/Xu_Wiens_NFiji.mod'
	else if(imantle.eq.15) then
		mantlemod='/data/herbie/gu/dbase/model1D/Xu_Wiens_SFiji.mod'
	else if(imantle.eq.16) then
		mantlemod='/data/herbie/gu/dbase/model1D/Xu_Wiens_Pacific.mod'
	endif
	if(imantle.lt.1.or.imantle.gt.16) stop 'unreasonable choice!'
	return
	end


	subroutine get_depval1(depthin,ndepin,depm,vpm,nmod,ifth,ifsamedep,
     +			dmin, dmax, istart,depth_new,vp,ngood)
c
c  put interpolated model values to an array that start at index istart
c
c	INPUT:
c		depthin --- array of input depth or model depth (can be thickness also)
c		ndepin  --- number of elements in depthin
c		  ifth  --- if it is thickness
c		  depm  --- model depths
c		   vpm  --- model value needed to interpolate, can be vp, vs, rho, or whatever
c		  nmod  --- number of model layers
c	     ifsamedep  --- if desired output depth sampling depthin are the same as model depths depm
c	     	  dmin  --- minimum depth used
c	     	  dmax  --- maximum depth used
c	        istart  --- starting index for output array
c	OUTPUT:
c	     depth_new  --- output depth
c	     	    vp  --- final vp
c	     	 ngood  --- number of output depths
c
	dimension depthin(1), vpm(1)
	dimension depth_new(1), vp(1), depm(1)
	integer ifth, istart, ifsamedep, ngood, nmod, i
	real	dep0, dep1
	
	i=1
	dep0=0.0
	dep1=0.0
	ncrust=0
	do while(i.le.nmod) 
			if(ifth.eq.1) then	! input thickness rather than depth
				dep1=dep1+depm(i)
			else
				dep1=depm(i)
			endif
			if(ifsamedep.eq.1) then
				j=i+istart-1
				depth_new(j)=dep1
				vp(j)=vpm(i)
			endif
			dep0=dep1
			i=i+1
	 enddo
	 if(ifsamedep.eq.0) then
		ngood=0
		ind=istart-1
		do i=1, ndepin
		   if(depthin(i).gt.dmax) then
			goto 220
		   endif
		   if(depthin(i).gt.dmin) then
			dep0=0.0
			dep1=0.0
			do j=1, nmod
				if(ifth.eq.1) then	! input thickness rather than depth
					dep1=dep1+depm(j)
				else
					dep1=depm(j)
				endif
				if(depthin(i).gt.dep0.and.depthin(i).le.dep1) then
					ind=ind+1
					depth_new(ind)=depthin(i)
					if(j.eq.1) then
						vp(ind)=vpm(1)	
					else
						call xlinearint_float(depthin(i),dep0,dep1,vpm(j-1),vpm(j),vp(ind))
					endif
					ngood=ngood+1
					continue
				endif
				dep0=dep1
			enddo
		   endif
		enddo
	else
		ngood=nmod
	endif
220	print*, 'total depth confirmed = ', ngood
	return
	end


	subroutine get_depval(depthin,ndepin,depm,vpm,vsm,nmod,ifth,ifsamedep,
     +			dmin, dmax, istart,depth_new,vp,vs,ngood)
c
c  put interpolated model values to an array that start at index istart
c
c	INPUT:
c		depthin --- array of input depth or model depth (can be thickness also)
c		ndepin  --- number of elements in depthin
c		  ifth  --- if it is thickness
c		  depm  --- model depths
c		   vpm  --- model P velocity
c		   vsm  --- model S velocity
c		  nmod  --- number of model layers
c	     ifsamedep  --- if desired output depth sampling depthin are the same as model depths depm
c	     	  dmin  --- minimum depth used
c	     	  dmax  --- maximum depth used
c	        istart  --- starting index for output array
c	OUTPUT:
c	     depth_new  --- output depth
c	     	    vp  --- final vp
c	     	    vs  --- final vs
c	     	 ngood  --- number of output depths
c
	dimension depthin(1), vpm(1), vsm(1)
	dimension depth_new(1), vp(1), vs(1), depm(1)
	integer ifth, istart, ifsamedep, ngood, nmod, i
	real	dep0, dep1
	
	i=1
	dep0=0.0
	dep1=0.0
	ncrust=0
	do while(i.le.nmod) 
			if(ifth.eq.1) then	! input thickness rather than depth
				dep1=dep1+depm(i)
			else
				dep1=depm(i)
			endif
			if(ifsamedep.eq.1) then
				j=i+istart-1
				depth_new(j)=dep1
				vp(j)=vpm(i)
				vs(j)=vsm(i)
			endif
			dep0=dep1
			i=i+1
	 enddo
	 if(ifsamedep.eq.0) then
		ngood=0
		ind=istart-1
		do i=1, ndepin
		   if(depthin(i).gt.dmax) then
			goto 220
		   endif
		   if(depthin(i).gt.dmin) then
			dep0=0.0
			dep1=0.0
			do j=1, nmod
				if(ifth.eq.1) then	! input thickness rather than depth
					dep1=dep1+depm(j)
				else
					dep1=depm(j)
				endif
				if(depthin(i).gt.dep0.and.depthin(i).le.dep1) then
					ind=ind+1
					depth_new(ind)=depthin(i)
					if(j.eq.1) then
						vp(ind)=vpm(1)	
						vs(ind)=vsm(1)	
					else
						call xlinearint_float(depthin(i),dep0,dep1,vpm(j-1),vpm(j),vp(ind))
						call xlinearint_float(depthin(i),dep0,dep1,vsm(j-1),vsm(j),vs(ind))
					endif
					ngood=ngood+1
					continue
				endif
				dep0=dep1
			enddo
		   endif
		enddo
	else
		ngood=nmod
	endif
220	print*, 'total depth confirmed = ', ngood
	return
	end

      subroutine  check_if_inbound(xla,xlo,bla0,blo0,bla1,blo1,iftrue)
      real xla, xlo, bla0, blo0, bla1, blo1
      integer iftrue
c
c check if a point is within a given boundary lat,lon box
c
      clo = xlo
      clo0 = blo0
      clo1 = blo1
      if(clo.lt.0.0) clo = clo+360.0
      if(clo0.lt.0.0) clo0 = clo0+360.0
      if(clo1.lt.0.0) clo1 = clo1+360.0
      
      iftrue=0
      if(xla.ge.bla0.and.xla.le.bla1) then
	if(clo.ge.clo0.and.clo.le.clo1) then
		iftrue=1
	endif
      endif
      return
      end
	
      subroutine monte_carlo(xmin, xmax, n, xrand)
c
c This subroutine uses the time function to generate a random number
c between 0 and 1 and then scale it to the range provided by user.
c written by Jeff Gu, 2005.
c      
c  Input:  
c	xmin ---- minimum of the range
c	xmax ---- maximum of the range
c	n    ---- number of random number desired
c  Output:
c	xrand ---- a real array containing the random numbers in range
      dimension xrand(*) 
      real xmin, xmax
      real xx, range
      integer idum

      range=xmax-xmin
      mytime=time()
	print*, 'current time is ', mytime
      idum=-mytime    ! idum must be negative
      do i=1, n
      	xx=ran1(idum)
	idum=-mytime-i
      	xrand(i)=xmin+xx*range      
      enddo
      return
      end

      subroutine edit_anymodel(xx, ind, fac)
c
c This subroutine adds or subtracts a certain percentage from a 
c given array of values.
c      
c  Input:  
c	xx      ---- array to modify
c	ind     ---- index of the element of the array to modify
c	fac     ---- percentage to add or subtract
c  Output:
c	xx ---- modified array
      dimension xx(*) 
      real*4    fac
      integer   ind

      xx(ind)=xx(ind)+xx(ind)*fac
      return
      end

c
c some functions related to statistics and bootstrapping
c
      FUNCTION RAN1(IDUM)
      DIMENSION R(97)
      PARAMETER (M1=259200,IA1=7141,IC1=54773,RM1=3.8580247E-6)
      PARAMETER (M2=134456,IA2=8121,IC2=28411,RM2=7.4373773E-6)
      PARAMETER (M3=243000,IA3=4561,IC3=5134)
      DATA IFF /0/
      IF (IDUM.LT.0.OR.IFF.EQ.0) THEN
        IFF=1
        IX1=MOD(IC1-IDUM,M1)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX2=MOD(IX1,M2)
        IX1=MOD(IA1*IX1+IC1,M1)
        IX3=MOD(IX1,M3)
        DO 11 J=1,97
          IX1=MOD(IA1*IX1+IC1,M1)
          IX2=MOD(IA2*IX2+IC2,M2)
          R(J)=(FLOAT(IX1)+FLOAT(IX2)*RM2)*RM1
11      CONTINUE
        IDUM=1
      ENDIF
      IX1=MOD(IA1*IX1+IC1,M1)
      IX2=MOD(IA2*IX2+IC2,M2)
      IX3=MOD(IA3*IX3+IC3,M3)
      J=1+(97*IX3)/M3
      IF(J.GT.97.OR.J.LT.1)PAUSE
      RAN1=R(J)
      R(J)=(FLOAT(IX1)+FLOAT(IX2)*RM2)*RM1
      RETURN
      END

      SUBROUTINE MOMENT(DATA,N,AVE,ADEV,SDEV,VAR,SKEW,CURT)
      DIMENSION DATA(N)
      IF(N.LE.1)PAUSE 'N must be at least 2'
      S=0.
      DO 11 J=1,N
        S=S+DATA(J)
11    CONTINUE
      AVE=S/N
      ADEV=0.
      VAR=0.
      SKEW=0.
      CURT=0.
      DO 12 J=1,N
        S=DATA(J)-AVE
        ADEV=ADEV+ABS(S)
        P=S*S
        VAR=VAR+P
        P=P*S
        SKEW=SKEW+P
        P=P*S
        CURT=CURT+P
12    CONTINUE
      ADEV=ADEV/N
      VAR=VAR/(N-1)
      SDEV=SQRT(VAR)
      IF(VAR.NE.0.)THEN
        SKEW=SKEW/(N*SDEV**3)
        CURT=CURT/(N*VAR**2)-3.  
      ELSE
        PAUSE 'no skew or kurtosis when zero variance'
      ENDIF
      RETURN
      END

	subroutine varred(ar1, ar2, n, vred, iopt)
c
c computes variance reduction in percent.  There are two ways to do so
c 1. keep mean   2. remove mean, which is specified by iopt.
c
	real ar1(*), ar2(*)
	real vred, sum1, sum2
	integer n, iopt
	
	sum1=0.0
	sum2=0.0
	if(iopt.eq.1) then
		do i=1, n
			sum1=sum1+(ar1(i)-ar2(i))*(ar1(i)-ar2(i))
			sum2=sum2+ar1(i)*ar1(i)
		enddo
	else
		call submean_1array(ar1,n)
		call submean_1array(ar2,n)
		do i=1, n
			sum1=sum1+(ar1(i)-ar2(i))*(ar1(i)-ar2(i))
			sum2=sum2+ar1(i)*ar1(i)
		enddo
	endif
	vred=100.0*(1.0-sum1/sum2)
	return
	end

        subroutine correlate_func(x1,x2,x3,x4,n,lagfun,corrfun)
c
c does cross-correlation function for a selected window with both
c seismograms
c
        real x1(*), x2(*), x3(*), x4(*), corrfun(*), lagfun(*)
        real corrwin(5000), padded(5000)
        real avg1, avg2, sum, sum1, sum2
        sum1 = 0.0
        sum2 = 0.0
        do i=1, n
                corrwin(i)=0.0  
                padded(i)=0.0   
        enddo
        do i=1,n
                corrwin(i)=x2(i)
        enddo
c
c  use n*2 as the number of lags, from -lag to 0 lag to positive lag
c  such that x4 will come into the window and then move completely out
c  of the window.
c
        n2 = n*2
        ilag=-n
c  finds total length of trace, start with lag=-length of it.
        xlag=-(x1(n)-x1(1))
        do i=1, n2-1
c  initialize padded each time
                do j=1, n
                        padded(j)=0.0
                enddo
c  add non-zero elements, move x4 into padded, from left to right
c  note fortran indices start from 1, not 0
                k=i
                if(i.le.n) then
c before passing 0 lag
                        do j=1, i
                                padded(j)=x4(n-i+j)
                        enddo
                else
                        j0=i-n+1
                        k=1
                        do j=j0, n
                                padded(j)=x4(k)
                                k=k+1
                        enddo
                endif
                call relate(x1, x2, x3, padded, n, xcorr)
                lagfun(i)=xlag
                corrfun(i)=xcorr
                ilag=ilag+1
c assume uniform sampling
                xlag=xlag+x1(2)-x1(1)
        enddo
        close(3)
        end

        subroutine relate(x1,x2,x3,x4,n, rfit)
c       This subroutine should calculate the cross correlation between two
c       seismograms.  The fit is between 0 (poor) and 1 (awesome!!).
        real x1(*), x2(*), x3(*), x4(*)
        real rfit, avg1, avg2, sum, sum1, sum2, sum3
        k = 1
c
c       suppose we have the simple case  i = j for now
c       
        sum1 = 0.0
        sum2 = 0.0
        sum3 = 0.0
        sum = 0.0
        do k=1, n
                sum1 = sum1 + x2(k)
                sum2 = sum2+ x4(k)
        enddo
        avg1=sum1/n
        avg2=sum2/n
        sum1 = 0.0
        sum2 = 0.0
        sum3 = 0.0
        sum = 0.0
        do 300 k = 1, n
                temp1 = (x2(k)-avg1)**2
                temp2 = (x4(k)-avg2)**2
                temp3 = (x2(k)-avg1)
                temp4 = (x4(k)-avg2)
                sum3 = sum3 + temp3*temp4
                sum1 = sum1 + temp1
                sum2 = sum2 + temp2
300     continue
        sum = (sum1*sum2)**0.5
        rfit = sum3/sum
        return
        end

