#include <stdio.h>
#include <math.h>
#include <assert.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#define radian 0.01745329
void Bounce_Point(lat1,lon1,lat2,lon2,bplat, bplon);
void Deltaz(float xlat,float xlon,float stlat,float stlong,float *delta,float *azep,float *azst);


void Bounce_Point(lat1,lon1,lat2,lon2,bplat, bplon)
/* this subroutine calculates the coordinates of bounce points */
float lat1, lon1, lat2, lon2;
float *bplat, *bplon;
{
float theta1, phi1, theta2, phi2;
float x, y, z , r, theta, phi;
   /* coordinates of earthquake */
   geosph(lat1,lon1,&theta1,&phi1);
   /* coordinates of station   */
   geosph(lat2,lon2,&theta2, &phi2);
   /* the bounce points */
   x = sin(theta1)*cos(phi1)+sin(theta2)*cos(phi2);
   y = sin(theta1)*sin(phi1) + sin(theta2)*sin(phi2);
   z = cos(theta1)+cos(theta2);
   sphere(x, y, z, &r,&theta, &phi);
   theta = theta/radian;
   phi = phi/radian;
   *bplat = 90.0 - theta;
   *bplon = phi;
   if(*bplon >= 180.0)
      *bplon = *bplon - 360.0;
}
#define MAXSTR 256
#define pi     3.14159265359
#define radian 0.01745329
int Seed;

/*   This subroutine will convert the geographic latitude and longitude 
 *   to spherical coordinate
 *   Input:  xlat -- latitude  (in degree)
 *          xlong-- longitude (in degree)
 *   Output: theta, phi -- spherical coordinates (in radian)
 */
void geosph(float lat, float lon, float *theta, float *phi)
{
   *theta = (90.0-lat)*radian;
   *phi = lon;
   if(*phi < 0.0) 
      *phi =* phi + 360.0;
   *phi = *phi*radian;
}

/*     this subroutine translates cartesian coordinates to spherical
 *     coordinates
 *.....input:
 *              x, y, z       -- the input cartesian coordinates
 *.....output:
 *              r, theta, phi -- the resulting spherical coordinates
 *                 theta and phi are in radian
 *.....note:    when theta is less than 1.0e-6, phi is set to 0.0
 *              when r is less than 1.0e-6, phi, theta are set to 0.0
 */
void sphere(float x, float y, float z, float *r, float *theta, float *phi)
{
   *theta = 0.0;
   *phi = 0.0;
   *r = sqrt(x*x + y*y + z*z);
   if(*r > 0.000001) {
      *theta =  acos(z/ *r);
      if(fabs(*theta) > 0.000001) {
	 *phi = atan2(y, x);
	 if(*phi < 0.0)
	    *phi = *phi + 2.0 * pi;
      }
   }
}

void Deltaz(float xlat,float xlon,float stlat,float stlong,float *delta,float *azep,float *azst)
{
      float RAD,REPRAD,GEOCO;
      float SQ3O2, TWOPI;
      float stl,slon,elat,elon, a, b, c, d;
      float cdif, sdif, cdel;
      float as, bs, cs, ds, el;
      float aze, azs, caze, cazs, dif, delt, sdel;
      float temp1, temp2;

      RAD=0.0174533;
      REPRAD=57.295779;
      GEOCO=0.993277;
      SQ3O2=0.8660254;
      TWOPI=6.2831853;
         
/* First calculate the necessary values that are needed later
 * but only do it once.
 */
      if(xlat>=90.) xlat=89.98;
      if(xlat<=-90.) xlat=-89.98;
      el=atan(GEOCO*tan(xlat*RAD));
      el=TWOPI*0.25-el;
      if(stlat>=90.) stlat=89.98;
      if(stlat<=-90.) stlat=-89.98;
      stl=atan(GEOCO*tan(stlat*RAD));
      stl=TWOPI*0.25-stl;
      elon=xlon*RAD;
      slon=stlong*RAD;
      as=cos(stl);
      bs=sin(stl);
      cs=cos(slon);
      ds=sin(slon);
 
      a=cos(el);
      b=sin(el);
      c=cos(elon);
      d=sin(elon);

      cdel=a*as+b*bs*(c*cs+d*ds);
      if(cdel>1.0) cdel=1.0;
      if(cdel<-1.0) cdel=-1.0;
      delt=acos(cdel);
      *delta=REPRAD*delt;

      sdel=sin(delt);
      caze=(as-a*cdel)/(sdel*b);
      if(caze>1.0) caze=1.0;
      if(caze<-1.0) caze=-1.0;
  
      aze=acos(caze);
      if(bs>0.0) cazs=(a-as*cdel)/(bs*sdel); 
      if(bs==0.0) {
         if(cazs>=0.0) caze=1.0;
         if(cazs<0.0) caze=-1.0;
      }
      if(cazs>1.0) cazs=1.0;
      if(caze<-1.0) cazs=-1.0;
      azs=acos(cazs);
      dif=ds*c-cs*d;
      if(dif<0.) aze=TWOPI - aze;
      *azep=aze*REPRAD;
      if(dif>0.) azs=TWOPI - azs;
      *azst=azs*REPRAD;
}

void RotateToGCP(float *arrayN, float *arrayE, int nt, float bazm, float *xRadial, float *xTrans)
{  /* rotate NS, EW component seismograms to great circle path, 
      assuming bazm is the station-event back-azimuth in degrees.
      Radial points to station from earthquake.  All rotations are
      clockwise.
    */
    int i;
    float bazm1;
    float sinx, cosx;
	
     printf("bazm = %f\n", bazm);
    if(bazm<0.0) bazm=bazm+360.;
    bazm1=bazm*radian;
    sinx=-sin(bazm1);
    cosx=-cos(bazm1);
    for(i=0;i<nt;i++){ 
      xRadial[i] = arrayN[i]*cosx+arrayE[i]*sinx;
      xTrans[i] = arrayE[i]*cosx-arrayN[i]*sinx;
    }	
}

void SubMean(float *array1, int n)
{
  int i;
  float mean;
    mean=0.0;
    for(i=0; i<n; i++){
	mean += array1[i];
    }
    mean /= (float) n;
    for(i=0; i<n; i++){
	array1[i] -= mean;
    }
}



void CopyArray(float *array1, int n, float *array2)
{
  int i;
    for(i=0; i<n; i++){
	array2[i]=array1[i];
    }
}


void RotateThrough(float *arrayN, float *arrayE, int nt, float angx)
{  /* rotate NS, EW component seismograms through a
    *  given angxle angx. 
    */
    int i;
    float sinx, cosx;
    float yn, ye;
	
    angx=angx*radian;
    sinx=sin(angx);
    cosx=cos(angx);
    for(i=0;i<nt;i++){
      yn = arrayN[i];
      ye = arrayE[i];
      arrayN[i] = yn*cosx + ye*sinx;
      arrayE[i] = ye*cosx - yn*sinx;
    }	
}

FILE *Open_AppendFile(name)
char name[MAXSTR];
{
  FILE *fp;
  int i;
  char s[MAXSTR];

  i = 0;
  while ((fp=fopen(name,"a"))==NULL){
    printf("Something is wrong with file %s \n",name);
    printf("enter another file name:\n");
    if(++i>10) exit(1);
  }
  return fp;
}
 
void read2colFloatData(char *filename, float *xx, float *yy, int *npt)
{  /* reads a 2-col xy file */
    FILE *fpr2fd;
    int i;

    fpr2fd=fopen(filename, "r");
    i=0;
    while((fscanf(fpr2fd, "%f %f", &xx[i], &yy[i])) != EOF){
	i++;
    }
    printf("total number of data read = %d\n", i);
    *npt = i;
    fclose(fpr2fd);
}

void makePowerOf2(float *xx, int npt, int *npow)
{  /* reads a 2-col xy file */
    int i, j, nn, npt1, nptnew;
 
    printf("npt = %d\n", npt);
    npt1=1;
    for(i=1; i<16; i++){
	   npt1 *= 2;  
	   if(npt>npt1 && npt<=npt1*2) {
 	      nn=i+1;
	      printf("The power of 2 is %d\n", nn);
 	      nptnew=pow(2,nn);
		printf("nptnew = %d\n", nptnew);
	   }
    }
 /* padd zeros */
    for(i=npt; i<nptnew; i++){
	   xx[i]=0.0;
    }
    *npow = nn-1;
}


void calcFrequency(float *xx, float *wtau, int npt, float tshift, float dt)
{  /* reads a 2-col xy file */
    int i, j, nf, io, ie;
    float tmpreal, tmpimag, df;
    float ppi;

    ppi = 3.141592653579; 
    df = 1.0/npt;
    nf = npt/2+1;
    for(i=0; i<nf; i++){
	   wtau[i] = 2.0*ppi*i*df*tshift/dt;
    }
    for(i=0; i<nf; i++){
  	   io=2*i;
  	   ie=2*i+1;
  	   tmpreal = xx[io];
  	   tmpimag = xx[ie];
  	   xx[io] = tmpreal*cos(wtau[i]) + tmpimag*sin(wtau[i]);
  	   xx[ie] = tmpimag*cos(wtau[i]) - tmpreal*sin(wtau[i]);
    }
}

void windowData_float(float *xarr,float *yarr,int ndat,float t0, 
	float t1, int *nfinal)
{ /* find data set within a user provided
   * time window.
    INPUT:
      xarr[]  ---  time array
      yarr[]  ---  data array
      t0      ---  beginning of selected time window
      t1      ---  ending of selected time window
      ndat    ---  total number of data in array
   OUTPUT:

      xarr    ---  new x or time array
      yarr    ---  new x or time array
      nfinal    ---  total number of selected data
*/
   int i, j;

   *nfinal=0;
   j=0;
   for(i=0; i<ndat; i++){
	if(t0 <= xarr[i] && t1 >= xarr[i]){
		if(i!=j) xarr[j]=xarr[i];
		j++;
	}
   }
   *nfinal=j;
}

void getminmax_float(float *xarr, float *yarr, float t0, float t1, 
	int ndat, float *ymin, float *ymax, float *x0, 
	int *indymin, int *indymax, int *nfinal)
{ /* find min and max in given data set within a user provided
   * time window.
    INPUT:
      xarr[]  ---  time array
      yarr[]  ---  data array
      t0      ---  beginning of selected time window
      t1      ---  ending of selected time window
      ndat    ---  total number of data in array
   OUTPUT:

      ymin      ---  minimum y value
      ymax      ---  maximum y value
      x0        ---  beginning of selected time window
      indymin   ---  index of x where y min occurs
      indymax   ---  index of x where y max occurs
      nfinal    ---  total number of selected data
*/
   int i, j;

   *indymax = -999;
   *indymin = -999;
   *ymin = 1.0e35;
   *ymax = -1.0e35;
   *nfinal=0;
   *x0=-999.0;
   j=0;
   for(i=0; i<ndat; i++){
	if(t0 <= xarr[i] && t1 >= xarr[i]){
		if(j==0) *x0=xarr[i];
		if(*ymin > yarr[i]) {
			*ymin=yarr[i];
			*indymin=j;
		}
		if(*ymax < yarr[i]) {
			*ymax=yarr[i];
			*indymax=j;
		}
		j++;
	}
   }
   *nfinal=j;
}

float find_absolutemax(float xmin, float xmax)
{
   float absmax;
   if(fabs(xmin)>fabs(xmax)){
	absmax=fabs(xmin);
   }else{
	absmax=fabs(xmax);
   }
   return absmax;
}


void simple_xcorr(float *xx1, float *yy1, int n1, float *xx2, float *yy2, 
 int n2, float xmin, float xmax, int winmin, int winmax, float *xcorr,
 float *xpeak, float *peakval)
{ /* use correlation window to find a rough correlation function between
     synthetic and data */
  float corr_win[10000], peak0;
  int indpeak0;
  int i, j, k, indwin0, indwin1;
  int icount;
  float dt, cross, tempy;


 /* computes simple discrete correlation function */
	printf("xin = %f, xmax=%f\n", xmin, xmax);
	printf("winmin = %f, winmax=%f\n", winmin, winmax);
	printf("tt0 = %f, tt1=%f\n", xx1[0], xx1[n1]);
	if(fabs(xx1[2]-xx1[1]-xx2[2]+xx2[1])>0.00001) {
		printf("sample rate not the same in correlation!!\n");
		exit(-1);
	}
	j = 0;
	for(i=0; i<n1; i++){
		if(xx1[i]>=xmin && xx1[i]<=xmax){
			corr_win[j] = yy1[i];
			j++;
		}
	}
	icount=0;
	for(i=0; i<n2; i++){
/* assume monochromatic grow of x data */
		if(xx2[i]>=winmin && icount==0){
			indwin0=i;
			icount++;
		}
		if(xx2[i]>winmax){
			indwin1=i-1;
			icount++;
		}	
		if(icount==2) break;
	}
	printf("index for correlation window: min=%d, max=%d\n", indwin0, indwin1);
	/* synthetic correlation function with phase window */
        for(i=indwin0; i<=indwin1; i++) {
     	    cross = 0.0;
	    xcorr[i] = 0.0;
     	    for(k=0; k<j-1; k++) {
	       if((k+i)<indwin1) tempy=yy2[k+i];
	       else tempy=0.0;
     	       cross=cross+corr_win[k]*tempy;
     	    }
     	    xcorr[i] = cross;
     	}
	peak0=0.0;
	indpeak0=-999;
	for(i=indwin0; i<=indwin1; i++) {
		if(peak0<fabs(xcorr[i])) {
			*xpeak=xx2[i]-xx2[indwin0];
			peak0=fabs(xcorr[i]);
			*peakval=xcorr[i];
		}	
	}
	printf("peak found %f sec after initial window, value =%f\n", *xpeak, *peakval);
}

	void Initialize_f(float *arr, int n) {
	   int i;

	   for(i=0; i<n; i++){
	         arr[i]=0.0;	   
           }
      	}
