/*
 *  This library contains useful subroutines from the numerical recipes.
 *  Most of the programs are based on NR, but with some modifications,
 * 
 */

#include <stdio.h>
#include <math.h>
#include <string.h>

void indexx(n,arrin,indx)
/* this standard C subroutine allows me to arrange seismograms
 * in increasing order of epicentral distances.
 */
int n,indx[];
float arrin[];
{
	int l,j,ir,indxt,i;
	float q;

	for (j=1;j<=n;j++) indx[j]=j;
	l=(n >> 1) + 1;
	ir=n;
	for (;;) {
		if (l > 1)
			q=arrin[(indxt=indx[--l])];
		else {
			q=arrin[(indxt=indx[ir])];
			indx[ir]=indx[1];
			if (--ir == 1) {
				indx[1]=indxt;
				return;
			}
		}
		i=l;
		j=l << 1;
		while (j <= ir) {
			if (j < ir && arrin[indx[j]] < arrin[indx[j+1]]) j++;
			if (q < arrin[indx[j]]) {
				indx[i]=indx[j];
				j += (i=j);
			}
			else j=ir+1;
		}
		indx[i]=indxt;
	}
}

/* the following is a subroutine which finds the
 * statistical parameters such as standard deviation
 * from the data.  It is used for bootstraping result
 */
void moment(data,n,ave,adev,sdev,svar,skew,curt)
int n;
float data[],*ave,*adev,*sdev,*svar,*skew,*curt;
{
        int j;
        float s,p;
                                                                                
        if (n <= 1) printf("n must be at least 2 in MOMENT");
        s=0.0;
        for (j=1;j<=n;j++) s += data[j];
        *ave=s/n;
        *adev=(*svar)=(*skew)=(*curt)=0.0;
        for (j=1;j<=n;j++) {
                *adev += fabs(s=data[j]-(*ave));
                *svar += (p=s*s);
                *skew += (p *= s);
                *curt += (p *= s);
        }
        *adev /= n;
        *svar /= (n-1);
        *sdev=sqrt(*svar);
        if (*svar) {
                *skew /= (n*(*svar)*(*sdev));
                *curt=(*curt)/(n*(*svar)*(*svar))-3.0;
        } else printf("No skew/kurtosis when variance = 0 (in MOMENT)");
}

