typedef struct DECKPREM{
    double *r;        /* values at every radius                        */
    double *rho;
    double *vpv;
    double *vsv;
    double *qshear;
    double *qkappa;
    double *vph;
    double *vsh;
    double *eta;
    double **u;
    double **ui;
    double radius;
    double rmoho;
    double *rtop;     /* for the value at top and bottom of each region */
    double *rbot;     /* the region is separated by discontinuities     */
    double *uptop;    /* or the source                                  */
    double *ustop;
    double *upbot;
    double *usbot;
    double **delt;
    double **ttime;
    int    **nps;
    int    *itop;
    int    *ibot;
    int    tlay;      /* total number of layers */
    int    icb;       /* layer number for the inner-outer core boundary      */
    int    cmb;       /* layer number for the core-mantle boundary           */
    int    isor;      /* layer number for the source                         */
    int    numreg;    /* number  of  regions  separated  by  discontinuities */
    int    *icn;      /* to save the layer numbers for the discontinuities   */
}DECKPREM ;

typedef struct RAY{
    int  reflay;		   /* top reflection layer, could be earth's surface
                              220, 400, or 670 */
    int  reconv;           /* conversion layer, could be 220, 400 or 670  */
    int  nleg;             /* number of ray legs in the mantle and core   */
    int  raytype[132];     /* ray type                                    */
    int  raydir[132];      /* ray direction                               */
    int  rayregion[132];   /* ray region                                  */
    int  raybeglay[132];   /* ray beginning layer                         */
    int  rayendlay[132];   /* ray ending layer                            */
    int  itube;            /* number of ray tubes                         */
    int  nb;               /* number of ray branches                      */
    int  *lastlay;         /* deepest layer of this branch ray can get    */
    double **para;         /* ray parameters  [i][0]--pmax,[i][1]--pmax   */
    double *theta;
    double *radius;
    double *fac;
    double *vp;
    double *vs;
    double *tmp;
    int  **layer;
    int  *nlast;
    char rname[132];
} RAY;

#define MANTLEREG    1
#define OUTCOREREG   2
#define INNERCOREREG 3

#define PWAVE   1
#define SWAVE   2
#define DOWN    3
#define UP      4
#define REFLECT 5

#define TURNING 1
#define THROUGH 2

#define JMAX  400    /* max number of bisections in FindPara */
#define NMAX  4000
#define XACC  0.0001 /* accuracy */


#define  OTHERKER 0
#define  MUKER_P	1
#define  KAPPAKER_P 2


void GetSphModel(DECKPREM *prem);
void SplitModel(DECKPREM *prem, double dep);
void GetRayName(RAY *ray, DECKPREM prem, char *);
void GetRayPara(RAY *ray, DECKPREM prem, int ifkernel);
void RayTube(RAY *ray,DECKPREM prem,int lay0,int lay1,int raytype,int raychar, int *itube);
void errormessage(int type);
void GetRayBranch(RAY *ray, DECKPREM prem);
void Para(DECKPREM prem, int region, int wavetype, double *pmin, double *pmax);
void IntgrateDelTim(RAY ray,double para, DECKPREM prem, double *time,double *delta, int *ierr);
void IntgrateDel(double para, DECKPREM prem, double *delta, int *ierr);
void FindPara( DECKPREM prem, double p1,double x1,double p2,double x2,double xacc, double dist, double *p, double *delta, int *ierr);
void CalThetaR(double para, DECKPREM prem,RAY *ray,int *ierr);
double CalDtDh(DECKPREM prem, RAY ray, double para);
void VertTime(DECKPREM prem, RAY ray, double *ttime, double *delta);
void PrintPrem(DECKPREM prem);
void CheckDepth(DECKPREM prem, double *);
void RayPath(double para, DECKPREM prem, RAY ray, int *ierr, int *nn, int diff, double dd, int region);
void KINint(int i1,int i2, RAY ray, double **sum, int kmax, int lmax, int region, int ker);
void KINintMoho(int i1, int i2, RAY ray, double **sum,int lmax);
void drspln(int i1, int i2, double *x, double *y, double **q);
double drsple(int i1, int i2, double *x, double *y, double **q, double s);
void radfun(double r, double *f, int region, int kmax);
void DChebyFun(double x, int kmax, double *f);
void ChooseTabRad(int *kmax, int *lmax, int *region, int *choice);
void calc_premtt_kernel(double dep,char *rayname,double dmin, int ifkernel, int *iferr, double *premtt);

