#include <stdio.h>
#include <math.h>
#include "wsc.h"
#include "const.h"
#define max(a,b)           ((a>b) ? a : b)
#define min(a,b)           ((a<b) ? a : b)

void read_bs_ipm_param(int fd, int iformat);
void IPMStats(float *ata, float *atd,  int numatd);

void read_ipm_(float *atd, float *ata, int *numatd, char *comment)
{
int n_read;
int iformat;
int fd, numata,ntopo;
char buf[132],filenm[132];

   printf("IPM file:\n");
   GetLine(buf,132);
   sscanf(buf,"%s",filenm); 

   fd=SysOpen(filenm,0);

   n_read=read(fd,&iformat,sizeof(int));
/*  n_read=read(fd,comment,120); */	/* 120 is the number of chars for the comments */

   if (iformat==IPM_GEN){
	/* do not need to do anything */
   }else{
	read_bs_ipm_param(fd,iformat);	/* need to skip some parameters */
   }
   read(fd,numatd,sizeof(int));
   if(iformat==IPM_BSP_R_TOPO){
/* waveform ATA or surface wave only */
	read(fd,&ntopo,sizeof(int));	/* number of topo element */
	printf("*****waveform or surfacewave ATA*****\n");
	printf("number of topography parameters=%d\n", ntopo);
   }

   printf("......reading ata......\n");

   numata=(*numatd*(*numatd+1))/2;
   n_read=read(fd,ata,numata*sizeof(float));
   if(n_read!=(numata*sizeof(float))){
      printf("Error in read_ipm_, unable to read ata\n");
      exit(-1);
   }

   printf("......reading atd......\n");

   n_read=read(fd,atd,*numatd*sizeof(float));
   if(n_read!=(*numatd*sizeof(float))){
     printf("Error in read_ipm_, unable to read atd\n");
     exit(-1);
   }
   IPMStats(ata,atd,*numatd);
   close(fd);
}

void read_bs_ipm_param(int fd, int iformat)
{
int i,numrknt,numgpt;
float theta_phi[2];
float *rknt;

     printf("B-spline model ipm\n");
     read(fd,&numgpt,sizeof(int));	/* number of points on sphere */

     for(i=0;i<numgpt;i++){		/* coordinates */
         read(fd,theta_phi,2*sizeof(float));
     }
     read(fd,&numrknt,sizeof(int));	/* number of radial knots */
     if(numrknt>1){
        if(iformat==IPM_BSP_R)
		printf("Radial knots described as radius\n");
        if(iformat==IPM_BSP_D)
		printf("Radial knots described as depths\n");
        rknt=(float *)malloc(numrknt*sizeof(float));
        read(fd,rknt,numrknt*sizeof(float));
        free(rknt);
     }
}


void IPMStats(float *ata, float *atd,  int numatd)
{
int i,k, numata;
float sum=(float)0,sum2=(float)0,atak,atdk;
float atamin,atamax,atavg,atavar,atdvg,atdvar;
float atdmin,atdmax;

  atamin=(float)1.0e38;   
  atamax=-atamin;

  for(k=i=0;i<numatd;i++){
      atak=ata[k];
      sum+=atak;
      sum2+=atak*atak;
      atamin=min(atamin,atak);
      atamax=max(atamax,atak);
      k+=i+2;
  }

  atavg=sum/(float)numatd;
  sum2/=(float)numatd;
  atavar=sum2-atavg*atavg;

  atdmin=(float)1.0e38;
  atdmax=-atdmin;

  sum=sum2=(float)0.0;
  for(i=0;i<numatd;i++){
    atdk=atd[i];
    sum+=atdk;
    sum2+=atdk*atdk;
    atdmin=min(atdmin,atdk);
    atdmax=max(atdmax,atdk);
  }

  atdvg=sum/(float)numatd;
  sum2/=(float)numatd;
  atdvar=sum2-atdvg*atdvg;

  printf(" numata=%d\n",(numatd*(numatd+1))/2);
  printf(" numatd=%d\n",numatd);
  
  printf("min diagonal element of ata =%g\n",atamin);
  printf("max diagonal element of ata =%g\n",atamax);
  printf("average of diagonal element of ata=%g\n",atavg);
  printf("variance of diagonal element of ata=%g\n",atavar);

  printf("min diagonal element of atd =%g\n",atdmin);
  printf("max diagonal element of atd =%g\n",atdmax);
  printf("average of all elements of atd=%g\n",atdvg);
  printf("variance of all elements of atd=%g\n",atdvar);
   
}
