//typedef	float		real;		//  in single precision
typedef	double	real;		//  in double precision
#define	POLY4(a0,a1,a2,a3,a4,a5,x)	\
           ((a0)+(x)*((a1)+(x)*((a2)+(x)*((a3)+(a4)*(x)))))

const int	maxleg	=20;		// max number of up and downs for a ray
const real	RADIUS	=6371.0;
const real	DEPMOHO	=24.37109;
const real	DEP220	=220.0;
const real	DEP400	=400.0;
const real      DEP520  =520.0;
const real	DEP670	=670.0;
const real	DEPCMB	=2891.04;
const real	DEPICB	=5149.51;
const int	DOWN	=1;
const int	UP	=2;
const real	PI	=3.14159265;
const real	RAD	=0.017453292519943;	// (pi/180.0)
const real	XACC	=0.0001;		// accuracy
const int	JMAX	=400;			// max # of bisections in FindPara
const real	HPI	=0.5*PI;		// 0.5*pi
const real	TWOPI	=2.0*PI;		// 0.5*pi
const int       MAXCAP  = 500;                           // max number of caps
const real      GEOCO   = 0.993277;                       // geocentric convert
const real      REPRAD  = 57.29578;
// the following are kernels for calculating a surface reflection   
// eta for S400S, SH wave, PREM velocities
// -2/r in Morrelli and Dziewonski 
const real      ETA670BOT = (RADIUS-DEP670)/5.94513;  	// opposite to M&D 87
const real      ETA670TOP = (RADIUS-DEP670)/5.57013;
const real      ETA400BOT = (RADIUS-DEP400)/4.93252;
const real      ETA400TOP = (RADIUS-DEP400)/4.76989;
const real      FACT670   = 2.0/(RADIUS-DEP670);
const real      FACT400   = 2.0/(RADIUS-DEP400); 

// const int resrcl=36;	    // summary residual file record length
const int resrcl = 72;      // Xianfeng's residual file record length
const int MAXLAYER = 10;

enum Boolean {FALSE,TRUE};
enum INOUT {IN,OUT};
		
class PolyPrem{			// polynomial PREM (one polynomial for each region)
   friend class DeckPrem;
	public:
	   void ReadPolyPrem();
	   void PrintPolyPrem();
	   void InitPolyPrem(int);
	   ~PolyPrem();
	private:
	   int numreg;			// number of regions defined in prem
	   real radius;			// Earth's radius
	   real *rtop,*rbot;		// radius at top and bot of each region
	   real *rhopol[5],*qspol[5],*qkpol[5],*etapol[5];
	   real *vpvpol[5],*vsvpol[5],*vphpol[5],*vshpol[5];
};

enum Phase {P=0,S,PP,SS,PcP,ScS,PKPab,PKPbc,PKPdf,	// always let P=0, S=1
		PPP,SSS,PS,SP,pP,sP,sS,pS,P670S,S670P,
		P400P, P670P, S400S, S670S, SKS,SKKS,SKKKS,
		LAST};

class DeckPrem{				// PREM in layers
      friend class PHASE;
      friend class P_wave;
      friend class S_wave;
      friend class PP_wave;
      friend class SS_wave;
      friend class PcP_wave;
      friend class ScS_wave;
      friend class PKPab_wave;
      friend class PKPbc_wave;
      friend class PKPdf_wave;
      friend class PPP_wave;
      friend class SSS_wave;
      friend class PS_wave;
      friend class SP_wave;
      friend class pP_wave;
      friend class sP_wave;
      friend class sS_wave;
      friend class pS_wave;
      friend class P670S_wave;
      friend class S670P_wave;
      friend class P400P_wave;
      friend class P670P_wave;
      friend class S400S_wave;
      friend class S670S_wave;
      friend class SKS_wave;
      friend class SKKS_wave;
      friend class SKKKS_wave;
      friend void TimDelContr(class PHASE&,class DeckPrem&);
      friend void DelContr(class PHASE&,class DeckPrem&);
      friend void TimDelTotal(class PHASE&,class DeckPrem&, int, real);
      friend void DelTotal(class PHASE&,class DeckPrem&, int, real);
      public:
	   void PrintDeckPrem();
	   void InitDeckPrem(class PolyPrem&);
	   void DeckPremUpUs(char);
	   int  GetNlay() const { return numlay;}
	   int  GetNreg() const { return numreg;}
	   real GetRadius() const { return rr[numlay-1];}
	   real GetMoho() const { return rr[moho];}
	   real GetCMB() const { return rr[icmb];}
	   real Get670() const { return rr[i670];}
	   real GetIOB() const { return rr[iiob];}
	private:
	   int numreg;		// number of regions defined in prem
	   int numlay;		// number of layers defined in deck prem
	   int *itop,*ibot,*nlay;
	   int umbot,umtop,lmbot,lmtop,icbot,ictop,ocbot,octop;
	   int abovetz, tztop;        // transition zone at 400 km
	   int moho,i670,i400,i220,icmb,iiob;
	   real *rr,*vpv,*vsv,*vph,*vsh,*v[2],*u[2];
//	   real *rho,*qs,*qk,*eta;		// not used
	   real *fac[2];				// precalculated factors for P and S
};

class PHASE{
	friend void TimDelContr(class PHASE&,class DeckPrem&);
	friend void DelContr(class PHASE&,class DeckPrem&);
	friend void TimDelTotal(class PHASE&,class DeckPrem&, int, real);
	friend void DelTotal(class PHASE&,class DeckPrem&, int, real);
	public:
	 virtual void init(class DeckPrem&)=0;
         void newarray(class DeckPrem &DPrem, char modeltype);
         void init_eq(class DeckPrem &DPrem, real);
         inline void initarray(class DeckPrem &DPrem);
         inline virtual void SetPara(real);
         real Gettime() const {return ttime;}
         real GetDelta() const {return delta;}
         real GetPmin() const {return pmin;}
         real GetPmax() const {return pmax;}
         real GetTRad(int wave) const { return tr[wave];}
         void CheckPara(real) const;
         real GetSetDtDh(real);
         real GetEqr() const { return eqr;}
         void SetNumPt(int m){ numpt=m;}
         void CalDlength(class BSModel& );
         void CalDlength_split(class BSModel& );
         void CalDlength_multisplit(class BSModel& );
// the following takes care of transimitted wave bounce point
	 void TransBoundContri(class BSModel&, class EqSt&, 
			float *, real, int);
	 void CapTransBoundContri(class BSModel&, class CAPRES&, 
			float *,real, int, int);
//
         int GetMaxpt() const { return maxpt;}
         Boolean IsPhase(Phase);
         Boolean IsPhase(Phase,Phase);
         void CheckSize(int m) const { if(m>=maxpt){cerr <<"# of points exceeds limits\n";
         	  cerr << "increase 'maxpt' in PHASE::newarray\n"; exit(-1);}}
	   int	numpt;			// number of points on the raypath
	   real *rad,*phi;		// coordinate of the ray path
	   real *vel;			// velocity
	   real *rlen;			// ray length
	   real *rfac[4];		// radial factor
	   real *rfac_um[4];		// radial fac for split model UM
	   real *rfac_lm[4];		// radial fac for split model LM
	   real *rfac_mult[MAXLAYER][4];// radial fac for multiple split model
	   int  *krad;
	   int  *krad_um;		// for split model
	   int  *krad_lm;		// for split model
	   int  **krad_mult;		// for multply split model
	protected:
	   int	numseg;			// number of ray segment
	   int	maxpt;			// max number of points allowed for each ray
	   Boolean *integ[2];		// an indicator, whether dt, dd has been cal.
	   Boolean Exist[2];		// indicate whether there is P or S wave
	   real pmin,pmax,pmaxp;
	   real *para;			// ray parameter for different regions
	   real eqr;			// radius at the earthquake
	   int  eqreg,eqlay;		// region and layer numbers
	   real eqv[2],eqdd,eqdt;	// velocity, ddelta and dt integral at source
	   real *dt[2],*dd[2];		// time & delta integration for each layer
	   int *rbeg,*rend;		// begin and end region # for this ray segment
	   int *dir;			// ray travelling direction for each segment
	   int *pors;			// wave type (P or S) for each segment
	   int tlay[2];			// ray turning layer, will not change whence defined
	   int tlay1[2], treg1[2];	// ray turning layer and region, will change for different rays
	   real tr[2],tv[2];		// ray turning radius, and velocity
         real ttime,delta;
	   char* rayname;
};


#define CLASS(wave)	class wave: public PHASE{ \
 public: \
 wave(); \
 void init(class DeckPrem&); }

CLASS(P_wave);
CLASS(S_wave);
CLASS(PP_wave);
CLASS(SS_wave);
CLASS(PcP_wave);
CLASS(ScS_wave);
CLASS(PKPab_wave);
CLASS(PKPbc_wave);
CLASS(PKPdf_wave);
CLASS(PPP_wave);
CLASS(SSS_wave);
CLASS(PS_wave);
CLASS(SP_wave);
CLASS(pP_wave);
CLASS(sP_wave);
CLASS(sS_wave);
CLASS(pS_wave);
CLASS(P670S_wave);
// CLASS(S670P_wave);
CLASS(P400P_wave);
CLASS(P670P_wave);
CLASS(S400S_wave);
CLASS(S670S_wave);
CLASS(SKS_wave);
CLASS(SKKS_wave);
CLASS(SKKKS_wave);

class S670P_wave: public PHASE{			// need special treatment
         public: S670P_wave();
         inline void init(class DeckPrem&); 
         inline void SetPara(real); 
};

// prototypes of functions

void TimDelContr(class PHASE&,class DeckPrem&);
void DelContr(class PHASE&,class DeckPrem&);
void TimDelTotal(class PHASE&,class DeckPrem&, int, real);
void DelTotal(class PHASE&,class DeckPrem&, int, real);
void FindPara(class PHASE &, DeckPrem &, real, real, real, 
            real, real, const real, real*, int);

Phase ChooseRay();
Phase SetRay(int);
void Contribution(class PHASE&, class BSModel&, class EqSt&, float *);
void Contribution_split(class PHASE&, class BSModel&, class EqSt&, float *);
void Contribution_multisplit(class PHASE&, class BSModel&, class EqSt&, float *);

template <class Type>
inline Type min (Type a, Type b){
   return a<b ? a : b;
}

template <class Type>
inline Type max (Type a, Type b){
   return a>b ? a : b;
}

inline void swap(real *a, real *b)
{
real c;
   c=*a;
   *a=*b;
   *b=c;
}

class Geographic{
friend class Spherical2;
friend class Cartesian;
friend class EqSt;
friend class CAPRES;
public:
  real lat,lon;
};

class Cartesian{
friend class Spherical2;
friend class Geographic;
friend class TransMatrix;
friend class EqSt;
friend class CAPRES;
public:
   void Normalize();
   void ConvFromSph2(class Spherical2&);
   Cartesian OutProdWith(class Cartesian&);
private:
   real x,y,z;
};

class Spherical2{
friend class Cartesian;
friend class Geographic;
friend class TransMatrix;
friend class EqSt;
friend class CAPRES;
public:
  Spherical2 ForwardTrans(real *);
  Spherical2 InverseTrans(real *);
  void ConvFromGeo(Geographic);
  real theta,phi;
};

class TransMatrix{
friend class Spherical2;
friend class EqSt;
friend class CAPRES;
public:
  void SetFromVec(class Cartesian&, class Cartesian&, class Cartesian&);
  real matrix[9];
};

class EqSt{			// earthquake and station
friend void SaveArow(int,class EqSt &,float *,short *,int);
friend void Save_arow_pred(int,class EqSt &,float *,short *,int);
friend void SaveArow_tttopo(int,class EqSt &,float *,short *,int, int);
public:
   void CalEqStDist();
   real GetDist() const { return dist;}
   real GetEqdep() const { return eqdep;}
   real GetTrueRes() const { return res1;}
   void ReadEq(class SUMRES&);
   void ReadStRes(class SUMRES &);
   void CalTrans4();
   TransMatrix GetTrans4() const { return trans;}
   void ForwardTrans4(const int &);
   void InverseTrans4(const int &);
   void InvTransRPhi(const real*, const real*, const int&);
   void Init(int);
   void SavePredRes(real res){ res2=res;}
   void SavePMtime(real res) {time = res;}

   int  numpt;
   real *r,*theta,*phi;	// to save the raypath between eq and st
   real *dl;		// ray path lengths
private:
   real dist;		// distance between eq and st (in radian)
   real eqdep;		// earthquake depth
   real res1,res2;	// observed and predicted residuals
   real time;		// Prem predicted travel time
   Geographic geq,gst;	// geographic location of eq and st
   Spherical2 seq,sst;	// spherical coordinate of eq and station
   Cartesian ceq,cst;	// eq and st in Cartesian coordinates
   TransMatrix trans;	// transformation matrix
   Spherical2 eqnew,stnew;
};
const int	ST		=1;
const int	EQ		=2;
const int	EQST		=3;

inline void PHASE::initarray(class DeckPrem &DPrem)
{
int i,k,numlay=DPrem.numlay,numreg=DPrem.numreg;

  for(k=P;k<=S;k++){
    if(Exist[k]){			// if there is P/S wave component
       for(i=0;i<numlay;i++)dt[k][i]=dd[k][i]=0.0;
       for(i=0;i<numreg;i++)integ[k][i]=FALSE;
    }
  }
  ttime=delta=0.0;
}

inline void S670P_wave::SetPara(real para1)
{
real p1=pmaxp<pmax ? pmaxp : pmax;

    for(int i=0;i<numseg-1;i++)para[i]=para1;
    if(para1>=pmin && para1<=p1){		// normal ray
       para[numseg-1]=para1;
    }else{						// scattered
       para[numseg-1]= p1;
    }
} 

inline void PHASE::SetPara(real para1)
{

    for(int i=0;i<numseg;i++)para[i]=para1;
}


enum MDCHOICE{MODALL,KNOTS,REGIONAL,SPLIT,SPLIT_REGIONAL, MULTISPLIT};
enum REGION{WHOLEMANTLE,UPPERMANTLE,LOWERMANTLE,UPLOWMANTLE};
class Model{
	public:
        virtual void ReadModel(int, char *)=0;
	virtual void ReadModel_split(int, char *)=0;
	virtual void ReadModel_multisplit(int, char *)=0;
        int  GetRegion() const { return region;}
        int  GetMaxK() const { return maxk;}
        int  GetMaxK_um() const { return maxk_um;}
        int  GetMaxK_lm() const { return maxk_lm;}
	protected:
	  int maxk;		// order of radial functions or # B-splines
	  int maxk_um, maxk_lm;	// Split Model, radial B-splines of UM, LM
	  int region;		// the region for which model is defined
	  real rtop,rmid,rbom;
	  real *coeff;		// model coefficients
};

class BSModel: public Model{
      friend void contrinution(BSModel &bsmodel,float *rr,float *theta,float *vel,int &npts,float *transmat);  // temp
      friend void Contribution(class PHASE&, class BSModel&, class EqSt&,float *);
      friend void Contribution_split(class PHASE&, class BSModel&,
					class EqSt&,float *);
      friend void Contribution_multisplit(class PHASE&, class BSModel&, 
					class EqSt&,float *);
      friend void CapContribute(class PHASE& , class BSModel&,
                    class CAPRES &, float *arow);
      friend void CapContribute_split(class PHASE& , class BSModel&,
                    class CAPRES &, float *arow);
      friend PHASE;
      friend real  SumArow(BSModel &l,float *,short *,int &);
      friend real  SumArow_topo(BSModel &l,float *,short *,int &, int &);
      friend void  FindSurfBspline(BSModel &, real, real, real, short *, real *, int &);
      public:
        void ReadModel(int choice=MODALL,char *flname="sphbspline.dat");
        void ReadModel_split(int choice=MODALL,char *flname="s12wm13.XBS");
        void ReadModel_multisplit(int choice=MODALL,char *flname="s12wm13_mutisplit.XBS");
	void Read1stLine(fstream &inFile, int choice);
        void InitKnots(int);
	void InitKnots_split(int);
        void CalAvgDist();
        real GetAvgDist() const { return avgdist;}
        real DistBt2(const int&, const int&);
        int  GetHPts() const { return gpt;}
	int  GetNumSplit() const {return nsplit_lm+nsplit_um;}
      private:
        int gpt;	// number of horizontal grid points
        real *theta,*phi;
        real *rknt;	// knots expressed as depths
	real *rknt_lm;	// for split B-spline model
	real *rknt_um;	// for split B-spline model
	real **rknt_mult; // for multiply split B-spline model
        real avgdist;	// average distance between grid points
        int  *ajknts;	// to save adjacent points
// the following are useful for multiply split model
	int  nsplit_um;
	int  nsplit_lm;
	int  nbsperlayer[MAXLAYER];
};


#define between(x,y,z)     (((x)<=(y) && (z)>(y) )|| ((z)<(y) && (x)>=(y)) )

void RadContrib(class PHASE &,real*,real*,real*,int*,int,int,Spherical2 &, real &, real *, int, real *);
void RadContrib_split(class PHASE &,real*,real*,real*,int*,int,int,Spherical2 &, real &, real *, int, real *, int uplowopt);
void RadContrib_multisplit(int ilayer, class PHASE &,real*,real*,real*,int*,int,int,Spherical2 &, real &, real *, int, real *);
real LinearInt(real &,real &,real &,real &,real &);
int GetReg(real,real *,int);
real fdelta(real&, real&);

void BsplineFun(int, real, real *, int, real *);
int SysOpen(char *,int);

#define DIST(theta,phi,phi1)	acos(sin(theta)*cos((phi)-(phi1)))
#define PERMS 0666    /* RW for owner, group, others */
void SaveArow(int,class EqSt &,float *,short *,int);
void Save_arow_pred(int,class EqSt &,float *,short *,int);
void SaveArow_tttopo(int,class EqSt &,float *,short *,int, int);
void SaveArow_tttopo(int,class EqSt &,float *,short *,int, int);

void SaveModelPara(int &,int &,int &, int &);


class  SUMRES{  // class of ISC or Xianfeng's data
friend class EqSt;
  public:
       void   Init(int);
void	OpenResFile();
	void SeektoRec(long rec);
	real GetDelta() const {return delta;}
	int TotalNumRes();
	int ReadSumRes();
	short  *indsurf_t;  // transmitted wave
	real *surfbs_t;
	int  nbsp_t;
	
  private:
	int fd;  	   	// input file descriptor
	char	stname[5];	// station name
	float slat;		// summary station latitude (in degree)
	float slon;		// summary station longitude (in degree)
	float stelv;	// station elevation (in km)
	char	eqname[9];	// earthquake name
	float elat;		// summary earthquake latitude (in degree)
	float elon;		// longitude
	float edep;		// depth
	float res;		// observed residual
	float het;		// heterogeneity correction
	float var;		// variance, see sumres.f & rewrite_sumres.f
	float sum;		// total weight
	float ttime;	// total travel time
	float elipc;	// ellipcity correction
	float crustcor;	// crustal correction
	float evcor;	// station elevation correction
	float delta;	// epicentral distance
	float coric;	// innor core correction
	int	ig;		// grade
};

class CAPRES {  // class of SS precursor data
   public:
       void Init(int);
       void ReadCapRes(FILE *);
       void ReadTopoRes();  // reads in discontinuity topography residuals
       void SurfKer(real,real, real,real &);  // convert times to depth
       friend void GetCapinfo(real readbuf[], class CAPRES &);
       friend void LocateCap(real, real, class CAPRES&, real &, real &, real &);
       friend void ConvEq(class CAPRES &);
       friend void ConvSt(class CAPRES &);
       friend void CapSaveArow(int,class CAPRES &,float *,short *,int,int);
       void   CalTrans5();
       int  fd;
       int  ncap;
       int  ntopo;
       int  ntrace;
       int  indtopo[MAXCAP];
       int  npercap[MAXCAP];
       int  whichphase;   // 0 for S410S, 1 for S660S, 2 for joint
       real elat;
       real elon;
       real slat;
       real slon;
       real dist;
       real bplat;
       real bplon;
       real capsize;
       real PMtime;
       real topores;
       real dep;
       real sfkernel;
       real caplat[MAXCAP];
       real caplon[MAXCAP];
       real restopo[MAXCAP];
       real sdvtopo[MAXCAP];
       real hettopo;
       short  *indsurf_r;  // contributing b-spline indices
       real *surfbs_r;     // contributing b-spline coefficients
       int  nbsp_r;        // number of contributing b-splines
       short  *indsurf_t;  // transmitted wave
       real *surfbs_t;
       int  nbsp_t;

       void CalTrans4();
       TransMatrix GetTrans4() const { return trans;}
       void ForwardTrans4(const int &);
       void InverseTrans4(const int &);
       void InvTransRPhi(const real*, const real*, const int&);
   private:
       Geographic geq,gst;	// geographic location of eq and st
       Spherical2 seq,sst;	// spherical coordinate of eq and station
       Cartesian ceq,cst;	// eq and st in Cartesian coordinates
       TransMatrix trans;	// transformation matrix
       
};
void ChooseFormat(int &, int, int, int &, int &, int &);
void ChooseDelRange(int &, real &, real &);
void PrintMenu();
void CheckMenu(int argc, char *argv[], char &model, int &begrec, int &endrec, char &modeltype);
void RayInfo(int &, char *);
real CalcBspline(real, real);
void CalcDelta(real, real, real, real, real &);
real CalcDist(real, real, real, real);
real SurfTransKer(int, real);
 
real SumArow(BSModel &l,float *,short *,int &);
real SumArow_topo(BSModel &l,float *,short *,int &, int &);
void Get_MultiReg(int nsplit, int *knot, int *krad, int *ilayer);

