#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "ttimelib.h"
// This program contains routines which integrate along a ray path.
// The model initialization are also accomplished here. The 
// subroutines are,
//	TimDelContr
//	DelContr
//	TimDelTotal
//	DelTotal
//	FindPara
//	DeckPrem::InitDeckPrem
//	PolyPrem::InitPolyPrem
//	PolyPrem::PrintPolyPrem
//	DeckPrem::PrintDeckPrem
//	PolyPrem::ReadPolyPrem
//	PolyPrem::~PolyPrem
//	DeckPrem::DeckPremUpUs
//	PHASE::IsPhase
//	PHASE::IsPhase
//	PHASE::newarray
//	PHASE::init_eq
//	PHASE::GetSetDtDh
//	PHASE::CheckPara
//	PHASE::CalDlength
//

/*
 * calculate the total time and distance contribution for a ray
 * with specific parameter
 */
void TimDelContr(class PHASE &ray, DeckPrem &DPrem)
{
int numseg=ray.numseg,wave,*tlay,*treg;
int i,j,k,kbeg,kend,rbeg,rend,dir;
real *u,*v,*dt,*dd,uk,temp,*rr,*tr,*tv,r0,r1,v0,v1,a,b;
real eqv,up0,up1,ddelta,ddt,*fac,u0,u1,para,para2;
Boolean *intg;

  ray.tlay1[P]=ray.tlay[P];	// ray.tlayp,s--deepest possible layer, do not change
  ray.tlay1[S]=ray.tlay[S];	// ray.tlay1[P],s1--deepest possible layer for current ray
  rr=DPrem.rr;
  
  for(i=0;i<numseg;i++){		// loop for every ray segment
     para=ray.para[i];			// diff ray parameter for special case only
     para2=para*para;			// e.g. scaterred S670P
     wave=ray.pors[i];
     fac=DPrem.fac[wave];		// precalculated factors
     v=DPrem.v[wave];
     u=DPrem.u[wave];
     dt=ray.dt[wave];
     dd=ray.dd[wave];
     tlay=&(ray.tlay1[wave]);
     tr=&(ray.tr[wave]);
     tv=&(ray.tv[wave]);
     intg=ray.integ[wave];

     if(ray.dir[i]==DOWN){		// cal. from top to bottom
        rbeg=ray.rbeg[i];
        rend=ray.rend[i];
     }else{
        rend=ray.rbeg[i];
        rbeg=ray.rend[i];
     }
     
     for(j=rbeg;j>=rend;j--){		// loop for different region
         if(!intg[j]){			// not cal. yet & above turning point
            intg[j]=TRUE;		// mark it, not cal. again
            kbeg=DPrem.itop[j];
            kend=max(DPrem.ibot[j],ray.tlay[wave]);	// cal. until possible layer
            u1=u[kbeg];
            temp=u1*u1-para2;
            if(temp>0.0){				// cal. dt, dd for each layer
               up1=sqrt(temp);
               for(k=kbeg-1;k>=kend;k--){
                   u0=u[k];
                   temp=u0*u0-para2;
                   r0=rr[k];
                   v0=v[k];
                   if(temp>=0.0){
                      up0=sqrt(temp);
                      if(para==0.0){			// for vertical rays
                         if(k!=0){
                            dt[k]=fac[k]*(u1-u0);
                            dd[k]=0.0;
                         }else{
                            dt[0]=fac[0]*u1;
                            dd[0]=0.5*PI;		// phase shift at center
                         }
                      }else{
                         ddelta=(para2+up0*up1)/(u1*u0);
                         if(fabs(ddelta)>1.0)ddelta=1.0;	// prevent round-off error
                         dd[k]=fabs(fac[k])*acos(ddelta);
                         dt[k]=fac[k]*(up1-up0);
                      }
                   }else{
                      r1=rr[k+1];
                      v0=v[k];
                      v1=v[k+1];
                      a=(v1-v0)/(r1-r0);
                      b=v0-a*r0;
                      v0=b/(1.0-a*para);		// velocity at turning point
                      r0=v0*para;			// turning radius
                      b=1.0/(1.0-log(v0/v1)/log(r0/r1));
                      dd[k]=b*acos(para/u1);	// ddelta=acos(para2*v0/(u1*r0));
                      dt[k]=b*up1;
                   }
                   up1=up0;
                   u1 =u0;
                   if(temp<=0.0){	// there is a small bug here
                      *tr=r0;
                      *tv=v0;
                      *tlay=k;
                      break;
                   }
               }   	// end of k loop
            }else if(temp==0.0){		// no integration is necessary
               *tr=rr[k];			// turning radius
               *tv=v[k];
               *tlay=k;				// turning layer
               break;
            }else if(temp<0.0){
               break;
            }
         }			// end of if(!intg[j])
     }			// end of j loop
  }				//end of i loop
  
  for(i=0;i<DPrem.numreg;i++){
      if(ray.tlay1[P]<=DPrem.itop[i] && ray.tlay1[P]>=DPrem.ibot[i])break;
  }
  ray.treg1[P]=i;
  for(i=0;i<DPrem.numreg;i++){
      if(ray.tlay1[S]<=DPrem.itop[i] && ray.tlay1[S]>=DPrem.ibot[i])break;
  }
  ray.treg1[S]=i;  
/*
 * integration for the source layer
 */
  wave=ray.pors[0];
  v=DPrem.v[wave];
  u=DPrem.u[wave];
  eqv=ray.eqv[wave];

  uk=ray.eqr/eqv;
  temp=uk*uk-para2;
  if(temp==0.0 && ray.dir[0]==DOWN){// contribution is in the upper going leg
     ray.eqdd=ray.eqdt=0.0;					// skip
  }else if(temp<0.0){
     cerr << " error in calculating source contribution " << endl;
     exit(-1);
  }else{
     if(ray.dir[0]==UP){		// for upgoing rays such as pP, sP
        r0=ray.eqr;
        v0=eqv;
        u0=r0/v0;
        up0=sqrt(u0*u0-para2);

        r1=rr[ray.eqlay+1];
        v1=v[ray.eqlay+1];
        u1=u[ray.eqlay+1];
        up1=sqrt(u1*u1-para2);
     }else{					// for downgoing rays
        r1=ray.eqr;
        v1=eqv;
        u1=r1/v1;
        up1=sqrt(u1*u1-para2);

        r0=rr[ray.eqlay];
        v0=v[ray.eqlay];
        u0=u[ray.eqlay];
        temp=u0*u0-para2;
        if(temp<0.0){			// if ray turns between source and rr[eqlay]
           a=(v1-v0)/(r1-r0);
           b=v0-a*r0;
           v0=b/(1.0-a*para);		// velocity at turning point
           r0=v0*para;			// turning radius
           u0=r0/v0;			// revised u0
           up0=0.0;
        }else{
           up0=sqrt(temp);
        }
     }
     
     b=1.0-log(v0/v1)/log(r0/r1);
     ddelta=(para2+up0*up1)/(u0*u1);
     ray.eqdd=acos(ddelta)/b;		// contribution from the source layer
     ray.eqdt=(up1-up0)/b;
  }
   
}

//
// This function is very similar to TimDelContri, but only calculates the
// distance contribution from each layer to speed up the whole process
//
void DelContr(class PHASE &ray, DeckPrem &DPrem)
{
int numseg=ray.numseg,wave,*tlay,*treg;
int i,j,k,kbeg,kend,rbeg,rend;
real *u,*v,*dd,uk,temp,*rr,*tr,*tv,r0,r1,v0,v1,a,b;
real eqv,up0,up1,ddelta,ddt,*fac,u0,u1,para,para2;
Boolean *intg;

  ray.tlay1[P]=ray.tlay[P];	// ray.tlayp,s--deepest possible layer, do not change
  ray.tlay1[S]=ray.tlay[S];	// ray.tlay1[P],s1--deepest possible layer for current ray
  rr=DPrem.rr;
  
  for(i=0;i<numseg;i++){		// loop for every ray segment
     para=ray.para[i];			// diff ray parameter for special case only
     para2=para*para;			// e.g. scaterred S670P
     wave=ray.pors[i];
     fac=DPrem.fac[wave];		// precalculated factors
     v=DPrem.v[wave];
     u=DPrem.u[wave];
     dd=ray.dd[wave];
     tlay=&(ray.tlay1[wave]);
     tr=&(ray.tr[wave]);
     tv=&(ray.tv[wave]);
     intg=ray.integ[wave];

     if(ray.dir[i]==DOWN){		// cal. from top to bottom
        rbeg=ray.rbeg[i];
        rend=ray.rend[i];
     }else{
        rend=ray.rbeg[i];
        rbeg=ray.rend[i];
     }
     
     for(j=rbeg;j>=rend;j--){		// loop for different region
         if(!intg[j]){			// not cal. yet & above turning point
            intg[j]=TRUE;		// mark it, not cal. again
            kbeg=DPrem.itop[j];
            kend=max(DPrem.ibot[j],ray.tlay[wave]);	// cal. until possible layer
            u1=u[kbeg];
            temp=u1*u1-para2;
            if(temp>0.0){				// cal. dt, dd for each layer
               up1=sqrt(temp);
               for(k=kbeg-1;k>=kend;k--){
                   u0=u[k];
                   temp=u0*u0-para2;
                   r0=rr[k];
                   v0=v[k];
                   if(temp>=0.0){
                      up0=sqrt(temp);
                      if(para==0.0){			// for vertical rays
                         if(k!=0){
                            dd[k]=0.0;
                         }else{
                            dd[0]=0.5*PI;		// phase shift at center
                         }
                      }else{
                         ddelta=(para2+up0*up1)/(u1*u0);
                         if(fabs(ddelta)>1.0)ddelta=1.0;	// prevent round-off error
                         dd[k]=fabs(fac[k])*acos(ddelta);
                      }
                   }else{
                      r1=rr[k+1];
                      v0=v[k];
                      v1=v[k+1];
                      a=(v1-v0)/(r1-r0);
                      b=v0-a*r0;
                      v0=b/(1.0-a*para);		// velocity at turning point
                      r0=v0*para;			// turning radius
                      b=1.0/(1.0-log(v0/v1)/log(r0/r1));
                      dd[k]=b*acos(para/u1);	// ddelta=acos(para2*v0/(u1*r0));
                   }
                   up1=up0;
                   u1 =u0;
                   if(temp<=0.0){	// there is a small bug here
                      *tr=r0;
                      *tv=v0;
                      *tlay=k;
                      break;
                   }
               }   	// end of k loop
            }else if(temp==0.0){		// no integration is necessary
               *tr=rr[k];			// turning radius
               *tv=v[k];
               *tlay=k;				// turning layer
               break;
            }else if(temp<0.0){
               break;
            }
         }			// end of if(!intg[j])
     }			// end of j loop
  }				//end of i loop
  
  for(i=0;i<DPrem.numreg;i++){
      if(ray.tlay1[P]<=DPrem.itop[i] && ray.tlay1[P]>=DPrem.ibot[i])break;
  }
  ray.treg1[P]=i;
  for(i=0;i<DPrem.numreg;i++){
      if(ray.tlay1[S]<=DPrem.itop[i] && ray.tlay1[S]>=DPrem.ibot[i])break;
  }
  ray.treg1[S]=i;  
/*
 * integration for the sourcce layer
 */
  wave=ray.pors[0];
  v=DPrem.v[wave];
  u=DPrem.u[wave];
  eqv=ray.eqv[wave];

  uk=ray.eqr/eqv;
  temp=uk*uk-para2;
  if(temp==0.0 && ray.dir[0]==DOWN){// contribution is in the upper going leg
     ray.eqdd=0.0;					// skip
  }else if(temp<0.0){
     cerr << " error in calculating source contribution " << endl;
     exit(-1);
  }else{
     if(ray.dir[0]==UP){		// for upgoing rays such as pP, sP
        r0=ray.eqr;
        v0=eqv;
        u0=r0/v0;
        up0=sqrt(u0*u0-para2);

        r1=rr[ray.eqlay+1];
        v1=v[ray.eqlay+1];
        u1=u[ray.eqlay+1];
        up1=sqrt(u1*u1-para2);
     }else{					// for downgoing rays
        r1=ray.eqr;
        v1=eqv;
        u1=r1/v1;
        up1=sqrt(u1*u1-para2);

        r0=rr[ray.eqlay];
        v0=v[ray.eqlay];
        u0=u[ray.eqlay];
        temp=u0*u0-para2;
        if(temp<0.0){			// if ray turns between source and rr[eqlay]
           a=(v1-v0)/(r1-r0);
           b=v0-a*r0;
           v0=b/(1.0-a*para);		// velocity at turning point
           r0=v0*para;			// turning radius
           u0=r0/v0;			// revised u0
           up0=0.0;
        }else{
           up0=sqrt(temp);
        }
     }
     
     b=1.0-log(v0/v1)/log(r0/r1);
     ddelta=(para2+up0*up1)/(u0*u1);
     ray.eqdd=acos(ddelta)/b;		// contribution from the source layer
  }
   
}

void TimDelTotal(class PHASE &ray,DeckPrem &DPrem,int diff, real dseg)
{
int wave,*tlay,*treg,adpt;
int i,j,k,m,itop,ibot,kbeg,kend,rbeg,rend,dir,jinc,kinc;
real *vv,*dt,*dd,*tr,*tv,para,*u,temp,dx,x;
real *ttime,*delta,*rad,*theta,*vel,*rr;
int numlay=DPrem.numlay;
int numseg=ray.numseg;
  
  ttime=&(ray.ttime);
  delta=&(ray.delta);
  rad = ray.rad;
  theta = ray.phi;
  vel   = ray.vel;
  rr=DPrem.rr;
  
  rad[0]=ray.eqr;
  theta[0]=0.0;
  vel[0]=ray.eqv[ray.pors[0]];
  temp=ray.eqr/ray.eqv[ray.pors[0]];
  
  for(m=1,i=0;i<numseg;i++){
    para=ray.para[i];
    wave=ray.pors[i];
    u=DPrem.u[wave];
    vv=DPrem.v[wave];
    dt=ray.dt[wave];
    dd=ray.dd[wave];
    tlay=&(ray.tlay1[wave]);
    treg=&(ray.treg1[wave]);
    tr=&(ray.tr[wave]);
    tv=&(ray.tv[wave]);		// velocity at phase turning point
    rbeg=ray.rbeg[i];
    rend=ray.rend[i];
    dir =ray.dir[i];
    
    kinc=jinc=(dir==DOWN)?-1:1;
     
     for(j=rbeg;;j+=jinc){			// loop for different region
         if(j>=*treg){				// above the turning region
           if(dir==DOWN){
              kbeg=DPrem.itop[j];
              kend=DPrem.ibot[j];
           }else{
              kbeg=DPrem.ibot[j];
              kend=DPrem.itop[j];
           }
           
           for(k=kbeg;;k+=kinc){
             if(i==0 && ((dir==DOWN && k>ray.eqlay) ||	// above  eq for downgoing ray
                         (dir==UP && k<ray.eqlay)) ){		// below  eq for upgoing ray
                // skip
             }else if(i==0 && k==ray.eqlay){
                *ttime+=ray.eqdt;
                *delta+=ray.eqdd;
                rad[m]=rr[k];
                theta[m]=*delta;
                vel[m]=vv[k];
                ray.CheckSize(m++);
             }else{
                if(k>=*tlay){
                   if(dir==UP){
                      theta[m]=*delta;
                   }
                   *ttime+=dt[k];
                   *delta+=dd[k];
                   if(dir==DOWN){
                      theta[m]=*delta;
                   }
                   if(k==*tlay){
                      rad[m]=*tr;
                      vel[m]=*tv;
                      if(diff){
                         adpt=(int)(2.0*dseg/RAD);	// add a point roughly every half a degree
                         dx=dseg/(real)adpt;
                         for(x=dx;x<dseg;x+=dx){
                             theta[m]=x+(*delta);
                             rad[m]=*tr;
                             vel[m]=*tv;
                             ray.CheckSize(m++);
                         }
                         *delta+=dseg;
                         theta[m]=*delta;
                         rad[m]=*tr;
                         vel[m]=*tv;
                         diff=FALSE;
                      }
                   }else{
                      rad[m]=rr[k];
                      vel[m]=vv[k];
                   }
                   ray.CheckSize(m++);
                }
             }
             if(k==kend)break;
           }
         }
         if(j==rend)break;
     }		//end of j loop
  }			//end of i loop
  ray.SetNumPt(m);
}

void DelTotal(class PHASE &ray,DeckPrem &DPrem,int diff, real dseg)
{
int i,j,k,wave,*tlay,*treg,kbeg,kend,rbeg,rend,dir,jinc,kinc;
real *dd,*delta;
int numseg=ray.numseg;
  
  delta=&(ray.delta);
  
  for(i=0;i<numseg;i++){
    wave=ray.pors[i];
    dd=ray.dd[wave];
    tlay=&(ray.tlay1[wave]);
    treg=&(ray.treg1[wave]);
    rbeg=ray.rbeg[i];
    rend=ray.rend[i];
    dir =ray.dir[i];
    
    kinc=jinc=(dir==DOWN)?-1:1;
     
    for(j=rbeg;;j+=jinc){			// loop for different region
         if(j>=*treg){				// above the turning region
           if(dir==DOWN){
              kbeg=DPrem.itop[j];
              kend=DPrem.ibot[j];
           }else{
              kbeg=DPrem.ibot[j];
              kend=DPrem.itop[j];
           }
           
           for(k=kbeg;;k+=kinc){
             if(i==0 && ((dir==DOWN && k>ray.eqlay) ||	// above  eq for downgoing ray
                         (dir==UP && k<ray.eqlay)) ){		// below  eq for upgoing ray
                // skip
             }else if(i==0 && k==ray.eqlay){			// contribution from source layer
                *delta+=ray.eqdd;
             }else{
                if(k>=*tlay){
                   *delta+=dd[k];
                   if(k==*tlay && diff){				// contribution from diff. part
                      *delta+=dseg;
                       diff=FALSE;
                   }
                }
             }
             if(k==kend)break;
           }
         }
         if(j==rend)break;
    }			//end of j loop
  }			//end of i loop
  
}


void FindPara(class PHASE &ray, DeckPrem &DPrem, real dist, real p1,
real x1, real p2, real x2, const real xacc, real *p, int diff)
{

real dep=DPrem.GetRadius()-ray.GetEqr();
real f,fmid,rtb,xmid,dx;

    if(fabs(dist-x1)<0.0001){
       *p=p1;
        ray.initarray(DPrem);
        ray.SetPara(*p);
        TimDelContr(ray,DPrem);
        TimDelTotal(ray,DPrem,FALSE,0.0);
        return;
    }else if(fabs(dist-x2)<0.0001){
       *p=p2;
        ray.initarray(DPrem);
        ray.SetPara(*p);
        TimDelContr(ray,DPrem);
        TimDelTotal(ray,DPrem,FALSE,0.0);
        return;
    }
    if(diff){
       *p=p2;
        ray.initarray(DPrem);
        ray.SetPara(*p);
        TimDelContr(ray,DPrem);
        TimDelTotal(ray,DPrem,TRUE,dist-x2);
        return;
    }

    x1-=dist;
    x2-=dist;
    f=x1;
    fmid=x2;
    
    if(f*fmid>=0.0){
       cerr << " error 12 \n";
       exit(-1);
    }
    rtb = f<0.0 ? (dx=p2-p1,p1) : (dx=p1-p2,p2);
    
    for(int j=1;j<=JMAX;j++){
        dx*=0.5;
        xmid=rtb+dx;
        ray.initarray(DPrem);		// cal max delta
        ray.SetPara(xmid);
        DelContr(ray,DPrem);
        DelTotal(ray,DPrem,FALSE,0.0);
        fmid=ray.GetDelta();
        fmid-=dist;
        if(fmid<0.0)rtb=xmid;
        if(fabs(dx)<xacc || fmid==0.0){
           *p=xmid;
            ray.initarray(DPrem);		// cal max delta
            ray.SetPara(xmid);
            TimDelContr(ray,DPrem);
            TimDelTotal(ray,DPrem,FALSE,0.0);
            return;
        }
    }
    cerr << " error 13 \n";
    exit(-1);
}

void DeckPrem::InitDeckPrem(PolyPrem &PPrem)
{
fstream inFile;
int i,j,k;
real dr,rn,rlay,radius;
real rhopol[5],vpvpol[5],vsvpol[5],vphpol[5],vshpol[5];

 numreg=PPrem.numreg;		// number of regions in PREM
 radius=PPrem.radius;		// Earth's radius

 itop = new int[numreg];	// array for layer number at top of each region
 ibot = new int[numreg];	// at the bottom
 nlay = new int[numreg];	// total number of layers in DeckPrem

 inFile.open("/scratch/troika23/gu/dbase/premlayer.dat",ios::in);	// open layer file for read
 
 if(!inFile) { 					// open failed?
     cerr << "Cannot open ``premlayer.dat'' for input\n";
     exit(-1);
 }
 
 for(numlay=i=0;i<numreg;i++){	// read in the number of layers in each region
     inFile >> nlay[i];
     numlay+=nlay[i];			// calculate total number of layers
 }

 inFile.close();
 
 rr  = new real[numlay];			// radius at each layer
// rho = new real[numlay];			// density
// qs  = new real[numlay];
// qk  = new real[numlay];
// eta = new real[numlay];
 vpv = new real[numlay];
 vsv = new real[numlay];
 vph = new real[numlay];
 vsh = new real[numlay];
 u[P]  = new real[numlay];
 u[S]  = new real[numlay];
 fac[P] = new real[numlay];
 fac[S] = new real[numlay];
  
 for(k=0,i=0;i<numreg;i++){
     for(j=0;j<5;j++){
        rhopol[j]=PPrem.rhopol[j][i];	// polynomial coefficients for rho at this region
        vpvpol[j]=PPrem.vpvpol[j][i];
        vsvpol[j]=PPrem.vsvpol[j][i];
        vpvpol[j]=PPrem.vpvpol[j][i];
        vphpol[j]=PPrem.vphpol[j][i];
        vshpol[j]=PPrem.vshpol[j][i];
     }
     dr=(PPrem.rtop[i]-PPrem.rbot[i])/(real)(nlay[i]-1);
     ibot[i]=k;
     rlay=PPrem.rbot[i];
     for(j=0;j<nlay[i];j++,rlay+=dr,k++){		// interplate at different r
         rr[k]=rlay;					// make sure rr agrees with rtop
         if(j==nlay[i]-1){
            rr[k]=PPrem.rtop[i];
            itop[i]=k;
         }
         rn=rlay/radius;
//         rho[k]=POLY4(rhopol[0],rhopol[1],rhopol[2],rhopol[3],rhopol[4],rhopol[5],rn);
         vpv[k]=POLY4(vpvpol[0],vpvpol[1],vpvpol[2],vpvpol[3],vpvpol[4],vpvpol[5],rn);
         vsv[k]=POLY4(vsvpol[0],vsvpol[1],vsvpol[2],vsvpol[3],vsvpol[4],vsvpol[5],rn);
         vph[k]=POLY4(vphpol[0],vphpol[1],vphpol[2],vphpol[3],vphpol[4],vphpol[5],rn);
         vsh[k]=POLY4(vshpol[0],vshpol[1],vshpol[2],vshpol[3],vshpol[4],vshpol[5],rn);
     }
 }

 delete [] nlay;
 
 for(i=0;i<numlay;i++){
     dr=rr[numlay-1]-rr[i];
     if     (fabs(dr-DEP220)<0.01)i220=i;
     else if(fabs(dr-DEP400)<0.01)i400=i;
     else if(fabs(dr-DEP670)<0.01)i670=i;
     else if(fabs(dr-DEPCMB)<0.01)icmb=i;
     else if(fabs(dr-DEPICB)<0.01)iiob=i;
     else if(fabs(dr-DEPMOHO)<0.01)moho=i;
 }
 
 umtop=numreg-1;
 icbot=0;
 
 for(i=0;i<numreg;i++){
     if(ibot[i]==i670)umbot=i;
     if(itop[i]==i670-1)lmtop=i;
     if(ibot[i]==icmb)lmbot=i;
     if(itop[i]==icmb-1)octop=i;
     if(ibot[i]==iiob)ocbot=i;
     if(itop[i]==iiob-1)ictop=i;
     if(ibot[i]==i400)  abovetz=i;
     if(itop[i]==i400-1) tztop=i;
 } 
}

void PolyPrem::InitPolyPrem(int nr)
{
  numreg =  nr;
  rtop = new real[numreg];
  rbot = new real[numreg];
  
  for(int i=0;i<5;i++){
     rhopol[i] = new real[numreg];
     vpvpol[i] = new real[numreg];
     vsvpol[i] = new real[numreg];
     qspol[i]  = new real[numreg];
     qkpol[i]  = new real[numreg];
     vphpol[i] = new real[numreg];
     vshpol[i] = new real[numreg];
     etapol[i] = new real[numreg];
  }
}

void PolyPrem::PrintPolyPrem()
{
int i,j;
   cout << "Number of regions in prem = " << numreg << endl;
   cout << "Radius of Earth = " << radius << endl;
   cout.precision(6);
   
   for(i=0;i<numreg;i++){
      cout << i << " dep0 = " << radius-rbot[i] << " dep1 = " << radius-rtop[i] << endl;
   }
   cout << endl << endl;
   for(i=0;i<numreg;i++){
      cout << "i = " << i << "rbot = " << rbot[i] << "rtop = " << rtop[i] << endl;
      for(j=0;j<5;j++){printf(" %8.2g", rhopol[j][i]);} cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", vpvpol[j][i]);} cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", vsvpol[j][i]);} cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", qspol[j][i]);}  cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", qkpol[j][i]);}  cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", vphpol[j][i]);} cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", vshpol[j][i]);} cout << endl;
      for(j=0;j<5;j++){printf(" %8.2g", etapol[j][i]);} cout << endl;
   }
}

void DeckPrem::PrintDeckPrem()
{
int i;
   cout << "Total # of layers in deck prem = " << numlay << endl;
   cout << "220 is found at layer = " << i220 << endl;
   cout << "400 is found at layer = " << i400 << endl;
   cout << "670 is found at layer = " << i670 << endl;
   cout << "cmb is found at layer = " << icmb << endl;
   cout << "inner-outer core boundary is found at layer = " << iiob << endl;
   
   cout << " inner core from region " << icbot << " to " << ictop << endl;
   cout << " outer core from region " << ocbot << " to " << octop << endl;
   cout << " lower mantle from region " << lmbot << " to " << lmtop << endl;
   cout << " upper mantle from region " << umbot << " to " << umtop << endl;
   
   for(i=0;i<numreg;i++){
      cout << itop[i]-ibot[i] << " layers in region " << i << " ibot = " 
           << ibot[i] << " itop = " << itop[i] << endl;
   }
   
   for(i=0;i<numlay;i++){
      cout << " i= " << i << " r = " << rr[i] << " vsv = " << vsv[i] << " vsh = " << vsh[i] << endl;
   }
}

void PolyPrem::ReadPolyPrem()
{
fstream inFile;
int j,nr;

 inFile.open("/scratch/troika23/gu/dbase/premod.dat",ios::in);		// open prem file for read
 
 if(!inFile) { 					// open failed?
     cerr << "Cannot open ``premod.dat'' for input\n";
     exit(-1);
 }
 inFile >> nr >> radius;
 
 InitPolyPrem(nr);
 
 for(int i=0;i<numreg;i++){
   inFile >> rbot[i] >> rtop[i];
   for(j=0;j<5;j++)inFile >> rhopol[j][i];
   for(j=0;j<5;j++)inFile >> vpvpol[j][i];
   for(j=0;j<5;j++)inFile >> vsvpol[j][i];
   for(j=0;j<5;j++)inFile >> qspol[j][i];
   for(j=0;j<5;j++)inFile >> qkpol[j][i];
   for(j=0;j<5;j++)inFile >> vphpol[j][i];
   for(j=0;j<5;j++)inFile >> vshpol[j][i];
   for(j=0;j<5;j++)inFile >> etapol[j][i];
 }
 inFile.close();
}

PolyPrem::~PolyPrem()
{
  delete []rtop;
  delete []rbot;
  
  for(int i=0;i<5;i++){
//   delete [] rhopol[i];
     delete [] vpvpol[i];
     delete [] vsvpol[i];
//   delete [] qspol[i];
//   delete [] qkpol[i];
     delete [] vphpol[i];
     delete [] vshpol[i];
//   delete [] etapol[i];
  }
}

/*
 * wave == 'H'  --- transverse wave
   wave == 'V'  --- longitudinal
 */
void DeckPrem::DeckPremUpUs(char wave)
{
real temp;
  
  if(wave=='H' || wave == 'h'){
     v[P]=vph;
     v[S]=vsh;
  }else if(wave=='V' || wave == 'v'){
     v[P]=vpv;
     v[S]=vsv;
  }else{
     cerr << "wrong character for the wave polarize direction\n";
     cout << "use either ``H'' or ``V'' \n";
     exit(-1);
  }
  cout << "P=" << P <<", and S=" << S << endl;
  for(int i=0;i<numlay;i++){
      u[P][i]=rr[i]/v[P][i];
      u[S][i]=(i>=iiob && i<icmb) ? 0.0 : rr[i]/v[S][i];	// outcore VS=0, set us=0
      if(i!=0 && i!=numlay-1){
         temp=log(rr[i]/rr[i+1]);
         fac[P][i]=1.0/(1.0-log(v[P][i]/v[P][i+1])/temp);
         if(i>=iiob && i<icmb)fac[S][i]=0.0;
         else fac[S][i]=1.0/(1.0-log(v[S][i]/v[S][i+1])/temp);
      }else if(i==0){
         temp=log(rr[1]);
         fac[P][0]=1.0/(1.0-log(v[P][1]/v[P][0])/temp);
         fac[S][0]=1.0/(1.0-log(v[S][1]/v[S][0])/temp);
      }
  }
  fac[P][numlay-1]=fac[S][numlay-1]=0.0;
}

Boolean PHASE::IsPhase(Phase pha)
{
  for(int i=0;i<numseg;i++)
      if(pors[i]==pha)return TRUE;
  return FALSE;
}

Boolean PHASE::IsPhase(Phase pha1,Phase pha2)
{
int ph1exist=FALSE,ph2exist=FALSE;

  for(int i=0;i<numseg;i++){
      if(pors[i]==pha1)ph1exist=TRUE;
      if(pors[i]==pha2)ph2exist=TRUE;
  }
  return (ph1exist && ph2exist)?TRUE:FALSE;
}

//
// phase independent member functions
//

void PHASE::newarray(class DeckPrem &DPrem, char modeltype)
{
int i,j,size,numlay=DPrem.numlay,numreg=DPrem.numreg;

  Exist[P]=IsPhase(P);
  Exist[S]=IsPhase(S);
  for(int k=P;k<=S;k++){
     if(Exist[k]){			// if there is P/S wave component
        dt[k]	= new real[numlay];
        dd[k]	= new real[numlay];
        integ[k]= new Boolean[numreg];
     }
  }
  
  rbeg = new int[numseg];
  rend = new int[numseg];
  para = new real[numseg];
  
  maxpt  = 6*numlay;		// large enough for PPP
  rad   = new real[maxpt];
  phi   = new real[maxpt];
  vel   = new real[maxpt];
  rlen  = new real[maxpt];
  krad  = new int[maxpt];
  if(modeltype=='S') {  	// for split model
     krad_um  = new int[maxpt];
     krad_lm  = new int[maxpt];
     for(i=0;i<4;i++)rfac_um[i]=new real[maxpt];
     for(i=0;i<4;i++)rfac_lm[i]=new real[maxpt];
  }
  if(modeltype='M'){
	for(i=0;i<MAXLAYER;i++){
		for(j=0;j<4;j++)rfac_mult[i][j]=new real[maxpt];
	}	
	krad_mult=new int* [MAXLAYER];
   	for(i=0; i<MAXLAYER; i++){
		krad_mult[i] = new int[maxpt];
   	}
	for(i=0;i<MAXLAYER;i++)
		for(j=0;j<maxpt;j++)	
			krad_mult[i][j]=0;

  }
  for(i=0;i<4;i++)rfac[i]=new real[maxpt];
}



void PHASE::init_eq(class DeckPrem &DPrem, real dep)
{
int i,k,numlay=DPrem.numlay,numreg=DPrem.numreg;
int itop,ibot;
real v0,r0,dv,dr;

  eqr=DPrem.GetRadius()-dep;
  if(eqr<=0.0 || eqr>DPrem.GetRadius()){
     cerr << "Invalid earthquake depth = " << dep << endl;
     exit(-1);
  }else if(eqr==DPrem.GetRadius()){
     eqr=DPrem.GetRadius()-0.001;	// to avoid some complications
  }
  if(dir[0]==DOWN){
     for(i=numreg-1;i>=0;i--){
         ibot=DPrem.ibot[i];
         if(eqr>DPrem.rr[ibot])break;
     }
     eqreg=i;		// earthquake region
     itop=DPrem.itop[i];
     ibot=DPrem.ibot[i];
     for(i=itop-1;i>=ibot;i--){
         if(eqr>DPrem.rr[i])break;
     }
     eqlay=i;		// earthquake layer
  }else{
     for(i=0;i<numreg;i++){
         itop=DPrem.itop[i];
         if(eqr<DPrem.rr[itop])break;
     }
     eqreg=i;		// earthquake region
     itop=DPrem.itop[i];
     ibot=DPrem.ibot[i];
     for(i=ibot+1;i<=itop;i++){
         if(eqr<DPrem.rr[i])break;
     }
     eqlay=i-1;		// earthquake layer
  }
  
  r0=DPrem.rr[eqlay];
  dr=(eqr-r0)/(DPrem.rr[eqlay+1]-r0);
  
  tlay[S]=tlay[P]=numlay;		// set deepest layer for P and S to be out
  
  for(k=P;k<=S;k++){
     if(Exist[k]){
        v0=DPrem.v[k][eqlay];	// use linear interpolation
        dv=DPrem.v[k][eqlay+1]-v0;
        eqv[k]=v0+dv*dr;		// P velocity at source
        tlay[k]=eqlay;			// at least at earthquake layer
        tr[k]  =eqr;			// at least to eq radius
     }else{
        tr[k]  =DPrem.rr[numlay-1];
     }
  }
  
  rbeg[0]=eqreg;				// ray begins at source
  rend[numseg-1]=DPrem.umtop;	// ends at the surface
  eqdt=eqdd=0.0;
  
}

real PHASE::GetSetDtDh(real para1)
{
real u,dtdh;

   u=eqr/eqv[pors[0]];         
   
   dtdh=-sqrt(u*u-para1*para1)/eqr;
   if(dir[0]==UP)dtdh=-dtdh;
   
   return dtdh;
}

void PHASE::CheckPara(real dep) const
{
  if(pmin>pmax){
      cerr <<"It is not possible to have this phase";
      cerr <<" for source depth = " << dep << endl;
      exit(-1);
  }
}

void PHASE::CalDlength(class BSModel& bsmodel)
{
real vv,rr0,rr1,pp0,pp1,dr,dphi,*rknt,B[4];
int i,ireg,maxk;

   rknt=bsmodel.rknt;
   maxk=bsmodel.maxk;
   
   rr0=rad[0];		// raypath which can be plotted using rr[], pp[]
   pp0=phi[0];		// numpt is the number of point for raypath
   
   for(i=0;i<numpt;i++){
      if(i!=numpt-1){
	   rr1=rad[i+1];
	   pp1=phi[i+1];
      }else{
	   rr1=rr0;
	   pp1=pp0;
      }
      ireg=krad[i] = GetReg(rad[i],rknt,maxk);
      if(ireg>0 && ireg<=maxk){
//	 cout << rad[i] << "  " << phi[i] << endl;
         BsplineFun(ireg, rad[i], rknt, maxk, B);
         vv=0.5/vel[i];
         rfac[0][i]=vv*B[0];
         rfac[1][i]=vv*B[1];
         rfac[2][i]=vv*B[2];
         rfac[3][i]=vv*B[3];
      }else{	
         rfac[0][i]=rfac[1][i]=rfac[2][i]=rfac[3][i]=0.0;
      }

      dr=rr1-rr0;
      dphi=(pp1-pp0)*rr1;
      rlen[i]=sqrt(dr*dr+dphi*dphi);
      rr0=rr1;
      pp0=pp1;
   }
}

