#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <assert.h>

#include "wsmath.h"

#ifdef UNISTD
#include <unistd.h>
#endif


#ifdef UNISTD		// unix standard lib is available
int  GEN_IPM::ReadIpm()
{
int i,fd,n_read;
char filename[132];


  cout <<"Inner product matrix (type q to finish):\n";
  cin >> filename;
  
  if(!strcmp(filename,"q")){
     cout<<"do not read matrix, return\n";
     return FALSE;
  }
  cout <<"......Reading ipm from file...... "<<filename<<endl;

  fd=SysOpen(filename,O_RDONLY);
   
  read(fd,&format,sizeof(int));
  ReadAtaEtc(fd);		// shared code, read in ata, atd etc
  close(fd);
  return TRUE;
}

int  BS_IPM::ReadIpm()
{
int i,fd,n_read;
float theta_phi[2];
char filename[132];

  cout <<"Inner product matrix (type q to finish):\n";
  cin >> filename;
  cout <<"......Reading ipm from file: "<<filename<<endl;
  
  if(!strcmp(filename,"q")){
     cout<<"finished reading matrices\n";
     return FALSE;
  }

  fd=SysOpen(filename,O_RDONLY);
   
  read(fd,&format,sizeof(int));
  cout << "ATA input format number = " << format << endl;
  if(format==BSP_R_MAT || format==BSP_R_MAT_NEW
     || format == BSP_R_TOPO_MAT){ // 3D B-spline only or waveform ATAs
     cout <<"B-spline model ipm\n";
     read(fd,&numgpt,sizeof(int));	// number of points on sphere
     theta = new float[numgpt];  
     assert(theta!=0);
     phi   = new float[numgpt];  
     assert(phi!=0);
     for(i=0;i<numgpt;i++){		// coordinates
         read(fd,theta_phi,2*sizeof(float));
         theta[i]=theta_phi[0];
         phi[i]=theta_phi[1];
     }
     read(fd,&numrknt,sizeof(int));	// number of radial knots
     if(numrknt>1){
        cout <<"Radial knots described as radius\n";
        rknt=new float[numrknt];
        assert(rknt!=0);
        read(fd,rknt,numrknt*sizeof(float));
     }else{
        cout<<"Error in BS_IPM::ReadIpm, not complete yet\n";
     }
  }else{
     cerr<<"Error in BS_IPM::ReadIpm, format "<<format<<" incompatible or not implemented. \n";
     exit(-1);
  }
  printf("numgpt=%d, num radial knot=%d\n", numgpt, numrknt) ;
  numatd = numgpt*numrknt;
  for(i=0; i<numrknt; i++){
     printf("i=%d, knot=%f\n", i, rknt[i]);
  }
  ReadAtaEtc(fd);		// shared code, read in ata, atd etc
  close(fd);
  return TRUE;
}

int  BS_IPM::ReadIpm_Topo(int invopt)
{ // reads general info of ATA with 2D topography
int i,fd,n_read;
float theta_phi[2];
char filename[132];

  cout <<"Inner product matrix (type q to finish):\n";
  cin >> filename;
  cout <<"......Reading ipm from file: "<<filename<<endl;
  
  if(!strcmp(filename,"q")){
     cout<<"finished reading matrices\n";
     return FALSE;
  }

  fd=SysOpen(filename,O_RDONLY);
   
  read(fd,&format,sizeof(int));
  if(invopt == 4) {
     if(format != BSP_R_MAT && format != BSP_R_TOPO_MAT
      && format != BSP_R_TOPO_ALL) {
        cout << "format error in joint topography inversion!!!" << endl;
        exit(-1);
     }
  }
  cout << "input ATA format = " << format << endl;
  if(format==BSP_R_MAT || format==BSP_R_MAT_NEW
     || format==BSP_R_TOPO_MAT
     || format==BSP_R_TOPO_410 || format==BSP_R_TOPO_660
     || format==BSP_R_TOPO_ALL ){ // 3D + 2D b-spline model
     cout <<"B-spline model ipm\n";
     read(fd,&numgpt,sizeof(int));	// number of points on sphere
     cout << "numgpt read = " << numgpt << endl;
     theta = new float[numgpt];  
     assert(theta!=0);
     phi   = new float[numgpt];  
     assert(phi!=0);
     for(i=0;i<numgpt;i++){		// coordinates
         read(fd,theta_phi,2*sizeof(float));
         theta[i]=theta_phi[0];
         phi[i]=theta_phi[1];
     }
     read(fd,&numrknt,sizeof(int));	// number of radial knots
     if(numrknt>1){
        cout <<"Radial knots described as radius\n";
        rknt=new float[numrknt];
        assert(rknt!=0);
        read(fd,rknt,numrknt*sizeof(float));
     }else{
        cout<<"Error in BS_IPM::ReadIpm, not complete yet\n";
     }
  }else{
     cerr<<"Error in BS_IPM::ReadIpm, format "<<format<<" incompatible or not implemented.\n";
     exit(-1);
  }
  for(i=0; i<numrknt; i++){
     cout << "i=" << i << " knot=" <<  rknt[i] << endl;
  }
  ReadAtaEtc_Topo(fd,invopt); // shared code, read in ata, atd etc
  close(fd);
  return TRUE;
}

int  BS_COM_IPM::ReadIpm()
{
int i,j,k,fd,n_read,ok;
float theta_phi[2];
char filename[132];

  cout <<"Inner product matrix (type q to finish):\n";
  cin >> filename;
  cout <<"Read ipm from file "<<filename<<endl;
  
  if(!strcmp(filename,"q")){
     cout<<"finished reading matrices\n";
     return FALSE;
  }

  fd=SysOpen(filename,O_RDONLY);
   
  read(fd,&format,sizeof(int));
  if(format==BSP_R_COM_MAT){			// joint 3D B-spline model
     cout <<"Joint regional-whole earth B-spline model ipm\n";
//
//  whole earth part
//     
     read(fd,&numgpt_we,sizeof(int));	// number of points on sphere
     cout <<" whole earth model numgpt_we="<<numgpt_we<<endl;
     theta_we = new float[numgpt_we];  
     assert(theta_we!=0);
     phi_we   = new float[numgpt_we];  
     assert(phi_we!=0);
     for(i=0;i<numgpt_we;i++){		// coordinates
         read(fd,theta_phi,2*sizeof(float));
         theta_we[i]=theta_phi[0];
         phi_we[i]=theta_phi[1];
     }
     read(fd,&numrknt_we,sizeof(int));	// number of radial knots
     cout <<" whole earth model numrknt_we="<<numrknt_we<<endl;
     if(numrknt_we>1){
        cout <<"Radial knots described as radius\n";
        rknt_we=new float[numrknt_we];
        assert(rknt_we!=0);
        read(fd,rknt_we,numrknt_we*sizeof(float));
     }else{
        cout<<"Error in BS_COM_IPM::ReadIpm, not complete yet\n";
     }
     for(i=0;i<numrknt_we;i++)cout<<i<<" radial knots "<<rknt_we[i]<<endl;
//
// regional part
//     
     read(fd,&numgpt_re,sizeof(int));	// number of points on sphere
     cout <<" regional model, numgpt_re="<<numgpt_re<<endl;
     theta_re = new float[numgpt_re];  
     assert(theta_re!=0);
     phi_re   = new float[numgpt_re];  
     assert(phi_re!=0);
     for(i=0;i<numgpt_re;i++){		// coordinates
         read(fd,theta_phi,2*sizeof(float));
         theta_re[i]=theta_phi[0];
         phi_re[i]=theta_phi[1];
     }
     read(fd,&numrknt_re,sizeof(int));	// number of radial knots
     cout <<" regional model, numrknt_re="<<numrknt_re<<endl;
     if(numrknt_re>1){
        cout <<"Radial knots described as radius\n";
        rknt_re=new float[numrknt_re];
        assert(rknt_re!=0);
        read(fd,rknt_re,numrknt_re*sizeof(float));
     }else{
        cout<<"Error in BS_COM_IPM::ReadIpm, not complete yet\n";
     }
     for(i=0;i<numrknt_re;i++)cout<<i<<" radial knots "<<rknt_re[i]<<endl;     
//     assert(numrknt_re==numrknt_we);
    if(numrknt_re!=numrknt_we){
       cout<<"The number of radial knots for the whole earth model\n";
       cout<<"is different from that of regional one ("<<numrknt_we<<"!="<<numrknt_re<<")"<<endl;
       cout<<"Is this OK? (yes=1, no=0)\n";
       cin >> ok;
       if(!ok)exit(-1);
    }
//
//  read index for the overlapped points
//
    i=numgpt_we*numrknt_we;
    index=new int[i];
    read(fd,index,i*sizeof(int));
  
 //   for(j=0;j<i;j++)cout <<j<<"  "<<index[j]<<endl;


  }else{
     cerr<<"Error in BS_COM_IPM::ReadIpm, format "<<format<<" has not been implemented yet\n";
     exit(-1);
  }
  ReadAtaEtc(fd);		// shared code, read in ata, atd etc
  close(fd);
//
// calculate the number of overlapped points
//
  k= numgpt_we*numrknt_we;
  for(j=i=0;i<k;i++)if(index[i]<0)j++;
  numovl=j;

  return TRUE;
}

void BS_COM_IPM::StatsPart()
{
int numatd0=numgpt_we*numrknt_we-numovl;
int i,k,numatd1, nx;
float sum=(float)0,sum2=(float)0,atak,atdk;

  if(format!=BSP_R_COM_MAT){			// joint 3D B-spline model
     cout<<"Warning, function BS_COM_IPM::StatsPart only works for\n";
     cout<<"the matrix for joint 3D B-spline models\n";
     return;
  }

  cout<<"........Some statistics for the whole earth part............\n";
  numatd_save=numatd;
  numatd=numatd0;
  nx=0;                     // quick fix, needs more considerations!!
  Stats(nx);
  numatd=numatd_save;	// restore it
  
  cout<<"........Some statistics for the regional  part............\n";
  atamin=(float)1.0e38;		// should use the value defined in limits.h
  atamax=-atamin;

  for(k=i=0;i<numatd;i++){
      if(i>=numatd0){
	  atak=ata[k];
	  sum+=atak;
	  sum2+=atak*atak;
	  atamin=min(atamin,atak);
	  atamax=max(atamax,atak);
	}
	k+=i+2;
  }
  
  numatd1=numatd-numatd0;

  atavg=sum/(float)numatd1;
  sum2/=(float)numatd1;
  atavar=sum2-atavg*atavg;

  atdmin=(float)1.0e38;		// should use the value defined in limits.h
  atdmax=-atdmin;

  sum=sum2=(float)0;
  for(i=numatd0;i<numatd;i++){	// the regional part
      atdk=atd[i];
      sum+=atdk;
	sum2+=atdk*atdk;
	atdmin=min(atdmin,atdk);
	atamax=max(atamax,atdk);
  }

  atdvg=sum/(float)numatd1;
  sum2/=(float)numatd1;
  atdvar=sum2-atdvg*atdvg;
  numatd=numatd1;
  PrintStats(nx);           // again, nx is just a quick fix
  numatd=numatd_save;	// restore it
}

void BS_COM_IPM::AddIpm(BS_COM_IPM &mat2,float &weight)
{
int i,total;

   assert(format==mat2.format);		// some error checking
   assert(format==BSP_R_COM_MAT);
   assert(numgpt_we==mat2.numgpt_we);
   assert(numgpt_re==mat2.numgpt_re);
   assert(numrknt_we==mat2.numrknt_we);
   assert(numrknt_re==mat2.numrknt_re);
   
   for(i=0;i<numrknt_we;i++)		// check further the radial knots
       assert(fabs(rknt_we[i]-mat2.rknt_we[i])<=0.001);	// if there is substantial diff in radial knots
   
   for(i=0;i<numrknt_re;i++)		// check further the regional part
       assert(fabs(rknt_re[i]-mat2.rknt_re[i])<=0.001);	// if there is substantial diff in radial knots

   for(i=0;i<numgpt_we;i++){
       assert(fabs(theta_we[i]-mat2.theta_we[i])<=0.001);	// horizontal grids
       assert(fabs(phi_we[i]-mat2.phi_we[i])<=0.001);
   }
   
   for(i=0;i<numgpt_re;i++){
       assert(fabs(theta_re[i]-mat2.theta_re[i])<=0.001);
       assert(fabs(phi_re[i]-mat2.phi_re[i])<=0.001);
   }
   
   total=numgpt_we*numrknt_we;
   
   for(i=0;i<total;i++)
    assert(index[i]==mat2.index[i]);
   
   AddAtaEtc(mat2,weight);
}


void IPM::ReadAtaEtc(int &fd)
{
int i,ind,n_read;
int numata_save;

  read(fd,&numatd_save,sizeof(int));
  cout << "OLD number of atd = " << numatd_save  << endl;
  cout << "NEW number of atd = " << numatd << endl;
  if(format==BSP_R_MAT || format==BSP_R_MAT_NEW) { 
     nsurf = 0;               // layers of 2-D surface B-splines
  }else if(format==BSP_R_TOPO_MAT) { // waveform B-splines
     read(fd,&nsurf, sizeof(int));
  }else {
     cout << "format error in ReadAtaEtc(), not compatible."<<endl;
     exit(-1);
  }
  numata=(numatd*(numatd+1))/2;
  numata_save=(numatd_save*(numatd_save+1))/2;
  if(ata != 0)  {
     delete [] ata;
  }
  if(atd != 0)  {
     delete [] atd;
  }
  ata=new float[numata];
  assert(ata!=0);
  atd=new float[numatd];
  assert(atd!=0);
  cout <<"......reading ata......\n";
  for(i=0; i<numatd;i++) atd[i]=0.0;
  for(i=0; i<numata; i++) ata[i]=0.0;
  n_read=read(fd,ata,numata_save*sizeof(float));
  if(n_read!=(numata_save*sizeof(float))){
     cerr<<"Error in IPM::ReadAtaEtc, unable to read ata\n";
     exit(-1);
  }
  cout <<"......reading atd......\n";
  n_read=read(fd,atd,numatd_save*sizeof(float));
  if(n_read!=(numatd_save*sizeof(float))){
     cerr<<"Error in IPM::ReadAtaEtc, unable to read atd\n";
     exit(-1);
  }
// check whether all the diagnal elements in ata are greater than 0.0

  for(ind=i=0;i<numatd_save;i++){
      if(ata[ind]<0.0){
         cout<<"Error in IPM::ReadAtaEtc, ata["<<i<<"]"
         <<"["<<i<<"]="<<ata[ind]<<"<0.0"<<endl;
         exit(-1);
      }
      ind+=i+2;
  }
  for(i=0;i<10;i++){
	cout << i  << " " << ata[i] <<" " << i+999999  << " " << ata[i+999999] << endl;
  }
}

#endif

void BS_IPM::ReadAtaEtc_Topo(int &fd, int invopt)
{
int i,ind,n_read;
int numata_save;

  read(fd,&numatd_save,sizeof(int));
  cout << "Saved number of element in atd = " << numatd_save  << endl;
  if(format==BSP_R_TOPO_MAT || format==BSP_R_TOPO_410
     || format==BSP_R_TOPO_660 || format==BSP_R_TOPO_ALL){ // case of waveform ATAs or SS precursor
     read(fd,&nsurf, sizeof(int));
     cout << "number of 2D bspline = " <<  nsurf << endl;
  }
  cout << "numrknt = " << numrknt <<endl; 
  if(invopt==3) {  // joint inversion with 1 topography layer
     numatd = numgpt*numrknt+numgpt;         // final #ATD
  } else if(invopt==4){
     numatd = numgpt*numrknt+numgpt+numgpt;  // final #ATD
  } else {
    cout << "format error in ReadAtaEtc_Topo(), exit."<<endl;
    exit(-1);
  }
  cout << "NEW number of atd = " << numatd << endl;
  numata=(numatd*(numatd+1))/2;
  numata_save = (numatd_save*(numatd_save+1))/2;
  if(ata != 0){
     delete [] ata;
  }
  if(atd != 0){
     delete [] atd;
  }
  ata=new float[numata];
  assert(ata!=0);
  atd=new float[numatd];
  assert(atd!=0);
  for(i=0; i<numatd;i++) atd[i]=0.0;
  for(i=0; i<numata; i++) ata[i]=0.0;
  cout <<"......reading ata......\n";
  n_read=read(fd,ata,numata_save*sizeof(float));
  if(n_read!=(numata_save*sizeof(float))){
     cerr<<"Error in IPM::ReadAtaEtc_Topo, unable to read ata\n";
     exit(-1);
  }
  cout <<"......reading atd......\n";
  n_read=read(fd,atd,numatd_save*sizeof(float));
//  for(i=5068;i<5430;i++){
//	cout << i << " " << atd[i] << endl;
//  }
  if(n_read!=(numatd_save*sizeof(float))){
     cerr<<"Error in IPM::ReadAtaEtc_Topo, unable to read atd\n";
     exit(-1);
  }
}

void BS_IPM::Replace_Info(int opt)
{
  if(opt<=2) { // no topography inversion
     nsurf = 0;
     numatd = numgpt*numrknt;
  }
  if(opt==3) { // one topography inversion
     nsurf = 1;
     numatd = numgpt*(numrknt + 1);
  }
  if(opt==4) { // two topography inversions
     nsurf = 2;
     numatd = numgpt*(numrknt + 2);
  }
  numata = numatd*(numatd+1)/2;
}

void BS_IPM::CheckAtaDiag()
{
int i, ind;

// check whether all the diagonal elements in ata are greater than 0.0
//
  for(ind=i=0;i<numatd;i++){
      if(ata[ind]<0.0){
         cout<<"Error in BS_IPM::CheckAtaDiag, ata["<<i<<"]"
         <<"["<<i<<"]="<<ata[ind]<<"<0.0"<<endl;
         exit(-1);
      }
      if(i>=5068){
         cout << ind << "  " << ata[ind] << endl;
      }
      ind+=i+2;
  }
}

void BS_IPM::WeightAta()
{
int i;
float wate;
   cout << "input weight for ATA matrix " << endl;
   cin >> wate;
   cout << "\n numata = " << numata << endl;
   cout << " numatd = " << numatd << endl;
   for(i=0;i<numata;i++) ata[i] *= wate;
   for(i=0;i<numatd;i++) atd[i] *= wate;
//   int j;
//   for(i=j=0; i<numatd; i++){
//      cout << "i= " << i << " ata=" << ata[j] << endl;
//      j+=i+2;
//   }

}

void BS_IPM::ScaleAta()
{  
// this is used to scale down ata due to the scaling
// factor of 2/3 in geomap
int i;
float scale=2.0/3.0;
   cout << "\n numata = " << numata << endl;
   cout << " numatd = " << numatd << endl;
   for(i=0;i<numata;i++) ata[i] *= scale;

}

void IPM::SubInitModel(float *coeff, int ataxopt, int ntopo, int numgpt)
{  // this subtract an initial model or perform a resolution test
   // ataxopt: 	    2. subtract initial model
   //		    3. resolution test, use initial model value for ATD
   //		    1. add back initial model
int i,j, k, ndum, ilayer;
int m, n;
int nhp, nkt;
float dumlat, dumlon;
float atam, dum;
float *tempatd;
float *tempcoef;
char flname[250];
fstream file2d;
   tempatd = new float [numatd+1];
//   tempcoeff = new float [numpt + 1]
   cout << "\n numata = " << numata << endl;
   cout << " numatd = " << numatd << endl;
   cout << " ataxopt = " << ataxopt << endl;
   cout << " ntopo = " << ntopo << endl;
   ndum = 0;
   ilayer = 0;
   k = numatd - ntopo;
   while((ntopo-ndum) >= numgpt) {
	cout << "enter name of layer " << ilayer+1 << endl;
	cin >>  flname;
 	file2d.open(flname,ios::in);	// open bspline model for read
 
 	if(!file2d) { 			// open failed?
     		cerr << "Cannot open "<<flname<<" for input\n";
     		exit(-1);
	}
	file2d >> nhp >> nkt;
	if(nhp != numgpt) {
		cerr << "inconsistent 3D and 2D model!" << endl;
		exit(-1);
	}
 	for(m=0;m<numgpt;m++){
   		file2d >> n >> dumlat >> dumlon;
   		assert((m+1)==n);
     		for(j=0;j<7;j++){
       			file2d >> n;
     		}
 	}
	
        for(j=0;j<numgpt;j++){
        	file2d >> dum;
        	coeff[k]=dum;
       		k++;
     	}
	file2d.close();
	ndum += numgpt;
	ilayer++ ;
   }
   MultAX(ata, coeff, numatd, tempatd);
   if(ataxopt == 1) {
	// add a model
   	cout << "**Add Initial Model**" << endl;
   	for(i=0;i<numatd;i++){
		atd[i] += tempatd[i];
   	}
   }else if(ataxopt == 2) {
	// subtract a model
   	cout << "**Subtract Initial Model**" << endl;
   	for(i=0;i<numatd;i++){
		atd[i] -= tempatd[i];
   	}
   }else if(ataxopt == 3){
	// resolution test
   	cout << "**Resolution Test**" << endl;
   	for(i=0;i<numatd;i++){
		atd[i] = tempatd[i];
   	}
        
   }
// if none of above, does nothing to ATD
   delete [] tempatd;
}

void IPM::MultAX(float *a, float *x, int dim, float *ax)
{
// this will calculate matrix ATA*X where ATA is saved
// as a lower triangular matrix
 int i, j, index;
 for(i=0;i<dim;i++){
	ax[i]=0.0;
	for(j=0;j<=i;j++){
		index = i*(i+1)/2+j;
		ax[i]+=x[j]*a[index];
	}
	for(j=i+1;j<dim;j++){
		index = j*(j+1)/2+i;
		ax[i]+=a[index]*x[j];
	}
 }
// cout << x[0] << " " << x[5067] << " " << x[5068] << endl;
// cout << x[5430] << " " << x[5791] << endl;
// cout << a[0] << " " << a[2] <<" " << a[12834709] << " " << a[12839777] << endl;
}

void IPM::FlipAta(int numgpt, int numknt)
{
int i,j,k,l,m,n;
int blockrow, blockcol;
int ind0,ind1;
int row, col;
double temp;
float *tempata, *tempatd;
   cout << "\n numata = " << numata << endl;
   cout << " numatd = " << numatd << endl;
   cout << " num of horizontal knots = " << numgpt << endl;
   cout << " num of radial knots = " << numknt << endl;
   tempatd = new float [numatd];
   for(i=0;i<=numatd;i++){
	tempatd[i] = 0.0;
   }
   for(i=0;i<numatd;i++){
     blockrow = i/numgpt;
     for(j=0;j<=i;j++){
        blockcol = j/numgpt;
	ind0 = i*(i+1)/2+j;
	row = (numknt-1-blockcol)*numgpt+i%numgpt;
	col = (numknt-1-blockrow)*numgpt+j%numgpt;
	if((numknt-1-blockcol)<blockrow) continue;
	ind1=row*(row+1)/2+col;
        temp = ata[ind0];
	ata[ind0]=ata[ind1];
	ata[ind1]=temp;
     }
   } 
   for(i=0;i<numatd;i++){
	j=(numknt-1-i/numgpt)*numgpt+(i%numgpt);
	tempatd[j]=atd[i];
   }
   for(i=0;i<=numatd_save;i++){
	atd[i]=tempatd[i];
   }
   delete [] tempatd;
}

void IPM::AddAtaEtc(IPM &mat2,float &weight)
{
int i, j;

   if(weight<0.0)
      cout<<"Warning IPM::AddAtaEtc, weight <0.0\n";

   if(numatd!=mat2.numatd || numata!=mat2.numata){
      cout<<"Error in IPM::AddAtaEtc, inconsistent matrix size\n";
      cout<<"numata1="<<numata<<" numata2="<<mat2.numata<<endl;
      cout<<"numatd1="<<numatd<<" numatd2="<<mat2.numatd<<endl;
      exit(-1);
   }
   for(i=10; i<15; i++) {
      j = i*100;
      printf("%d, %f %f\n", j, ata[j], atd[j]);
   } 
   for(i=10; i<15; i++) {
      j = i*100;
      printf("matrix 2 %d, %f %f\n", j, mat2.ata[j], mat2.atd[j]);
   } 
   cout<<"......adding ata......\n";
   for(i=0;i<numata;i++)ata[i]+=weight*mat2.ata[i];
   cout<<"......adding atd......\n";
   for(i=0;i<numatd;i++)atd[i]+=weight*mat2.atd[i];
   for(i=10; i<15; i++) {
      j = i*100;
      printf("%d, %f %f\n", j, ata[j], atd[j]);
   }
}

void IPM::AddAnisoAta(IPM &mat2, int nknt, int ngpt, int imat, int opt)
{
// Code to add a P-wave or S-wave matrix to the total anisotropic
// ATA matrix for anisotropic inversion.  P-matrix is on the
// top left of the final matrix, and S-wave on bottom right triangular
// portion.  The interaction terms, left middle are ignored.
int i, j, irow, icol, ind;
int numatd0, numata0;

   numatd0=nknt*ngpt;
   numata0=numatd0*(numatd0+1)/2;
   if(imat==0)  {
	numatd=numatd0*2;	// double the size of atd for PS inversion
	numatd_save=numatd;
	numata=numatd*(numatd+1)/2;
	cout << "numatd0 (small) =" << numatd0 << endl;
	cout << "numata0 (small) =" << numata0 << endl;
	cout << "numatd (big matrix) =" << numatd << endl;
	cout << "numata (big matrix) =" << numata << endl;
  	ata=new float[numata];
  	assert(ata!=0);
  	atd=new float[numatd];
  	assert(atd!=0);
  	for(i=0; i<numatd;i++) atd[i]=0.0;
  	for(i=0; i<numata; i++) ata[i]=0.0;
   }
   for(i=10; i<15; i++) {
      j = i*100;
      printf("%d, %f %f\n", j, ata[j], atd[j]);
   } 
   for(i=10; i<15; i++) {
      j = i*100;
      printf("matrix 2 %d, %f %f\n", j, mat2.ata[j], mat2.atd[j]);
   } 
   cout<<"......adding ata......\n";
   if(opt==1){	// P-wave portion, left top cornor of the final ATA mat.
   	for(i=0;i<numata0;i++)ata[i]+=mat2.ata[i];
   	cout<<"......adding atd......\n";
   	for(i=0;i<numatd0;i++)atd[i]+=mat2.atd[i];
   }else if(opt==2){ // S-wave portion, bottom right corner
	i=0;
	ind=0;
	for(irow=0;irow<numatd;irow++){
		for(icol=0;icol<=irow;icol++){
			if(irow>=numatd0 && icol>=numatd0){
				ata[ind]+=mat2.ata[i];
				i++;
				if(i<10) {
				  cout  << "irow=" << irow << " icol=" << icol << "ind=" << ind << "ata="<< ata[ind] << endl;
				}
			}
			ind++;
		}
	}
   	for(i=0;i<numatd0;i++)atd[i+numatd0]+=mat2.atd[i];
   }else{
	cout << "option of P-or-S matrix type not found!!" << endl;
	exit(-1);
   }
   for(i=10; i<15; i++) {
      j = i*100;
      printf("%d, %f %f\n", j, ata[j], atd[j]);
   }
}


void BS_IPM::AddIpm(BS_IPM &mat2,float &weight)
{ // add two inner product matrices and Atd matrices for different data sets
int i;
   cout << "format=" << format << ", mat2.format=" << mat2.format <<endl; 
   assert(format==BSP_R_MAT || format==BSP_R_MAT_NEW
         || format==BSP_R_TOPO_410 
         || format==BSP_R_TOPO_660 
         || format==BSP_R_TOPO_ALL
	 || format==BSP_R_TOPO_MAT);

   if(numgpt!=mat2.numgpt || numrknt!=mat2.numrknt){
         cout <<"parameter contained in matrix 1: numgpt="<<numgpt<<" numrknt="<<numrknt<<endl;
         cout <<"parameter contained in matrix 2: numgpt="<<mat2.numgpt<<" numrknt="<<mat2.numrknt<<endl;
         cout<<"Error in IPM::AddIpm, inconsistent model parameters\n";
         exit(-1);
   }
   for(i=0;i<numrknt;i++){		// check further the radial knots
          if(fabs(rknt[i]-mat2.rknt[i])>0.001){	// if there is substantial diff in radial knots
             cout<<"Error in IPM::AddIpm, inconsistent radial knots\n";
             cout<<"i="<<i<<"  rknt1="<<rknt[i]<<" rknt2="<<mat2.rknt[i]<<endl;
             exit(-1);
          }
   }
   AddAtaEtc(mat2,weight);
}

void BS_IPM::AddAnisoIpm(BS_IPM &mat2,int nknt, int ngpt, int imat, int opt)
{ // add a small inner product matrix to a large inner product
  // matrix for anisotropic or P-S inversion.
int i;
   numgpt=ngpt;
   format=11;
   numrknt=nknt*2;
   theta = new float[ngpt];  
   assert(theta!=0);
   phi   = new float[ngpt];  
   assert(phi!=0);
   for(i=0;i<ngpt;i++){
	theta[i]=mat2.theta[i];
	phi[i]=mat2.phi[i];
   }
   AddAnisoAta(mat2,nknt, ngpt, imat, opt);
}



void GEN_IPM::PrintIpm()
{
  cout<<"format="<<format<<endl;
  cout<<"matrices in general, not related to any model\n";
  PrintAtaEtc();		// shared code, print ata etc
}

void BS_IPM::PrintIpm()
{
int i, k;
   if(format==BSP_R_MAT || format==BSP_R_MAT_NEW
     || format==BSP_R_TOPO_MAT
     || format==BSP_R_TOPO_410
     || format==BSP_R_TOPO_660
     || format==BSP_R_TOPO_ALL){ // 3D B-spline model
      cout<<"B-spline ata matrix\n";
      cout<<"number of horizontal B-spline knots = "<<numgpt<<endl;
          
      cout<<"number of radial points to define B-spline = "<<numrknt<<endl;
      for(i=0;i<numrknt;i++)
          cout<<"i = "<<i<<" rknot ="<<rknt[i]<<endl;
      if(format==BSP_R_TOPO_MAT || format==BSP_R_TOPO_410 ||
          format==BSP_R_TOPO_660)
        cout << "#layer of topography B-splines= " << nsurf << endl;
   }else{
     cerr<<"Error in BS_IPM::~PrintIpm()\n";
     cerr<<"format "<<format<<" has not been implemented yet\n";
     exit(-1);
   }
   
// PrintAtaEtc();		// shared code, print ata etc
   
}

void BS_COM_IPM::PrintIpm()
{
  assert(format==BSP_R_COM_MAT);			// joint 3D B-spline model   
  PrintAtaEtc();		// shared code, print ata etc
}

void BS_COM_IPM::DampDiag(float& adamp1, float& adamp2, int numgpt)
{
int i,k;
int numatd1=numgpt_we*numrknt_we-numovl;

  cout<<" damping BS_COM_IPM\n";

  for(k=i=0;i<numatd;i++){
      if(i<numatd1)
          ata[k]+=adamp1;
      else
          ata[k]+=adamp2;
      k+=i+2;
  }

}

void BS_COM_IPM::DampRadialAta(float* dampmat, float *dampmat_re,
   float fac1, float fac2, int numknt, int numknt_re)
{
int i,k,ind,k1;
int numatd1=numgpt_we*numrknt_we-numovl;	/* # of coeff. related to whole earth part*/
int n1,n2,numlay;

  cout <<" numatd = "<<numatd<<endl;
  cout <<" numovl = "<<numovl<<endl;
  cout <<" numknt = "<<numknt<<endl;
  cout <<" numknt_re = "<<numknt_re<<endl;
  cout <<" numatd1 = "<<numatd1<<endl;
  
  cout <<" numgpt_we="<<numgpt_we<<endl;
  cout <<" numgpt_re="<<numgpt_re<<endl;
  
  n1=(numrknt_we-numknt_re)*numgpt_we;	/* # of grid whole earth point that don't have */
  							/*  overlap with regional ones*/

  numlay=numgpt_we-numovl/numknt_re;
  							
  cout << " numlay = "<< numlay<<endl;
  cout << " n1 = "<< n1<<endl;
    

  for(k=i=0;i<numatd;i++){
     if(i<numatd1){			/* ata elements for whole earth part */
        if(i<n1)
           k1=i/numgpt_we;
        else{
           k1=(i-n1)/numlay;
           k1+=numrknt_we-numknt_re;
        }
        ind=k1*numknt+k1;
        ata[k]+=dampmat[ind]*fac1;
//        cout<<"i = "<< i<<" ind="<< ind<<" mat="<< dampmat[ind]<<"   "<< fac1<<endl;
     }else{					/* ata elements for regional earth part */
        k1=(i-numatd1)/numgpt_re;
        ind=k1*(numknt_re+1);
        ata[k]+=dampmat[ind]*fac2;
//        cout<<"i = "<< i<<" ind="<< ind<<" mat="<< dampmat_re[ind]<<"   "<< fac2<<endl;
     }
     k+=i+2;
  }
  delete [] dampmat;
}

void BS_COM_IPM::DampRadialAta(float* dampmat, float fac1, float fac2, int numknt)
{
int i,k,ind,k1;
int numatd1=numgpt_we*numrknt_we-numovl;	/* # of coeff. related to whole earth part*/

  if((numatd%numknt)!=0){
      cout <<" Error in BS_COM_IPM::DampRadialAta 1\n";
      exit(-1);
  }
  
  for(k=i=0;i<numatd;i++){
     if(i<numatd1){				/* for whole earth part */
        k1=i/numgpt_we;
     }else{
        k1=(i-numatd1)/numgpt_re;
     }
     ind=k1*(numknt+1);
     ata[k]+=dampmat[ind]*fac2;
     cout<<"i = "<< i<<" ind="<< ind<<" mat="<< dampmat[ind]<<"   "<< fac1<<endl;
     k+=i+2;
  }

}

void IPM::PrintAtaEtc()
{
int i,j,k,natd=min(numatd,11);
   
  cout<<"\nnumber of element in ata ="<<numata<<endl;
  cout<<"ata=\n";
  for(k=0,i=0;i<natd;i++){
      for(j=0;j<=i;j++,k++)cout<<ata[k]<<" ";
      cout <<endl;
  }
  cout<<"\nnumber of element in atd ="<<numatd<<endl;
  for(i=0;i<natd;i++)cout <<"atd["<<i<<"]="<<atd[i]<<endl;
}

void BS_IPM::WriteIpm_topo(int ntopo,int ifnorm)
{
int i,j,k,fd,n_written;
float theta_phi[2];
char filename[132];

  cout<<"number of atd="<< numatd<<endl;

  cout <<"file to save ATA:\n";
  cin >> filename;
  cout <<"output ipm file="<<filename<<endl;

  fd=SysOpen(filename,1);
  cout << "output format = " << format << endl;
  cout << "numgpt = " << numgpt << endl;
  n_written=write(fd,&format,sizeof(int));
// write out model knots (horizontal)
  n_written=write(fd,&numgpt,sizeof(int));
  for(i=0;i<numgpt;i++){
      theta_phi[0]=theta[i];
      theta_phi[1]=phi[i];
      n_written=write(fd,theta_phi,2*sizeof(float));
  }
// write out model knots (radial)
  cout << "output number of radial knots = " << numrknt << endl;
  n_written=write(fd,&numrknt,sizeof(int));
  n_written=write(fd,rknt,numrknt*sizeof(float));
  
// write out matrices
  cout<<"number of atd elements ="<< numatd <<endl;
  cout<<"number of ata elements ="<< numata <<endl;
  n_written=write(fd,&numatd,sizeof(int));
  if(format > 19 || format==6) { 
  // numbers between 1-20 are reserved for 3D travel time only
  // format = 6 is waveform ATAs which also has this info
  	n_written=write(fd,&ntopo, sizeof(int));  // topography b-spline
        cout << "output number of 2D topo = " << ntopo << endl;
  }
  cout << "number of topography b-splines:" << ntopo << endl;
  if(ifnorm!=0) {
	cout << "waveform matrix, rescale ATA by 2/3" << endl;
  	for(i=0;i<numata;i++){
		ata[i]*=2.0/3.0;
  	}
  }
  n_written=write(fd,ata,numata*sizeof(float));
  n_written=write(fd,atd,numatd*sizeof(float));
  close(fd);
  
  for(i=0;i<5;i++)cout <<"atd["<<i<<"]="<<atd[i]<<endl;
  
  cout << "ata=\n";
  
  for(k=0,i=0;i<5;i++){
      for(j=0;j<=i;j++,k++){
          cout<<ata[k]<<" ";
      }
      cout <<endl;
  }
}


#ifdef UNISTD
void GEN_IPM::WriteIpm()
{
int fd,n_written;
char filename[132];

  cout <<"file to save ATA:\n";
  cin >> filename;
  cout <<"output ipm file="<<filename<<endl;
  fd=SysOpen(filename,1);

  n_written=write(fd,&format,sizeof(int));	// matrix format
  WriteAtaEtc(fd);
  close(fd);
}


void BS_COM_IPM::WriteIpm()
{
int i,fd,n_written;
char filename[132];
float theta_phi[2];

  cout <<"file to save ATA:\n";
  cin >> filename;
  cout <<"output ipm file="<<filename<<endl;
  fd=SysOpen(filename,1);

  n_written=write(fd,&format,sizeof(int));	// matrix format
  assert(n_written==sizeof(int));
  
  n_written=write(fd,&numgpt_we,sizeof(int));	// whole earth part
  assert(n_written==sizeof(int));

  for(i=0;i<numgpt_we;i++){				// coordinates
      theta_phi[0]=theta_we[i];
      theta_phi[1]=phi_we[i];
      n_written=write(fd,theta_phi,2*sizeof(float));
      assert(n_written==(2*sizeof(float)));
  }

  n_written=write(fd,&numrknt_we,sizeof(int));		// number of radial knots
  assert(n_written==sizeof(int));

  if(numrknt_we>1){
     n_written=write(fd,rknt_we,numrknt_we*sizeof(float));
     assert(n_written==(numrknt_we*sizeof(float)));
  }else{
     cout<<"Error in BS_COM_IPM::WriteIpm, not complete yet\n";
  }

  n_written=write(fd,&numgpt_re,sizeof(int));	// regional part
  assert(n_written==sizeof(int));

  for(i=0;i<numgpt_re;i++){				// coordinates
      theta_phi[0]=theta_re[i];
      theta_phi[1]=phi_re[i];
      n_written=write(fd,theta_phi,2*sizeof(float));
      assert(n_written==(2*sizeof(float)));
  }

  n_written=write(fd,&numrknt_re,sizeof(int));		// number of radial knots
  assert(n_written==sizeof(int));

  if(numrknt_we>1){
     n_written=write(fd,rknt_re,numrknt_re*sizeof(float));
     assert(n_written==(numrknt_re*sizeof(float)));
  }else{
     cout<<"Error in BS_COM_IPM::WriteIpm, not complete yet\n";
  }
  
  i=numgpt_we*numrknt_we;		//  overlapped points

  n_written=write(fd,index,i*sizeof(int));
  assert(n_written==(i*sizeof(float)));

  WriteAtaEtc(fd);
  close(fd);
}


void IPM::WriteAtaEtc(int &fd)
{
int n_written;
//
// write out matrices
//
  n_written=write(fd,&numatd,sizeof(int));
  numata=(numatd*(numatd+1))/2;
  n_written=write(fd,ata,numata*sizeof(float));
  cout<<"number of bytes saved for ata="<<n_written<<endl;
  n_written=write(fd,atd,numatd*sizeof(float));
  cout<<"number of bytes saved for atd="<<n_written<<endl;
}


#endif
BS_IPM::~BS_IPM()
{

   cout <<".....deleting ipm......\n";
   cout << "In destructor of BS_IPM, delete theta" << endl;
   delete [] theta;
   cout << "In destructor of BS_IPM, delete phi" << endl;
   delete [] phi;
   cout << "In destructor of BS_IPM, delete rknt" << endl;
   delete [] rknt;
   cout << "In destructor of BS_IPM, before DelAtaEtc" << endl;
   DelAtaEtc();
}

BS_IPM::BS_IPM()
{
   theta=NULL;
   phi=NULL;
   rknt=NULL;
}



BS_COM_IPM::BS_COM_IPM()
{
   theta_we=NULL;
   phi_we=NULL;
   rknt_we=NULL;
   theta_re=NULL;
   phi_re=NULL;
   rknt_re=NULL;
   index=NULL;
   ata=NULL;
   atd=NULL;
   
   numgpt_we=0;
   numrknt_we=0;
   numgpt_re=0;
   numrknt_re=0;
   numovl=0;
}

BS_COM_IPM::~BS_COM_IPM()
{

   cout <<".....deleting ipm......\n";
   delete [] theta_we;
   delete [] phi_we;
   delete [] rknt_we;
   delete [] theta_re;
   delete [] phi_re;
   delete [] rknt_re;
   delete [] index;
   
   DelAtaEtc();
}

GEN_IPM::~GEN_IPM()
{

   cout <<".....deleting ipm......\n";
   
   DelAtaEtc();
}

void IPM::DelAtaEtc()
{

   delete [] ata;
   cout << "recoving space from ata " << endl;
   delete [] atd;
   cout << "recoving space from atd " << endl;
}


void IPM::Stats(int num2d)
{ 
// num2d is read from actual ATAs, not the number of 2D
// b-splines that are  in the final ATA has.
int i,k, ndat;
int ind2d;
float sum=0.0,sum2=0.0,atak,atdk;

  atamin=(float)1.0e38;	// should use the value defined in limits.h
  atamax=-atamin;
  ndat = numatd_save-num2d;
  sum=sum2=(float)0.0;
  printf("in Stat(),num2d =%d, numatd_save=%d \n", num2d, numatd_save);
  cout << ata[0] <<" " << ata[100] << " "<<ata[1000] << " " << ata[2000]<<" " << ata[5000] << endl;
  for(k=i=0;i<ndat;i++){
	  atak=ata[k];
	  sum+=atak;
	  sum2+=atak*atak;
	  atamin=min(atamin,atak);
	  atamax=max(atamax,atak);
//	  printf("%d %f %f\n",i, sum, sum2);
	  k+=i+2;
  }

  atavg=sum/(float)ndat;
//cout << "ndat = "<< ndat << endl;
//cout<< " sum2=" << sum2 << endl;
  sum2/=(float)ndat;
  atavar=sum2-atavg*atavg;
//  cout << " atavg="<<atavg << endl; 
//  cout << " In Stats, ndat= " << ndat << "atavar = "<< atavar<< endl;

  atdmin=(float)1.0e38;	// should use the value defined in limits.h
  atdmax=-atdmin;

  sum=sum2=(float)0.0;
  for(i=0;i<ndat;i++){
	atdk=atd[i];
	sum+=atdk;
	sum2+=atdk*atdk;
	atdmin=min(atdmin,atdk);
	atdmax=max(atdmax,atdk);
  }

  atdvg=sum/(float)ndat;
  sum2/=(float)ndat;
  atdvar=sum2-atdvg*atdvg;
  
  PrintStats(num2d);
}

void BS_IPM::Stats_topo()
{
int i,k, ndat, ind2d, n;
int num2d, numatd_temp;
float sum=0.0,sum2=0.0,atak,atdk;

  if(nsurf == 0) {
     cout << "no surface b-splines for this ATA" << endl;
     return;
  }
  ndat = numrknt*numgpt;
  ind2d = ndat+numgpt;
  numatd_temp = ndat+nsurf*numgpt;
  n=0;
  cout << "In Stats_topo, numatd_temp= " << numatd_temp << "  numatd= " << numatd << endl;
  for(k=i=0; i<ndat;i++){
     k+=i+2;        // k index keeps increasing in the following loop
  }
  while(ind2d <= numatd_temp) {
     n++;
     printf("ndat = %d, ind2d=%d \n",ndat,ind2d);
     printf("\nIn the topography matrix:\n");
     atamin_2d=(float)1.0e38; // should use the value defined in limits.h
     atamax_2d=-atamin_2d;
     sum=sum2=(float)0.0;
     atavg_2d=0.0;
     for(i=ind2d-numgpt;i<ind2d;i++){
	 atak=ata[k];
         sum+=atak;
	 sum2+=atak*atak;
	 atamin_2d=min(atamin_2d,atak);
	 atamax_2d=max(atamax_2d,atak);
	 k+=i+2;
     }
     num2d = numgpt;
     atavg_2d=sum/(float)num2d;
     sum2/=(float)num2d;
     cout << "2D b-spline averaged over = " << num2d << endl;

     atavar_2d=sum2-atavg_2d*atavg_2d;
  
     atdmin_2d=(float)1.0e38; // should use the value defined in limits.h
     atdmax_2d=-atdmin_2d;

     sum=sum2=(float)0.0;
     for(i=ndat;i<ind2d;i++){
	atdk=atd[i];
	sum+=atdk;
	sum2+=atdk*atdk;
	atdmin_2d=min(atdmin_2d,atdk);
	atamax_2d=max(atamax_2d,atdk);
     }
     cout << "NOW atd[5068, 69]=" << atd[5068] << "  " <<   atd[5069] << endl;
     
     atdvg_2d=sum/(float)num2d;
     sum2/=(float)num2d;
     atdvar_2d=sum2-atdvg_2d*atdvg_2d;
     PrintStats_topo(n);
     ind2d += numgpt;
  }

}

void IPM::PrintStats(int num2d)
{
  cout << "\nOverall Statisics" << endl;
  cout << "-------------------------------------------------------"<<endl;
  cout<<"total numata="<<(numatd*(numatd+1))/2<<endl;
  cout<<"total numatd="<<numatd<<endl;
  
  cout << endl << endl << "3D b-spline statistics:" << endl;
  cout << "-------------------------------------------------------" << endl;
  cout<<"min diagonal element of ata ="<<atamin<<endl;
  cout<<"max diagonal element of ata ="<<atamax<<endl;
  cout<<"average of diagonal element of ata="<<atavg<<endl;
  cout<<"variance of diagonal element of ata="<<atavar<<endl;
  cout << "total 2D elements in atd = " << num2d << endl;
  cout<< "total 3D elements in atd = " << numatd-num2d << endl;
  cout<<"min element of atd ="<<atdmin<<endl;
  cout<<"max element of atd ="<<atdmax<<endl;
  cout<<"average of atd="<<atdvg<<endl;
  cout<<"variance of atd="<<atdvar<<endl;
}

void IPM::PrintStats_topo(int n)
{
   cout << endl <<endl << "2D Topography b-spline statistics:" << endl;
   if(n==1) {
      cout << "====================FIRST LAYER=======================" << endl;
   }else if(n==2) {
      cout << "====================SECOND LAYER=======================" << endl;
   }else if(n==3) {
      cout << "====================THIRD LAYER=======================" << endl;
   }
   cout<<"min diagonal element of ata ="<<atamin_2d<<endl;
   cout<<"max diagonal element of ata ="<<atamax_2d<<endl;
   cout<<"average of diagonal element of ata="<<atavg_2d<<endl;
   cout<<"variance of diagonal element of ata="<<atavar_2d<<endl;

   cout<<"min element of atd ="<<atdmin_2d<<endl;
   cout<<"max element of atd ="<<atamax_2d<<endl;
   cout<<"average of atd="<<atdvg_2d<<endl;
   cout<<"variance of atd="<<atdvar_2d<<endl;

}
void IPM::RedAtaAtd(int *inda, float& cutata)
{
int i,j,k,ii,natd,nata;

  numatd_save=numatd;		// save the original numatd;

  for(ii=0,k=i=0;i<numatd;i++){
      if(ata[k]<cutata){
         inda[i]=0;
         ii++;
 
      }
     // cout<<"i="<<i<<" ata["<<k<<"]="<<ata[k]<<endl;
	k+=i+2;
  }
  
  cout<<"number of unresolvable  coefficients ="<<ii<<endl;

  cout<<"......reducing atd......\n";
  natd=0;				// new number of atd
  for(i=0;i<numatd;i++){
      if(inda[i]==1){
         atd[natd]=atd[i];
         natd++;
      }
  }

  cout<<"......reducing ata......\n";
  nata=0;
  k=0;   
  for(i=0;i<numatd;i++){
     for(j=0;j<=i;j++){
       if(inda[i]==1 && inda[j]==1){
          ata[nata]=ata[k];
          nata++;
       }
       k++;
     }
  }
  cout<<" new numatd="<<natd<<endl;
  cout<<" new numata="<<nata<<endl;
  
  numatd=natd;		// reduced size of atd
  numata=numatd*(numatd+1)/2;

// for(i=0;i<numata;i++)cout<<"ata = "<<ata[i]<<endl;
// for(i=0;i<numatd;i++)cout<<"atd = "<<atd[i]<<endl;
}

void IPM::DampDiag(float& adamp, float adamp_topo[], int topolayer[],int ntopo)
{
int i,j, k;
int topo_layer[10], n3d;

  j=0;
  n3d = numatd - ntopo;
  cout << "In the diagonal damping:" << endl;
  cout << "number of 3D = " << n3d << endl;
  cout << "number of 2D = " << ntopo << endl;
  cout << "3D damping fac = " << adamp << endl;
  cout << "total number = " << numatd << endl;
  for(k=i=0;i<numatd;i++){
      if(i<n3d) {
         ata[k]+=adamp;
//         cout << "k = " << k << "  ata[k] = " << ata[k] << endl;
//	  cout << "i = " << i << "  atd[i] = " << atd[i] << endl;
      } else {  // damp the 2D portions, NEEDS CHANGE IF MORE THAN 2 LAYER
         if(i<topolayer[0]) {
           ata[k]+=adamp_topo[0];
//           printf("%d, %d, damping=%f\n", i, k, adamp_topo[0]);
         } else {
           ata[k]+=adamp_topo[1];
//           printf("%d, %d, damping 2=%f\n", i, k, adamp_topo[1]);
         }
      }
      k+=i+2;
  }
}
void IPM::DampDiag_Block(float &adamp,float *diagfac,int *kindex, int nk,
		int numgpt, float adamp_topo[],int topolayer[],int ntopo)
{
int i,j, k, nmin, nmax, ifdamped;
int topo_layer[10], n3d;

  j=0;
  n3d = numatd - ntopo;
  cout << "In the Block diagonal damping:" << endl;
  cout << "number of 3D = " << n3d << endl;
  cout << "number of 2D = " << ntopo << endl;
  cout << "3D damping fac = " << adamp << endl;
  cout << "# of Special K damping  = " << nk << endl;
  for(k=i=0;i<numatd;i++){
      if(i<n3d) {
	 ifdamped=0;
	 for(j=0;j<nk;j++){
  		nmin=(kindex[j]-1)*numgpt;
  		nmax=(kindex[j])*numgpt;
	 	if(i>=nmin && i<nmax){
			ifdamped = 1;
         		ata[k]+=diagfac[j];
			break;
	 	}
	}
	if(ifdamped==0) ata[k]+=adamp;
      } else {  // damp the 2D portions, NEEDS CHANGE IF MORE THAN 2 LAYER
         if(i<topolayer[0]) {
           ata[k]+=adamp_topo[0];
//           printf("%d, damping=%f\n", i, adamp_topo[0]);
         } else {
           ata[k]+=adamp_topo[1];
//           printf("%d, damping=%f\n", i, adamp_topo[1]);
         }
      }
      k+=i+2;
  }
}

/*
void IPM::Damp3D(float& adamp,float ,int ntopo)
{  // Damping the matrix using full 3D damping matrix
int i,j, k;
int topo_layer[10], n3d;

  j=0;
  n3d = numatd - ntopo;
  cout << "In the diagonal damping:" << endl;
  cout << "number of 3D = " << n3d << endl;
  cout << "number of 2D = " << ntopo << endl;
  cout << "3D damping fac = " << adamp << endl;
  cout << "total number = " << numatd << endl;
  for(k=i=0;i<numatd;i++){
      if(i<n3d) {
         ata[k]+=adamp;
//         cout << "k = " << k << "  ata[k] = " << ata[k] << endl;
//	  cout << "i = " << i << "  atd[i] = " << atd[i] << endl;
      } else {  // damp the 2D portions, NEEDS CHANGE IF MORE THAN 2 LAYER
         if(i<topolayer[0]) {
           ata[k]+=adamp_topo[0];
//           printf("%d, damping=%f\n", i, adamp_topo[0]);
         } else {
           ata[k]+=adamp_topo[1];
//           printf("%d, damping=%f\n", i, adamp_topo[1]);
         }
      }
      k+=i+2;
  }
}
*/

void IPM::CKBlockDiag_topo(int ntopo, int numknt, float* blkdiagavg)
{ // checks the size of each diagonal block in the ATA matrix
int i,j, k, k1,m, rowskip, colskip;
int topo_layer[10], n3d, ind;
int numgpt, numdamp;
float sumdiag;

  n3d = numatd - ntopo;
  numgpt = n3d/numknt;
  numdamp = numgpt*(numgpt+1)/2;
  cout << "n3d = " << n3d << "  ntopo=" << ntopo <<" numgpt=" << numgpt << " numdamp=" << numdamp<< endl;
  j=0;
  k=0;
  k1=0;
  ind=0;
  sumdiag = 0.0;
  for(i=0;i<numatd;i++){
     rowskip = i/numgpt;
     for(m=0;m<=i;m++){
        colskip = m/numgpt;
        if(ind == numdamp){
		ind=0 ;  // get damping matrix back to starting index
		blkdiagavg[rowskip-1] = sumdiag/(float)numdamp;
		cout << "Block "<< rowskip << ",  Average = " << blkdiagavg[rowskip-1] << endl;
		sumdiag = 0.0;
	}
        if(rowskip == colskip) {
	   sumdiag +=ata[k1];
           ind++;
        }
        k1++;
     }
  }
  blkdiagavg[rowskip] = sumdiag/(float)numdamp;
  cout << "Block "<< rowskip+1 << ",  Average = " << blkdiagavg[rowskip] << endl;
}

void IPM::CKDiag_topo(int ntopo, int numknt, float* blkdiagavg)
{ // checks the size of each diagonal element in the ATA matrix
int i,j, k, k1, m, rowskip, colskip;
int topo_layer[10], n3d, ind;
int numgpt, numdamp, inddiag,indknot;
float sumdiag;

  n3d = numatd - ntopo;
  numgpt = n3d/numknt;
  sumdiag = 0.0;
  inddiag=0;
  for(k=i=0;i<numatd;i++){
     k1=i%numgpt;
     sumdiag +=ata[k];
     if(k1==numgpt-1){
	blkdiagavg[inddiag] = sumdiag/(float)numgpt;
	cout << "k1=" << k1 << "  blkdiagavg=" << blkdiagavg[inddiag] << endl;
	sumdiag=0.0;
	inddiag++;
     }
     if(i<10){
     	cout <<" i= "<<i<<" k="<<k<<" inddiag="<<inddiag<< " ata=" << ata[k] << " diagonal sum ="<<sumdiag<<endl;
     }
     k+=i+2;
  }
}

void IPM::DampHBDAta(float* dampmat, int ntopo, int numknt, float* blkdiagavg, float *coeff0)
{  // damp the blocked diagonal of the ATA matrix
int i,j, k, k1,m, rowskip, colskip;
int topo_layer[10], n3d, ind, nblock;
int numgpt, numdamp, ntopolayer;
int optdamp;	// damping option 1=read parms, 2=use diagonal average
float dampfac;  // multiplication factor for block diag damping
float *damparm, *damparm_norm, tempdamp;
float *savedamp;
float *atddamp;
char parmfile[256];	
fstream dmpfile;

  n3d = numatd - ntopo;
  numgpt = n3d/numknt;
  ntopolayer = ntopo/numgpt;
  numdamp = numgpt*(numgpt+1)/2;
  for(i=0; i<10; i++) cout << i << "  H-damp = " << dampmat[i] << endl;
  cout << "n3d = " << n3d << "  ntopo=" << ntopo <<" numgpt=" << numgpt << " numdamp=" << numdamp << endl;
  damparm = new float [numknt+ntopolayer];
  damparm_norm = new float [numknt+ntopolayer];

// save the damping matrix
  savedamp = new float [numata];
  atddamp = new float [numatd];

  cout << "enter Block-Diag damping parameter file (" << numknt+ntopolayer << " parameters):" << endl;
  cout << "**format: col-1 = gradient damp;  col-2 = norm damp " << endl;
  cin >> parmfile;
  dmpfile.open(parmfile, ios::in);
  dmpfile >> optdamp >> dampfac; 
  nblock = 0;
  if(optdamp == 1) {
  	while((dmpfile >> damparm[nblock] >> damparm_norm[nblock]) != NULL){
	        cout << "damping parms read:" << nblock << "  " << damparm[nblock] << "  " << damparm_norm[nblock] << endl;
		nblock ++;
		if(nblock > numknt+ntopolayer) {
		   cout << "nblock =" << nblock << endl;
		   cout << "too many layers!"  << endl;
		   exit(-1);
		}
  	}
  }else{
  	cout << "damping with block diagonal average..." << endl;
	for(i=0;i<numknt+ntopolayer; i++){
		damparm[nblock] = blkdiagavg[i]*dampfac;
		nblock ++;
	}	
  }
  dmpfile.close();
  assert(nblock == numknt+ntopolayer);
  j=0;
  k=0;
  k1=0;
  ind=0;
  for(i=0;i<numatd;i++){
     rowskip = i/numgpt;
     for(m=0;m<=i;m++){
        colskip = m/numgpt;
	savedamp[k1] = 0.0;
        if(ind == numdamp){
		 cout << "processing block " << rowskip+1 << "  parm= " 
			<< damparm[rowskip] << endl;
		 ind=0 ;  // get damping matrix back to starting index
	}
        if(rowskip == colskip) {
	   tempdamp = damparm[rowskip]*dampmat[ind];
           ata[k1] += tempdamp;
//
// The following is used to damp the whole model, ATD = ATD-eta*X0 
// To damp toward target model, ATD = ATD-eta*(X0-Xtarget)
//
	   savedamp[k1] += tempdamp;	   

//		if(i > 5066){
//		  cout << i << " " << m << " " << k1 << " " << ind  << 
//		" " << ata[k1] << " " << damparm[rowskip] << " " << dampmat[ind] << endl;
//		}
	   if(i == m) {
		tempdamp = damparm_norm[rowskip];
		ata[k1] += tempdamp;  // norm damping
	   	savedamp[k1] += tempdamp;
//		  cout << k1 << " " << rowskip << " " << ind  << " " << damparm_norm[rowskip] << endl;
	   }
           ind++;
        }
        k1++;
     }
  }

// To damp the whole model, ATD = ATD-eta*X0.
// To damp toward target model, ATD = ATD+eta*X0.
// Otherwise, do nothing here.
  MultAX(savedamp, coeff0, numatd, atddamp);
//  for(i=0; i<numatd; i++) atd[i] += atddamp[i];
  for(i=0; i<numatd; i++) atd[i] -= atddamp[i];

  delete [] savedamp;
  delete [] damparm_norm;
  delete [] damparm;
}


void IPM::DampHorizAta(float* dampmat, float& adamp, float adamp_topo[], int topolayer[],int ntopo, int numknt)
{
int i,j, k, k1,m, rowskip, colskip;
int topo_layer[10], n3d, ind;
int numgpt, numdamp;

  n3d = numatd - ntopo;
  numgpt = n3d/numknt;
  numdamp = numgpt*(numgpt+1)/2;
  cout << "n3d = " << n3d << "  ntopo=" << ntopo <<" numgpt=" << numgpt << " numdamp=" << numdamp<< endl;
  for (i=0; i<2;i++){
     cout << "topolayer " << i << " : " << topolayer[i] << endl;
  }
  j=0;
  k=0;
  k1=0;
  ind=0;
  for(i=0;i<numatd;i++){
     rowskip = i/numgpt;
     for(m=0;m<=i;m++){
        colskip = m/numgpt;
        if(ind == numdamp){
		 ind=0 ;  // get damping matrix back to starting index
	}
        if(rowskip == colskip) {
           if(i<n3d) {
              ata[k1] +=adamp*dampmat[ind];
           } else {    // damp 2d, needs change if more than 2 layers
              if(i<topolayer[0]) {
//                cout << "i= " << i << " m=" << m << " k1=" << k1  << " ind=" << ind
 //                << " ata=" << ata[k1] << " damp= " << dampmat[ind]<< endl;
                ata[k1] += adamp_topo[0]*dampmat[ind];
              } else {
                ata[k1] += adamp_topo[1]*dampmat[ind];
//              printf("%d, damping=%f\n", i, adamp_topo[1]);
              }
           }
           ind++;
        }
        k1++;
     }
  }
}

void IPM::DampRadialAta(float* dampmat, float fac, int numknt)
{
int i,k,ind,k1,numgpt;

  if((numatd%numknt)!=0){
      cout <<" Error in IPM::DampRadialAta\n";
      exit(-1);
  }
  
  numgpt=numatd/numknt;

  for(k=i=0;i<numatd;i++){
     k1=i/numgpt;
     ind=k1*(numknt+1);
     ata[k]+=dampmat[ind]*fac;
//     cout <<" i= "<<i<<" k="<<k<<" ind="<<ind<<" damp="<<dampmat[ind]<<endl;
     k+=i+2;
  }

}


/*
 * to solve a positive definite sysmetric system ax=b
 *   a[0->n*(n+1)/2-1]        row-wise, lower triangle
 *   g[0->n*(n+1)/2-1]        cholesky storage
 *   b(0->n-1]                
 *   x[0->n-1]                answer vector
 *   n                      system dimension
 *   return value > 0 is the level at which p.d. failed
 */
int IPM::Choles(float *x)
{
float *y,*g,*a,*b;
int i,j,k,n,kj,kz,kzj,n1;
float sg,gkz;
     
     n=numatd;
     n1=n-1;
 
     g=a=ata;
     b=atd;
     if(a[0]<0.0)return 1;
     
     g[0]=(float)sqrt(a[0]);
     y = new float[n];
     y[0]=b[0]/g[0];
     
     for(i=1;i<n;i++){
         kz=(i*(i+1))/2;
         g[kz]=a[kz]/g[0];
         sg=g[kz]*g[kz];
         y[i]=b[i]-g[kz]*y[0];
         if(i>1){
            for(j=1;j<i;j++){
		kzj=kz+j;
                gkz=a[kzj];
                kj=(j*(j+1))/2;
          
                for(k=0;k<j;k++)
                    gkz-=g[kz+k]*g[kj+k];
          
                g[kzj]=gkz/g[kj+j];
                y[i]-=g[kzj]*y[j];
                sg+=g[kzj]*g[kzj];
            }
         }
      
         gkz=a[kz+i]-sg;
      
         if(gkz<=0.0){
            delete [] y;
            return i+1;
	 }
      
         g[kz+i]=(float)sqrt(gkz);
         y[i]/=g[kz+i];
      }
      
      kz=(n*n1)/2;
      x[n1]=y[n1]/g[kz+n1];
      if(n<=1) {
	 delete [] y;
	 return 0;
      }
      
      for(k=1;k<n;k++){
          i=n1-k;
          x[i]=y[i];
      
          for(j=i+1;j<n;j++){
              kj=(j*(j+1))/2;
              x[i]-=g[kj+i]*x[j];
          }

          kz=((i+1)*(i+2))/2;
          x[i]/=g[kz-1];
      }
       
      delete [] y;
      return 0;
}

void TwoPtsSph::SetCoordinate(Spherical2 &ss1, Spherical2 &ss2)
{
	sph1=ss1;
	sph2=ss2;
	cart1.FromSph2(sph1);		// further convert to cartesian coor
	cart2.FromSph2(sph2);
}

void TwoPtsSph::SetCoordinate(const int& choice)
{
   if(choice==GEOGRAPHICAL){
	cout<<"lat and lon for 1st point\n";
	cin >> geo1.lat >> geo1.lon;   
	cout<<"lat and lon for 2nd point\n";
	cin >> geo2.lat >> geo2.lon;
	sph1.FromGeographic(geo1);	// convert to spherical coordinates
	sph2.FromGeographic(geo2);
   }else if(choice==SPHERICAL){
	cout<<"theta and phi (in degrees) for 1st point\n";
	cin>>sph1.theta>>sph1.phi;
	cout<<"theta and phi (in degrees) for 2nd point\n";
	cin>>sph2.theta>>sph2.phi;
	sph1.theta*=RAD;
	sph1.phi*=RAD;
	sph2.theta*=RAD;
	sph2.phi*=RAD;
   }

   cart1.FromSph2(sph1);		// further convert to cartesian coor
   cart2.FromSph2(sph2);
}

float TwoPtsSph::CalDist()
{
  dist = (float)(cos(sph1.theta)*cos(sph2.theta)+
         sin(sph1.theta)*sin(sph2.theta)*cos(sph1.phi-sph2.phi));
  dist = (float)acos(dist);
  return dist;
}

void TwoPtsSph::FwdTrans_sph2s(const int &pt)
{
  if(pt==1 || pt==0)s1 = sph1.ForwardTrans(trans.matrix);
  if(pt==2 || pt==0)s2 = sph2.ForwardTrans(trans.matrix);
}

void TwoPtsSph::FwdTrans_sph(const int &pt)
{
  if(pt==1 || pt==0)sph1 = sph1.ForwardTrans(trans.matrix);
  if(pt==2 || pt==0)sph2 = sph2.ForwardTrans(trans.matrix);
}

void TwoPtsSph::InvTrans_s2sph(const int &pt)
{
  if(pt==1 || pt==0)sph1 = s1.InverseTrans(trans.matrix);
  if(pt==2 || pt==0)sph2 = s2.InverseTrans(trans.matrix);
}

void TwoPtsSph::InvTrans_s2s(const int &pt)
{
  if(pt==1 || pt==0)s1 = s1.InverseTrans(trans.matrix);
  if(pt==2 || pt==0)s2 = s2.InverseTrans(trans.matrix);
}

void Spherical2::FromGeographic(Geographic &geo)
{
const static float fac=(float)0.993305621334896;

   theta=((float)90-geo.lat)*RAD; 					// geographic colat  
   theta=(float)(HPI-atan(fac*cos(theta)/max(1.e-37,sin(theta))));	// geocentric colat
   
   phi = geo.lon;
   if(phi<0.0)phi+=(float)360;
   phi *= RAD;
}

void Spherical2::FromGeocentric(Geocentric &geo)
{

   theta=((float)90-geo.lat)*RAD; 					// geographic colat  
   
   phi = geo.lon;
   if(phi<0.0)phi+=(float)360;
   phi *= RAD;
}

void Cartesian::FromSph2(Spherical2 &sph)
{
  x =(float)sin(sph.theta);		// assume normalize coordinate i.e sph.r=1.0;
  y =x*(float)sin(sph.phi);
  x*=(float)cos(sph.phi);
  z =(float)cos(sph.theta);
}

//
// Input: sph1, sph2 --- two points which define the equator
// Purpose:  compute the transformation matrix which transforms
//           sph1 to be the new x axis and sph2 to be on the equator.
//
void TwoPtsSph::CalTrans4()
{
Cartesian c1,c2,c3;

  c1.x =(float)sin(sph1.theta);
  c1.y =c1.x*(float)sin(sph1.phi);
  c1.x*=(float)cos(sph1.phi);
  c1.z =(float)cos(sph1.theta);

  c2.x =(float)sin(sph2.theta);
  c2.y =c2.x*(float)sin(sph2.phi);
  c2.x*=(float)cos(sph2.phi);
  c2.z =(float)cos(sph2.theta);
  c3=c1.OutProdWith(c2);
  c2=c3.OutProdWith(c1);
  c1.Normalize();  
  c2.Normalize();
  c3.Normalize();
  trans.SetFromVec(c1,c2,c3);
}

void Cartesian::Normalize()
{
float temp;

  temp=x*x+y*y+z*z;
  if(temp>1.0e-35){
     temp=(float)sqrt(temp);
     x/=temp;
     y/=temp;
     z/=temp;
  }else{
     x=(float)2.0;
     y=(float)0.0;
     z=(float)0.0;
  }
}

/*
 * to do forward transformation on a normalized spherical vector
 */
Spherical2 Spherical2::ForwardTrans(float *matrix)
{
float x,y,z,x1,y1,z1;
Spherical2 temp;

  x =(float)sin(theta);
  y =x*(float)sin(phi);
  x*=(float)cos(phi);
  z =(float)cos(theta);
  
  x1=matrix[0]*x+matrix[1]*y+matrix[2]*z;
  y1=matrix[3]*x+matrix[4]*y+matrix[5]*z;
  z1=matrix[6]*x+matrix[7]*y+matrix[8]*z;

  temp.phi=(float)0.0;
  temp.theta=(float)acos(z1);
  if(fabs(temp.theta) > 1.0e-6){
     if(x1!=(float)0.0||y1!=(float)0.0)temp.phi=(float)atan2(y1,x1);
     if(temp.phi<(float)0)temp.phi+=TWOPI;
  }
  return temp;

}

Spherical2 Spherical2::InverseTrans(float *matrix)
{
float x,y,z,x1,y1,z1;
Spherical2 temp;

  x =(float)sin(theta);
  y =x*(float)sin(phi);
  x*=(float)cos(phi);
  z =(float)cos(theta);
  
  x1=matrix[0]*x+matrix[3]*y+matrix[6]*z;
  y1=matrix[1]*x+matrix[4]*y+matrix[7]*z;
  z1=matrix[2]*x+matrix[5]*y+matrix[8]*z;

  temp.phi=(float)0;
  temp.theta=(float)acos(z1);
  if(fabs(temp.theta) > 1.0e-6){
     if(x1!=(float)0.0||y1!=(float)0.0)temp.phi=(float)atan2(y1,x1);
     if(temp.phi<(float)0)temp.phi+=TWOPI;
  }
  return temp;
}

void TransMatrix::SetFromVec(Cartesian &v1, Cartesian &v2, Cartesian &v3)
{
   matrix[0]=v1.x; matrix[1]=v1.y; matrix[2]=v1.z;
   matrix[3]=v2.x; matrix[4]=v2.y; matrix[5]=v2.z;
   matrix[6]=v3.x; matrix[7]=v3.y; matrix[8]=v3.z;
   

}

Cartesian Cartesian::OutProdWith(Cartesian &c2)
{
Cartesian c3;

  c3.x=y*c2.z-z*c2.y;
  c3.y=z*c2.x-x*c2.z;
  c3.z=x*c2.y-y*c2.x;

  return c3;
}

//
// Input: s0   --- pole location
//	  azim --- the azimuth of new x axis
// Purpose: (1) compute the transformation matrix which transforms
//          the pole (s1) to be the new z axis, and the azim to
//          be the azimuth of new x axis (s0.theta=0, s0.phi=0,
//          azimuth=180 results in an identical matrix)
//          (2) compute two points which define the quator.

void POLARMAPELM::SetPoleTransMat(Spherical2& s0, float &azim)
{
Cartesian c1,c2,c3;
float ca,sa,ct,st,cp,sp;
Spherical2 ss1,ss2;

  Pole=s0;					// save pole location
  PoleCart.FromSph2(s0);		// convert to Cartesian coordinate

  ca=(float)cos(azim);
  sa=(float)sin(azim);
  ct=(float)cos(s0.theta);
  st=(float)sin(s0.theta);
  cp=(float)cos(s0.phi);
  sp=(float)sin(s0.phi);
  
  c3.x= st*cp;
  c3.y= st*sp;
  c3.z= ct;
  
  c1.x=-ca*cp*ct-sa*sp;
  c1.y= sa*cp-ca*ct*sp;
  c1.z= ca*st;
  
  c2.x= ca*sp-sa*ct*cp;
  c2.y=-ca*cp-sa*ct*sp;
  c2.z= sa*st;
  
  P12.trans.SetFromVec(c1,c2,c3);	// save transformation matrix
  ss1.theta=ss2.theta=(float)HPI;	// two points define the equator
  ss1.phi=(float)0;
  ss2.phi=(float)HPI;
  ss1.InverseTrans(P12.trans.matrix);
  ss2.InverseTrans(P12.trans.matrix);

  P12.SetCoordinate(ss1,ss2);
}

//
// Input: s1, s2 --- two points which define the equator
// Purpose: (1) compute the transformation matrix which transforms
//           s1 to be the new x axis and s2 to be on the equator.
//          (2) compute and save pole location under the old coordinate
//
void POLARMAPELM::SetPoleTransMat(Spherical2& s1, Spherical2& s2)
{
Spherical2 np;			// north pole

  P12.SetCoordinate(s1,s2);
  P12.CalTrans4();		// Calculate tranformation matrix
  np.theta=np.phi=(float)0;
  Pole=np.InverseTrans(P12.GetTrans().matrix);
  PoleCart.FromSph2(Pole);	// convert to Cartesian coordinate
}

POLARMAPELM::~POLARMAPELM()
{
}

void POLARMAPELM::ReleaseMem()
{
  if(numline>0){
	numline=0;
	delete [] numpt;
	delete [] line;
	delete [] xy;
  }
}

POLARMAPELM::POLARMAPELM()
{
  numline=0;
  // default pole location (Spherical2 and Cartesian)
}

// Input:  lon --- longitude for the meridian line ( use only 0 -> +180 )
//	   arclen --- appriximate increment in arc length to interpolate
//                 points (in degree)
// Purpose:  to calculate the meridian line in the map range, and save
//           in *line. The transformation matrix should have been calculated
//           already.
//
void POLARMAPELM::CalMeridianLine(float &lon, float &arclen)
{
int i,num;
Spherical2 *mline;

TransMatrix trans=P12.GetTrans();
float *matrix,theta;
float darc=arclen*RAD;
float phi=lon*RAD;

  if     (phi<0.0)phi+=PI;
  else if(phi>PI)phi-=PI;
  assert(phi>=0.0 && phi<=PI);

  num=(int)(PI/darc)+1;
  darc=PI/(float)(num-1);	// adjusted increment
  
  mline=new Spherical2[2*num];
  
  for(i=0,theta=(float)0;i<num;i++,theta+=darc){// north pole to south
      mline[i].phi=phi;
      mline[i].theta=theta;
  }

  num*=2;

  phi+=PI;										// for lon+180 part
  for(theta=PI;i<num;i++,theta-=darc){			// south pole to north
      mline[i].phi=phi;
      mline[i].theta=theta;
  }

  matrix=trans.matrix;

  for(i=0;i<num;i++)			// under new coordinate
      mline[i]=mline[i].ForwardTrans(matrix);
  
  MapOpenLine(mline, num);		// find out the visible part of the line
 
  delete [] mline;
  
}


// Input:  lat --- longitude for the parallel line ( lat  -90  -> +90 )
//	   arclen --- appriximate increment in arc length to interpolate
//                 points (in degree)
// Purpose:  to calculate the parallel line in the map range, and save
//           in *line. The transformation matrix should have been calculated
//           already.
//
void POLARMAPELM::CalParallelLine(float &lat, float &arclen)
{
int i,num;
Spherical2 *pline;
TransMatrix trans=P12.GetTrans();
float *matrix;
float darc=arclen*RAD;
float phi,theta=((float)90-lat)*RAD;

  assert(theta>=0.0 && theta<=PI);

  num=(int)(TWOPI/darc)+1;
  darc=TWOPI/(float)(num-1);	// adjusted increment
  
  pline=new Spherical2[num];
  
  for(i=0,phi=(float)0;i<num;i++,phi+=darc){
      pline[i].phi=phi;
      pline[i].theta=theta;
  }

  matrix=trans.matrix;

  for(i=0;i<num;i++)			// under new coordinate
      pline[i]=pline[i].ForwardTrans(matrix);
  
  MapOpenLine(pline, num);		// find out the visible part of the line
 
  delete [] pline;

}

void POLARMAPELM::PrintLine()
{
int i,j;
   for(i=0;i<numline;i++){
       for(j=0;j<numpt[i];j+=30){
           cout<<j<<" "<<line[j].theta/RAD<<" "<<line[j].phi/RAD<<endl;
       }
   }
}
// assume the transformation matrix has already been calculated
// Input: coastline  --  a closed piece of coast line in spherical coordinate,
//			 it will not be altered in this function.

void POLARMAPELM::MapCloseLine(Spherical2 *coastline, int &num, float &arclen)
{
int i,j,k,count,itotal,in0,in1,within=FALSE,out=FALSE;
float theta0,theta1,phi0,phi1,phi,darc,dphi,theta;
TransMatrix	trans=P12.GetTrans();
Spherical2	*coast,*coast0;

  coast=new Spherical2[2*num+1];		// twice the space for wrap around
  darc=arclen*RAD;

  for(i=0;i<num;i++){
      coast[i]=coastline[i].ForwardTrans(trans.matrix);	// under new coordinate
      if(coast[i].theta>Range)	out    =TRUE;
      else						within =TRUE;
  }
  if(!within){		// coast line is completely out of the map
      tnumpt=numline=0;
  }else if(!out){	// coast line is completely within map
      numline=1;	// one complete line
      numpt=new int[numline];
      numpt[0]=tnumpt=num;
      line = new Spherical2[tnumpt];
      xy   = new Cartesian2[tnumpt];
      for(i=0;i<tnumpt;i++)line[i]=coast[i];	// save it
  }else{		// partially in and partially out of the map
      itotal=num;
      theta0=coast[num-1].theta;
      for(i=0;i<num;i++,theta0=theta1){		// find the 1st in/out break point
         theta1=coast[i].theta;
          if((theta0<=Range && theta1> Range) ||
			(theta0> Range && theta1<=Range))break;
      }
      k=i;
	  for(i=0,j=num;i<k;i++,j++)coast[j]=coast[i];	// wrap the first part to the end
      coast0=&coast[k];				// shifted now
      coast0[itotal]=coast0[0];				// add a point and make it same as point 0
      itotal++;
      numline=1;					// one complete line
      numpt= new int[numline];
      line = new Spherical2[num+1800];			// do not know how many points are needed
							// to close  coast line, 1800 should be enough                     
      xy   = new Cartesian2[num+1800];

      theta0=coast0[0].theta;
      if(theta0<=Range){		// 1st point is in
         in0 = TRUE;
         line[0]=coast0[0];
         k=1;
         count=0;
      }else{				// 1st point is out
         in0 = FALSE;
         k=0;
         count=1;
         phi0=coast0[0].phi;
      }
      for(i=1;i<itotal;i++,in0=in1){
        theta=coast0[i].theta;
        if(theta<=Range)in1= TRUE;
        else            in1=FALSE;
       
        if ((in0 && !in1) ||(!in0 && in1) ){
           count++;
           if     (count==1)phi0=coast0[i].phi;
           else if(count==2)phi1=coast0[i].phi;
        }

        if(count==2){
          if(phi1>=phi0){
             if((phi1-phi0)<PI){
                 dphi = darc;
             }else{
                 phi1-= TWOPI;
                 dphi = -darc;
             }
          }else{
             if((phi0-phi1)<PI){
                 dphi=-darc;
             }else{
                 phi0-=TWOPI;
                 dphi =darc;
             }
          }
          
          if(phi1>phi0){
             for(phi=phi0; phi<=phi1; phi+=dphi,k++){
                 line[k].phi=phi;
                 if(line[k].phi<-PI)line[i].phi+=TWOPI;
                 line[k].theta=Range;
				 assert(k<num+1800);             }
          }else{
             for(phi=phi0; phi>=phi1; phi+=dphi,k++){
                 line[k].phi=phi;
                 if(line[k].phi<-PI)line[i].phi+=TWOPI;
                 line[k].theta=Range;
				 assert(k<num+1800);
             }
          }

          count=0;
         }
         if(in1){
            line[k]=coast0[i];
            k++;
			assert(k<num+1800);
         }
    }
    numpt[0]=tnumpt=k;
  }			// end of "partially in and partially out of the map"
  delete [] coast;

}
//
// arclen -- in degree
//
void POLARMAPELM::Frame(float &arclen)
{
float phi,dphi;
int i;

  numline=1;
  numpt= new int[numline];

  numpt[0]=tnumpt=(int)(TWOPI/(arclen*RAD))+1;
  line = new Spherical2[tnumpt];
  xy   = new Cartesian2[tnumpt];	// for later use, temp storage

  dphi=TWOPI/(float)(tnumpt-1);

  for(i=0,phi=(float)0;i<tnumpt-1;i++,phi+=dphi){
      line[i].theta=HPI;
      line[i].phi  =phi;
  }
  line[tnumpt-1]=line[0];
}


Cartesian2* POLARMAPELM::Geo2UserCoor()
{
float r;

   for(int i=0;i<tnumpt;i++){
       r=line[i].theta/RAD;
       xy[i].x=r*(float)cos(line[i].phi);
       xy[i].y=r*(float)sin(line[i].phi);
   }

   return xy;
}


Cartesian2* POLARMAPELM::GetLinePt(int &linenum)
{
int i, ind;
    
    for(i=ind=0;i<linenum;i++)ind+=numpt[i];
    return &xy[ind];
}

int LatLatLine::ReadData(char *flname)
{
fstream inFile;
int i,j,k;
Geocentric lalo;

 cout<<"filename = "<<flname<<endl;

 inFile.open(flname,ios::in);	// data file
 
 if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<flname<<" for input\n";
     return FALSE;
 }

 inFile >>numline>>tnumpt;	// total numbers of lines and points
 numpt = new int[numline];
 line  = new Spherical2[tnumpt];

 for(k=0,i=0;i<numline;i++){
     inFile >> numpt[i];	// num of points for this line
     for(j=0;j<numpt[i];j++,k++){
         inFile>>lalo.lat>>lalo.lon;
		 lalo.lat*=(float)0.1;
		 lalo.lon*=(float)0.1;
         line[k].FromGeocentric(lalo);
     }
 }

 cout <<" k = "<< k<<" tnumpt="<<tnumpt<<endl;
 inFile.close();

 return TRUE;
}

LatLatLine::~LatLatLine()
{
 delete [] numpt;
 delete [] line;
}


Spherical2 *LatLatLine::GetLinePt(int &linenum)
{
int i, ind;
    
    for(i=ind=0;i<linenum;i++)ind+=numpt[i];
    return &line[ind];
}

//
// to calculate visible part of the path between two points on a sphere
// arclen -- increment for the arc (in degree)
//
void POLARMAPELM::PathBt2Pts(TwoPtsSph *sr, int arc, float arclen)
{
int i,num;
Spherical2 *path;
float *matrix,phi,dist,phibeg,phiend;
float darc=arclen*RAD;

  dist=sr->CalDist();		// distance between two points (in radian)
  sr->CalTrans4();  		// trans mat which would trans two points
							// to equator (new coordinate).

  if(arc==GREATCIRCLE){
	 phibeg=(float)0;
	 phiend=TWOPI;
  }else if(arc==MAJORARC){
	 phibeg=(float)0;
	 phiend=dist;
  }else if(arc==MINORARC){
	 phibeg=dist;
	 phiend=TWOPI;
  }

  num=(int)((phiend-phibeg)/darc)+1;
  darc=(phiend-phibeg)/(float)(num-1);	// adjusted increment
  
  path=new Spherical2[num];
  
  for(i=0,phi=phibeg;i<num;i++,phi+=darc){
      path[i].phi=phi;
      path[i].theta=HPI;
  }

  matrix=sr->trans.matrix;

  for(i=0;i<num;i++)			// under new coordinate
      path[i]=path[i].ForwardTrans(matrix);
  
  MapOpenLine(path, num);		// find out the visible part of the line
 
  delete [] path;

}

//
// to solve A*X**2+B*X+C=0
// the return value indicates the number of solutions (=0 no solution)
//
int equation2(float *x, float A, float B, float C)
{
float value=B*B-(float)4*A*C;
int numroot=0;

 if(value>0.0){
   numroot=2;
   x[0]=(-B+(float)sqrt(value))/((float)2*A);
   x[1]=(-B-(float)sqrt(value))/((float)2*A);
 }else if(value==0.0){
   x[0]=-B/((float)2*A);
   numroot=1;
 }
 return numroot;
}

// Assume we already have a trans matrix in P12
// Input: line0  --  a line in spherical coordinate.
//			line0 will not be altered in this function.
// Output:	line[] -- the visible part of line0,
//					  it might be in a few different pieces.
void POLARMAPELM::MapOpenLine(Spherical2 *line0, int &num)
{
int i,j,k,in0,in1,npcl,within=FALSE,out=FALSE;
int in0_save,tnumpt_save,numline_save;
TransMatrix	trans=P12.GetTrans();
Spherical2	*tline,*linept;
float theta0,theta1;

  tline=new Spherical2[2*num];		// twice the space for wraping around

  for(i=0;i<num;i++){
      tline[i]=line0[i].ForwardTrans(trans.matrix);	// under new coordinate
      if(tline[i].theta<=Range)within =TRUE;
      else                     out    =TRUE;
  }
  if(!within){		// tline line is completely out of the map
      tnumpt=numline=0;
  }else if(!out){	// tline line is completely within map
      numline=1;	// one complete line
      numpt=new int[numline];
      numpt[0]=tnumpt=num;
      line = new Spherical2[tnumpt];
      xy   = new Cartesian2[tnumpt];
      for(i=0;i<tnumpt;i++)line[i]=tline[i];	// save it
  }else{		// partially in and partially out of the map
      theta0=tline[num-1].theta;
      for(i=0;i<num;i++,theta0=theta1){		// find 1st in/out break point
         theta1=tline[i].theta;
         if((theta0<=Range && theta1> Range) ||
			(theta0> Range && theta1<=Range))break;
      }
      k=i;
      for(i=0,j=num;i<k;i++,j++)tline[j]=tline[i];	// wrap 1st part to end

      linept=&tline[k];        // shifted now

      if(linept[0].theta<=Range){   // 1st point is in
         in0 = TRUE;
         numline=tnumpt=1;
      }else{                        // 1st point is out
         in0 = FALSE;
         numline=tnumpt=0;
      }
	  in0_save=in0;
	  for(i=1;i<num;i++,in0=in1){		// find out how many points are in
        if(linept[i].theta<=Range){		// and how many lines
		   in1=TRUE;
		   tnumpt++;
		}else{
		   in1=FALSE;
		}
		if(!in0 && in1)numline++;
	  }

	  numline_save=numline;
	  tnumpt_save=tnumpt;

	  numpt=new int[numline];       // to save the number of points
	  line =new Spherical2[tnumpt];
	  xy   =new Cartesian2[tnumpt]; // for later use

	  in0=in0_save;
	  if(in0){						// 1st point is in
		 numline=tnumpt=1;
		 npcl=1;					// num of pt for current line
		 line[0]=linept[0];
	  }else{
		 numline=tnumpt=0;
	  }

	  for(i=1;i<num;i++,in0=in1){
        if(linept[i].theta<=Range){
		   in1=TRUE;
		   line[tnumpt++]=linept[i];
		   assert(tnumpt<=tnumpt_save);
		   npcl++;
		}else{
		   in1=FALSE;
		}
		if(!in0 && in1){				// beginning of a new line
		   numline++;
		   assert(numline<=numline_save);
		   npcl=0;
		}else if(in0 && !in1){			// end of the previous line
		   numpt[numline-1]=npcl;
		}
	  }
     
	  if(in1)numpt[numline-1]=npcl;		// end of the last visible line
		 
	  assert(tnumpt==tnumpt_save);
	  assert(numline==numline_save);
  }

  delete [] tline;
}

AMAT_ROW::~AMAT_ROW()
{
  cout <<"........delete AMAT_ROW.........\n";
  if(numa>0){
     delete [] arow;
     delete [] irow;
  } 
}

BS_AMAT_ROW::BS_AMAT_ROW(int na)
{
  cout <<"........contruct BS_AMAT_ROW.........\n";
  Init(na);
  numgpt=numknt=neq=ist=0;
  amatopt = -99;	
  eqlat=eqlon=eqdep=stlat=stlon=stelev=res1=res2=time=0.0;
}


void BS_AMAT_ROW::RdAmatModPara(int &ngpt,int &maxk)
{

    read(fd,&numgpt,sizeof(int));
    read(fd,&numknt,sizeof(int));
    numknt++;				// number of points -1 is save in matrix file
    if(numknt!=maxk || numgpt!=ngpt){
       cout << "Inconsistent model and matrix\n";
       cout << "Model:  num of radial knots = " << maxk << endl;
       cout << "Matrix: num of radial knots = " << numknt << endl;
       cout << "Model:  num of horizontal grid points = " << ngpt << endl;
       cout << "Matrix: num of horizontal grid points = " << numgpt << endl;
       exit(1);
    }else{
       cout << "Matrix: num of radial knots = " << numknt << endl;
       cout << "Matrix: num of horizontal grid points = " << numgpt << endl;
    }

}

void BS_AMAT_ROW::RdAmatModPara(int &ngpt,int &maxk, int &opt)
{

    read(fd,&numgpt,sizeof(int));
    read(fd,&numknt,sizeof(int));
    numknt++;   // the numknt saved is numknt-1
    if(opt != 1) {  // opt=1 is the case of old format of Su et al.
       read(fd, &amatopt, sizeof(int));
       cout << "numknt = " << numknt << endl;
    }
    if(numknt!=maxk || numgpt!=ngpt){
       cout << "Inconsistent model and matrix\n";
       cout << "Model:  num of radial knots = " << maxk << endl;
       cout << "Matrix: num of radial knots = " << numknt << endl;
       cout << "Model:  num of horizontal grid points = " << ngpt << endl;
       cout << "Matrix: num of horizontal grid points = " << numgpt << endl;
       exit(1);
    }else{
       cout << "Matrix: num of radial knots = " << numknt << endl;
       cout << "Matrix: num of horizontal grid points = " << numgpt << endl;
    }
}



int BS_AMAT_ROW::CompARow(BS_AMAT_ROW& arow1)
{
int error=FALSE;

   if(fabs(eqlat-arow1.eqlat)>0.1)error=TRUE;
   if(fabs(eqlon-arow1.eqlon)>0.1)error=TRUE;
   if(fabs(eqdep-arow1.eqdep)>0.1)error=TRUE;
   if(fabs(stlat-arow1.stlat)>0.1)error=TRUE;
   if(fabs(stlon-arow1.stlon)>0.1)error=TRUE;
   return error;
}

int BS_AMAT_ROW::ReadARow()
{
float buf[9];
int ibuf[3];
int numrd;
int n1;

  numrd=read(fd,buf,sizeof(buf));
  if(numrd==0){
    cout<<"........Reached the end of file...........\n";
    return numrd;		// reaches the end of file
  }
  eqlat=(float)buf[0];
  eqlon=(float)buf[1];
  eqdep=(float)buf[2];
  stlat=(float)buf[3];
  stlon=(float)buf[4];
  stelev=(float)buf[5];		// has been set to 0 for summary stations
  res1=(float)buf[6];		// observed residual
  res2=(float)buf[7];		// predicted residual
  time=(float)buf[8];		// PREM predicted travel time, not used here
//  cout << eqlat << " " << eqlon  << " " << eqdep << " " << stlat 
//    << " " << stlon << " " << stelev << " " <<res1 << " " <<
//             res2 << " " <<time << endl;
  read(fd,ibuf,sizeof(ibuf));
  neq=ibuf[0];			// neq and ist are not used for summary residuals	
  ist=ibuf[1];	
  numnon0=ibuf[2];		// # of nonzero elements in this row of matrix A
  assert(numnon0<=numa);	// less than the size of arow	
  
  if(numnon0>0){
     read(fd,irow,numnon0*sizeof(short));
     read(fd,arow,numnon0*sizeof(float));
  }

//  cout <<"**********************************\n";
//  cout << " eqlat= "<< eqlat << endl;
//  cout << " eqlon= "<< eqlon << endl;
//  cout << " eqdep= "<< eqdep << endl;
//  cout << " stlat= "<< stlat << endl;
//  cout << " stlon= "<< stlon << endl;
//  cout << " stelev= "<< stelev << endl;
//  cout << " res1= "<< res1 << " res2= "<< res2 << endl;
//  cout << " time= "<< time << endl;
//  cout << " ist= " << ist << endl;
//  cout << " neq= " << neq << endl;
  
  return numrd;

}

void AMAT_ROW::ReduceARow(int *index)
{
int i,j;

   for(i=0;i<numnon0;i++){
      j=irow[i];			// old index number
      irow[i]=index[j];		// new index number, -1 indicates discarded element
   }
}

void BS_AMAT_ROW::SaveARow(int opt, float amat2res2)
{
char filename[300];
int maxk;
float buf[9];
int ibuf[3];

   if(!file_opened){
      cout<<"file name for output:\n";
      cin >>filename;
      maxk=numknt-1;
      fd_out=SysOpen(filename,O_WRONLY);
      write(fd_out,&numgpt,sizeof(int));
      write(fd_out,&maxk,sizeof(int));
      if(opt!=1) write(fd_out, &amatopt, sizeof(int));
      file_opened=1;
   }

    buf[0]=eqlat;
    buf[1]=eqlon;
    buf[2]=eqdep;
    buf[3]=stlat;
    buf[4]=stlon;
    buf[5]=stelev;			// station elevation, set to 0.0 for sumres
    buf[6]=res1;			// observed residual
    res2 -= amat2res2;			// subtract initial model pred
    buf[7]=res2;			// predicted residual
//    cout << "final prediction = " << buf[7] << endl;
    buf[8]=time;			// PREM predicted travel time
    ibuf[0]=neq;			// earthquake index, not used, set to 0
    ibuf[1]=ist;			// station index, not used, set to 0
    ibuf[2]=numnon0;			// number of nozero elements in the matrix
    assert(numnon0<=numa);		// less than the size of arow	
    
    write(fd_out,buf,sizeof(buf));
    write(fd_out,ibuf,sizeof(ibuf));
    if(numnon0>0){
       write(fd_out,irow,sizeof(short)*numnon0);
       write(fd_out,arow,sizeof(float)*numnon0);
    }

}

void BS_AMAT_ROW::SaveARow_Scripts(int opt, float amat2res2, float amat2premtt)
{ // Saves the differential times for the Scripts data format
  // note:  the PREM differential time is included in the measurements
char filename[300];
int maxk;
float buf[9];
int ibuf[3];

   if(!file_opened){
      cout<<"file name for output:\n";
      cin >>filename;
      maxk=numknt-1;
      fd_out=SysOpen(filename,O_WRONLY);
      write(fd_out,&numgpt,sizeof(int));
      write(fd_out,&maxk,sizeof(int));
      if(opt!=1) write(fd_out, &amatopt, sizeof(int));
      file_opened=1;
   }

    buf[0]=eqlat;
    buf[1]=eqlon;
    buf[2]=eqdep;
    buf[3]=stlat;
    buf[4]=stlon;
    buf[5]=stelev;			// station elevation, set to 0.0 for sumres
    buf[6]=res1;			// observed residual
//    cout << " res1 = " << res1 << endl;
// subtract initial model pred, and also remove PREM differential time
// from the Scripts data
//    cout << "res2=" << res2 << " time=" << time << " amat2premtt" << amat2premtt << endl;
//    res2 = res2 + time - amat2premtt - amat2res2;
//    cout << "res2 true=" << res2 << " amat2res2=" << amat2res2 <<endl;
    buf[7]=res2;			// predicted residual
    buf[8]=time-amat2premtt;		// PREM predicted travel time
    ibuf[0]=neq;			// earthquake index, not used, set to 0
    ibuf[1]=ist;			// station index, not used, set to 0
    ibuf[2]=numnon0;			// number of nozero elements in the matrix
    assert(numnon0<=numa);		// less than the size of arow	
    
    write(fd_out,buf,sizeof(buf));
    write(fd_out,ibuf,sizeof(ibuf));
    if(numnon0>0){
       write(fd_out,irow,sizeof(short)*numnon0);
       write(fd_out,arow,sizeof(float)*numnon0);
    }

}



void AMAT_ROW::Init(int na)
{
  numa=na;
  fd=fd_out=0;
  file_opened=0;
  if(numa>0){
     arow=new float[numa];
     irow=new short[numa];
  }else{
     arow=NULL;
     irow=NULL;
  }

}

void AMAT_ROW::OpenMatFile(char *flname)
{
   fd=SysOpen(flname,O_RDONLY);
}

void AMAT_ROW::SubARow(AMAT_ROW &amat_row)
{
// subtract A matrices for differential travel times
float *temp_arow;
int   i,k,*temp_irow;

  temp_arow= new float[numa];
  temp_irow= new int[numa];
  
  for(i=0;i<numa;i++){
      temp_arow[i]=0.0;
      temp_irow[i]=-1;
  }
  
  for(i=0;i<numnon0;i++){
      k=irow[i];
      temp_arow[k]+=arow[i];
      temp_irow[k]=k;
  }

  for(i=0;i<amat_row.numnon0;i++){
      k=amat_row.irow[i];
      temp_arow[k]-=amat_row.arow[i];
      temp_irow[k]=k;
  }
  
  for(k=i=0;i<numa;i++){
      if(temp_irow[i]>=0){
         arow[k]=temp_arow[i];
         irow[k]=i;
         k++;
      }
  }
  numnon0=k;		// new number of none-zero elements

  delete temp_arow;
  delete temp_irow;
}


float AMAT_ROW::SumArow(float *modcoeff)
{
int i,j;
float res=0.0;

  for(i=0;i<numnon0;i++){
      j=irow[i];
      res+=arow[i]*modcoeff[j];
//      cout << i << "  " << j << "  " <<  arow[i] << "  " << modcoeff[j] << endl;
  }
      
  return res;
}

float AMAT_ROW::SumArow_dcshift(float dc410, float dc660, 
			int num3d, int hpt)
{ // get correction due to average depths of 410 and 660
  // relative to 400 and 670 km of PREM
int i,j;
float res=0.0;

  for(i=0;i<numnon0;i++){
      j=irow[i];
      if(j >= num3d && j< num3d + hpt)  res+=arow[i]*dc410;
      if(j >= num3d + hpt) res += arow[i]*dc660;
  }
      
  return res;
}

/*
 * mode = 0 --- for read, 1 for write and 2 for read and write
 */

int SysOpen(char *flname,int mode)
{
int fd;

  if(mode==1){
     if(creat(flname,PERMS)==-1){
        cout <<"error, cannot creat file\n";
        return -1;
     }
  }
  if((fd=open(flname,mode))==-1){
      cout<<" erorr in  opening  file "<<flname<<endl;
      return -1;
  }else{
      cout<<"open file "<<flname<<endl;
  }
  return fd;
}
 
float *ReadDamMat(int numknt)
{
fstream inFile;
float *dmat;		// damping matrix
char filename[132];
int i,total=numknt*numknt;

  dmat=new float[total];
  
  cout<<"Input radial damping matrix file name:\n";
  cin>>filename;

  inFile.open(filename,ios::in);
  
  if(!inFile) { 			// open failed?
     cerr << "Cannot open "<<filename<<" for input\n";
     exit(-1);
  }
  
  for(i=0;i<total;i++){
      inFile >> dmat[i];
//      cout << dmat[i]<<endl;
  }
 
  inFile.close();
     
  return dmat;

}

void Read3DDamMat(int &ndamp, int *inddamp, float *dmat)
{
fstream inFile;
char filename[132], desc[132];
int i, fd, total;

  
  cout<<"Input radial damping matrix file name:\n";
  cin>>filename;

  fd=SysOpen(filename,O_RDONLY); 
  read(fd,&desc,132);                                              
  cout << "desc = " << desc << endl;
  read(fd,&total,sizeof(int));                                              
  read(fd,&ndamp,sizeof(int));                                              
  cout << "total = " << total << " ndamp = " << ndamp << endl;
  dmat=new float[ndamp];
  inddamp=new int[ndamp];
  read(fd,&inddamp[0],ndamp*sizeof(int));                                              
  read(fd,&dmat[0],ndamp*sizeof(float));                                              
  close(fd);   
  exit(1);
}


float *Read_H_DamMat(int numgpt)
{
FILE *fp;
float *dmat,*dmat1, weight;		// damping matrix
float diagsum, diagavg;
char filename[132];
int i,total=numgpt*numgpt, row, col, nonzero;
int index, j, k, truedim;



  dmat=new float[total];
  truedim= numgpt*(numgpt+1)/2;
  dmat1 = new float [truedim];
  
  cout<<"Input horizontal damping matrix file name:\n";
  cin>>filename;

  if((fp = fopen(filename, "r")) == NULL) {
     cout << "can't find the file " << filename << endl;
     exit(1);
  }
  cout << "total = " << total << endl;
  for(i=0; i<total; i++){
     dmat[i] = 0.0;
  }
  i=0;
  while(fscanf(fp,"%d %d %f %d", &row, &col, &weight, &nonzero)  != EOF) {
     index = (row-1)*numgpt+(col-1);
     dmat[index] = weight;
//     cout << "row="<<row<< " col="<< col << " index=" << index << " damt=" << dmat[index] << endl;
  }
  fclose(fp);

  k=0; 
  diagavg = diagsum = 0.0;
  for(i=0; i<numgpt; i++){
     for(j=0; j <= i; j++){
        index = i*numgpt+j;
        dmat1[k] = dmat[index];     // change the matrix in the ata form
//        cout << "k = " << k << "index=" << index << "  " <<  dmat1[k] << endl;
        if(i==j) {
           diagsum += dmat1[k];
        }
        k++;
     }
  
  }   
  diagavg = diagsum/(float)numgpt;
  cout << "Damping Matrix Statistics:" << endl;
  cout << "========================================" << endl;
  cout  << "Diagonal sum  = " << diagsum << endl;
  cout << "Diagonal average = " << diagavg << endl;
  cout << "========================================" << endl;
  delete [] dmat;
  return dmat1;

}

float *Read_H_DamMat_New(int numgpt)
{
FILE *fp;
float *dmat,*dmat1, weight;		// damping matrix
float diagsum, diagavg;
char filename[132];
int i,total=numgpt*(numgpt+1)/2+1, row, col, nonzero;
int index, j, k, truedim;



  dmat=new float[total];
  
  cout<<"Input horizontal damping matrix file name:\n";
  cin>>filename;

  if((fp = fopen(filename, "r")) == NULL) {
     cout << "can't find the file " << filename << endl;
     exit(1);
  }
  for(i=0; i<total; i++){
     dmat[i] = 0.0;
  }

  k=0; 
  diagavg = diagsum = 0.0;
  for(i=0; i<numgpt; i++){
     for(j=0; j <= i; j++){
        fscanf(fp, "%f", &dmat[k]);
        if(i==j) {
           diagsum += dmat[k];
        }
        k++;
     }
  }   
  diagavg = diagsum/(float)numgpt;
  cout << "Damping Matrix Statistics:" << endl;
  cout << "========================================" << endl;
  cout  << "Diagonal sum  = " << diagsum << endl;
  cout << "Diagonal average = " << diagavg << endl;
  cout << "========================================" << endl;
  fclose(fp);
  return dmat;

}

void BS_IPM::ReadFull3DDamp()
{  // read 3D damping matrix in Weijia's format, Jeff Gu, 1999
   // this is very similar to ReadIpm of BS_IPM with the exception
   // of  1.  format is 1 which is different
   //	  2.  a comment field exists
int i,fd,n_read;
float theta_phi[2];
char filename[132];
char comment[120];

  cout <<"Name of 3D matrix:\n";
  cin >> filename;
  cout <<"......Reading ipm from file: "<<filename<<endl;

  fd=SysOpen(filename,O_RDONLY);
  read(fd,&format,sizeof(int));
  read(fd,comment,sizeof(comment));
  format=11;	// change Weijia's format to the ATA format
  read(fd,&numgpt,sizeof(int));	// number of points on sphere
  if(theta==0) theta=new float[numgpt];  
  cout << "format=" << format << "  numgpt=" << numgpt << endl;
  assert(theta!=0);
  if(phi==0) phi= new float[numgpt];  
  assert(phi!=0);
  for(i=0;i<numgpt;i++){		// coordinates
      read(fd,theta_phi,2*sizeof(float));
      theta[i]=theta_phi[0];
      phi[i]=theta_phi[1];
  }
  read(fd,&numrknt,sizeof(int));	// number of radial knots
  if(numrknt>1){
      cout <<"Radial knots described as radius\n";
      if(rknt==0) rknt=new float[numrknt];
      assert(rknt!=0);
      read(fd,rknt,numrknt*sizeof(float));
  }else{
      cout<<"Error in BS_IPM::ReadIpm, not complete yet\n";
  }
  printf("numgpt=%d, num radial knot=%d\n", numgpt, numrknt) ;
  numatd = numgpt*numrknt;
  for(i=0; i<numrknt; i++){
     printf("i=%d, knot=%f\n", i, rknt[i]);
  }
  ReadAtaEtc(fd);		// shared code, read in ata, atd etc
  close(fd);
}

void BS_IPM::DampFull3D(BS_IPM &mat2, float weight)
{ // add two inner product matrices and Atd matrices for different data sets
int i,j;

  cout << "damp[1000]=" << mat2.ata[1000] << " damp[10000]=" << mat2.ata[10000]
	<< " damp[100000]=" << mat2.ata[100000] << endl;
  cout << "damp[2351]=" << mat2.ata[2351] << " damp[333333]=" << mat2.ata[333333] 
	<< " damp[4444444]=" << mat2.ata[4444444] << endl;
   if(weight<0.0)
      cout<<"Warning IPM::AddAtaEtc, weight <0.0\n";

   if(numatd!=mat2.numatd || numata!=mat2.numata){
      cout<<"Error in IPM::AddAtaEtc, inconsistent matrix size\n";
      cout<<"numata1="<<numata<<" numata2="<<mat2.numata<<endl;
      cout<<"numatd1="<<numatd<<" numatd2="<<mat2.numatd<<endl;
      exit(-1);
   }
   for(i=10; i<15; i++) {
      j = i*100;
      printf("%d, %f %f\n", j, ata[j], atd[j]);
   } 
   for(i=10; i<15; i++) {
      j = i*100;
      printf("matrix 2 %d, %f %f\n", j, mat2.ata[j], mat2.atd[j]);
   } 
   cout<<"......adding ata......\n";
   cout << "damping factor = " << weight << endl;
   for(i=0;i<50;i++){
	cout << "i=" << i << "  ata[i]=" << ata[i] << " damp="<<mat2.ata[i] << endl;
   }
   for(i=0;i<numata;i++)ata[i]+=weight*mat2.ata[i];
   for(i=0;i<50;i++){
	cout << "i=" << i << "  ata[i]=" << ata[i] << " weight="<<weight << endl;
   }
}
