#ifndef MODHEADER
#define	MODHEADER		// model related definitions

#include <wsconst.h>

class MODEL {
   public:
      void InitModel();
	virtual void ReadModel(int, char *)=0;
	virtual void PrintModel()=0;
	void	PrintModInfor();
	void	Check3D();
	int	GetRegion() const { return region;}
	int	GetDim() const { return dim;}
	int	GetFormat() const { return format;}
	int	GetRadFun()	const { return rfun;}
	int	GetHorFun() const { return hfun;}
	int	NumCoeff(int &deg) const { return (deg+1)*(deg+1);}
	virtual int	TotNumCoeff() =0;
	float	GetRtop() const { return rtop;}
	float	GetRbot() const { return rbot;}
	float	GetRmid() const { return rmid;}
	float*  GetModCoeff(void) const { return coeff;}
	void  Add(float&, MODEL &);
	virtual void	SumModelAtR(float &)=0;
	virtual void	SumRadCoeff(float*, int&, int&)=0;
   protected:
	int	format;	// model format, HVDHARM or HVDBS etc.
	int	dim;		// model dimension
	int	region;	// the region for which model is defined
	int	rfun;		// radial function type
	int	hfun;		// horizontal function type
	int 	numknt;		// # of radial knots
	int 	numknt_lm;	// # of radial knots in LM 
	int 	numknt_um;	// # of radial knots in UM 
	float	rtop,rmid,rbot;
	float	*coeff;	// model coefficients
	char	desc[132];	// model description 
};

class SP_MODEL: public MODEL{	// harvard format spherical harmonic model
   public:
	~SP_MODEL();
	void ReadModel(int choice=WHOLE_EARTH,char *flname="hvdharmdl.dat");
	void PrintModel();
	void PrintModCoeff(float*, int &);
	void	SumModelAtR(float &);
	void	SumRadCoeff(float*, int&, int&);
	int	TotNumCoeff();
   private:
	int maxwl;		// spherical harmonic degree for whole mantle model
	int maxul;		// for upper mantle
	int maxll;		// for lower mantle
	int maxwk;		// radial order for whole mantle
	int maxuk;		// for upper mantle
	int maxlk;		// for lower mantle
	int lcrust;		// max degree for crust
	int l670;		// max degree for 670
	float *coeff_crust;
	float *coeff_670;
	float *coeff_r;	// coeff synthesized at r;
	float *rknt;	// knots expressed as radius
	float *dknt;	// knots expressed as depths
	char *degmask;	// degree mask;
	char *laymask;	// layer mask
};

class CS_MODEL: public MODEL{	// 2D complex spherical harmonic model
   public:
	void ReadModel(int choice=WHOLE_EARTH,char *flname="ComplexHarm2D.dat");
	void PrintModel();
	void	SumModelAtR(float &);
	void	SumRadCoeff(float*, int&, int&);
	void WriteModel();
	int	TotNumCoeff()  { return (maxl+1)*(maxl+1);}
	~CS_MODEL();
   private:
	int maxl;		// spherical harmonic degree
	char *degmask;	// degree mask;
};

class BS_MODEL: public MODEL{
   public:
	BS_MODEL();
	~BS_MODEL();
	void ReadModel(int choice=WHOLE_EARTH,char *flname="S12WM13.XBS");
	void ReadModel_split(int choice=WHOLE_EARTH,char *flname="s12wm13_split.XBS");
	void ReadModel_multisplit(int choice=WHOLE_EARTH,char *flname="s12wm13_multisplit.XBS");
	void ReadTopoModel(int choice,char *flname, float *);
	void Read1stLine(fstream &inFile, int choice);
	void ScaleModel(int);
	void WriteModel(int,char);
	void PrintModel();
	void InitKnots(int);
	void CalAvgDist();
	void SetAvgDist(float avgd){ avgdist=avgd;}
	void ChangeHRegion(int hreg){ HRegion=hreg;}
	float GetAvgDist() const { return avgdist;}
	float DistBt2(const int&, const int&);
	int   GetNumgpt() const { return numgpt;}
	int   GetNumknt() const { return numknt;}
	float*   GetRadknt() const { return rknt;}
	float*   GetTheta() const { return theta;}
	float*   GetPhi() const { return phi;}
	int	TotNumCoeff();
	void  Add2Model(float *, int, int,int);
	void  Add2Model(float *, int *);
	void	SumModelAtR(float &);
	void	SumRadCoeff(float*, int&, int&);
	void  ScaleCoeff(float);
	int  GetNumSplit() const {return nsplit_um+nsplit_lm;}
	int   *nbsperlayer;
	int	ExcludeRegMod(BS_MODEL &, int*);
  private:
	int numgpt;		// number of horizontal grid points
	float *theta,*phi;
	float *rknt;	// knots expressed as radius
	float avgdist;	// average distance between grid points
	int *ajknts;	// to save adjacent points
	int nsplit_um, nsplit_lm;
	int HRegion;	// whether defined on whole earth or only regional
};

// void (*RadFuncs[2])(float,float*, int,float*);
int ChebyshevFun(float,float*, int, float*);
int BsplineFun(float, float*, int, float*);

class PolyPrem{			// polynomial PREM (one polynomial for each region)
   friend class DeckPrem;
	public:
	   void ReadPolyPrem();
	   void PrintPolyPrem();
	   void InitPolyPrem(int);
	   ~PolyPrem();
	private:
	   int numreg;			// number of regions defined in prem
	   float radius;			// Earth's radius
	   float *rtop,*rbot;	// radius at top and bot of each region
	   float (*rhopol)[5],(*qspol)[5],(*qkpol)[5],(*etapol)[5];
	   float (*vpvpol)[5],(*vsvpol)[5],(*vphpol)[5],(*vshpol)[5];
};


#endif		// end of #ifndef MODHEADER
