	program sphsph
c       The program interpolates the data input by using the
c	spherical harmonic expansion. It is designed
c	to calculate the model rms, power spectra at a 
c 	specific depth, or depths.
	implicit real*4 (a-h, o-z)
	parameter (lsize = 36 )       ! harmonic degree
	parameter (nunk = (lsize+1)**2)
	parameter (idsize = 5000)
	parameter (ilata = nunk*(nunk+1)/2)
	parameter(maxdata=100000)
	dimension xla(maxdata),xlo(maxdata)
c	
	real phi(maxdata), theta(maxdata), pi, val, templon
	real ylm(0:lsize, 0:lsize),rowa(nunk), x(nunk)
        integer idsize, lsize, nunk, idim,ilsq, lmx
	character*256 outfl

	pi = 3.14159265359
	call GetGrid(xla,xlo,dlat,dlon,xlatmin,xlatmax,xlonmin,xlonmax,numpt)
	
c	read in data and convert lat and long to theta, phi
	do i=1, numpt
		theta(i)=pi*0.5-xla(i)*pi/(180.0)
		templon = xlo(i)
		if(xlo(i).lt.0.0) templon = 360.0 + templon
		phi(i) = templon*pi/180.0
	enddo
	call get_coeff(lmx, x)
	print*, 'maximum degree = ', lmx
	write(*,"('output in [1=lat,lon,val; 2=gmt format; 3=dist_along_arc]:')")
	read(*,*) iformat
	print*, 'enter output file:'
	read(*, "(a)") outfl
	open(2, file=outfl, type='unknown')

	ilsq=(lmx+1)**2
c
c  initialize all the arrays.
c
	print*, 'initializing arrays...'
	do i =0, lmx
	   do j = 0, lmx
		ylm(i,j) = 0.0
	   end do
        end do
	do i=1, ilsq
		rowa(i)=0.0
	end do

	print*, 'numpt = ', numpt
	if(iformat.eq.3) then
		call delazc(xla(1), xlo(1), xla(numpt), 
     c			xlo(numpt), xdelta, azep,azst)
		print*, 'path length = ', xdelta	
		del0 = (180.0-xdelta)*0.5
		xdel = xdelta/(numpt-1)  ! dtheta for ray path
		del = 0.0
	endif
	do i=1, numpt
	  icol=0
	  call spherical(theta(i), lmx, ylm,lsize)

	  do j=0, lmx
	    do k=0, j
	     if(k.eq.0)  then
	         icol = icol +1
	         rowa(icol)=ylm(j,0)
	     else	    
	       icol = icol +1
	       rowa(icol)=ylm(j,k)*cos(k*phi(i))
	       icol=icol +1
	       rowa(icol)=ylm(j,k)*sin(k*phi(i))
	     endif
	    enddo
	  enddo
	  val = 0.0
	  do icl=1, icol
	        val = val+rowa(icl)*x(icl)
	  enddo
	  if(iformat.eq.1) then
		if(xlo(i).gt.180.) xlo(i)= xlo(i) - 360.
	   	write(2,*) xla(i), xlo(i), val			
	  else if(iformat.eq.2) then
	   	write(2,*) xlo(i), xla(i), val
c	   	print*, xlo(i), xla(i), val
	  else if(iformat.eq.3) then
	   	write(2,*) del, val
		del = del + xdel
	  endif
	enddo
	close(2)
	end

	subroutine get_coeff(lmx, x)
	integer ll, mm, lmx
	real temp, x(1)
	character*256 coef_file
 
	print*, 'maximum angular harmonic degree l :'
	read*, lmx 

	print*, 'enter spherical harmonic coefficients:'
	read(*, "(a)") coef_file
	open(15, file = coef_file, type = 'old')

	k = 0
	do l=0,lmx
	   k=k+1
	   read(15, *) ll, mm, x(k), temp
	   do m=1,l
	      k=k+1
	      read(15, *) ll, mm, x(k), x(k+1)
	      x(k) = x(k)*2.0
	      x(k+1) = -x(k+1)*2.0
	      k=k+1
	   enddo
	enddo
	print*, 'number of coefficients =', k
	close(15)
	return
	end
	
	

c.......................................................................
c.....ylm.f
c     This subroutine is to compute the spherical harmonics
c     see "Numberical Recipes in C", p194 for the definition of the
c     spherical harmonics.
c..Input:  theta-- spherical coordinate
c          lmax -- angular order
c..Output: p(0:lmax,0:lmax) -- spherical coordinate
c..Note: in this subroutine exp(im*phi) is omitted
c.......................................................................
      subroutine spherical(theta,lmax,p,lsize)
      implicit real*4 (a-h, o-z)
      real*4 p(0:lsize,0:lsize)
      data fpi /12.56637061435916/
   
      x=cos(theta)

      y0    =sqrt(1.0/fpi)
      p(0,0)=y0
      p(1,0)=sqrt(3.0/fpi)*x
      fac   =2.0
      sqrx2 =sqrt(1.0-x*x)
      
      do  l=1,lmax
         y0=-sqrt((fac+1.0)/fac)*sqrx2*y0
         fac=fac+2.0
         p(l,l)=y0
         if(l.ne.lmax)p(l+1.d0,l)=sqrt(fac+1.0)*x*y0
      enddo

      do l=2,lmax
         do m=0,l-2
            xl2=2*l
            xlmm=l-m
            xlpm=l+m
            fac =sqrt((xl2+1.00)/(xlmm*xlpm))
            fac1=sqrt (xl2-1.00)
            fac2=sqrt((xlmm-1)*(xlpm-1)/(xl2-3.0))
            p(l,m)=(x*fac1*p(l-1,m)-fac2*p(l-2,m))*fac
         enddo
      enddo
      return
      end

      subroutine choles_local(a,g,b,y,x,n,nono)

      implicit real*4 (a-h, o-z)
      real*4 a(1),g(1)
      real*4 b(1),y(1),x(1)
c
c        a= row-wise p.d. symm. system  n*(n+1)/2
c        g= cholesky storage
c        b= r.h.s. vector               n
c        y= temp. vector
c        x= answer vector
c        n= system dimension
c        nono .gt. 0 is the level at which p.d. failed
c
c        (a,g) and (b,y,x) may be equivalenced.
c
c----------------------------------------------------------
c-----
c     first compute cholesky decomposition

      nono=0
      
      if(a(1).le.0.) then
      nono=1
      return
      endif
      
      g(1)=sqrt(a(1))
      y(1)=b(1)/g(1)

      do 400 i=2,n
      
      kz=(i*(i-1))/2
      g(kz+1)=a(kz+1)/g(1)
      sg=g(kz+1)**2
      y(i)=b(i)-g(kz+1)*y(1)
      
      if(i.gt.2) then
      
      jmax=i-1
      
      do 200 j=2,jmax
      
      gkz=a(kz+j)
      kj=(j*(j-1))/2
      kmax=j-1
      
      do 100 k=1,kmax
      gkz=gkz-g(kz+k)*g(kj+k)
100   continue

      g(kz+j)=gkz/g(kj+j)
      y(i)=y(i)-g(kz+j)*y(j)
      sg=sg+g(kz+j)**2
      
200   continue

      endif
      
      gkz=a(kz+i)-sg
      
      if(gkz.le.0.) then
      nono=i
      return
      endif
      
      g(kz+i)=sqrt(gkz)
      y(i)=y(i)/g(kz+i)
      
400   continue

      kz=(n*(n-1))/2
      x(n)=y(n)/g(kz+n)
      if(n.le.1) return

c-----
c     compute solution for particular rhs
      
      do 600 k=2,n
      
      i=n+1-k
      x(i)=y(i)
      jmin=i+1
      
      do 500 j=jmin,n
      kj=(j*(j-1))/2
      x(i)=x(i)-g(kj+i)*x(j)
500   continue

      kz=(i*(i+1))/2
      x(i)=x(i)/g(kz)
      
600   continue

      return
      end
c
      real*4  function geocen(arg)
c***  returns geocentric colat given geographic colat as arg
c***  n.b. fac=(1-f)**2
c***  argument in radians
      implicit real*4(a-h,o-z)
      data pi2,fac/1.570796326794895d0,0.993305621334896d0/
      temp = sin(arg);
      if(sin(arg).lt.1.d-200) temp=1.d-200
      geocen=pi2-atan(fac*cos(arg)/temp)   
      return
      end

