typedef struct CARTESIAN {
        float x;
        float y;
        float z;
        } CARTESIAN;
        
typedef struct CARTESIAN_2 {
        float x;
        float y;
        } CARTESIAN_2;  
           
typedef struct TRANSMATRIX{
        struct CARTESIAN x;
        struct CARTESIAN y;
        struct CARTESIAN z;
        } TRANSMATRIX;
        
typedef struct SPHERE {
        float r;
        float theta;
        float phi;
        } SPHERE;
        
typedef struct SPHERE2 {
        float theta;
        float phi;
        } SPHERE2;
        
typedef struct CYLINDRIC {
        float r;
        float phi;
        } CYLINDRIC;

typedef struct RECTANGLE {
        float centerx;
        float centery;
        float dx;
        float dy;
        } RECTANGLE;

typedef struct PREM{  
        float r;                                       /* radius       */
        float rho0;float rho1;float rho2;float rho3;
        float g;
        float ell;
        float eta;
        float a0;float a1;float a2;float a3;   /* these variables are  */
        float c0;float c1;float c2;float c3;   /* defined in PREM      */
        float l0;float l1;float l2;float l3;   /* Dziewonski and Ander-*/
        float n0;float n1;float n2;float n3;   /* son (1981)           */
        float f0;float f1;float f2;float f3;
        float qshear;
        float qkappa;
       } PREM;
       
typedef struct MODEL{
        int     mod;            /* model                                  */
        char    *dsc;           /* model description                      */
        int     pors;           /* P or S velocity model                  */
        int     velfm;          /* velocity form (e.g. dv, dv**2/v dv/v)  */
        int     dim;            /* dimensions of the model (1, 2 or 3)    */
        int     mantle;         /* upper, lower, whole or separate mantle */
        int     radfun;         /* radial function                        */
        int     lmaxum;         /* upper mantle or whole mantle lmax      */
        int     kmaxum;         /* upper mantle or whole mantle kmax      */
        int     kmaxlm;         /* lower mantle  kmax                     */
        int     lmaxlm;         /* lower mantle  lmax                     */
        int     crust;          /* whether the model has a layer for crust*/
        int     p670;           /* layer for 670 perturbation             */
	float   **bsloc;	/* lat and lon of surface b-splines */
        float   **au;
        float   **bu;
        float   **al;
        float   **bl;
        float   *ac;
        float   *bc;
        float   *a670;
        float   *b670;
        float   *ar;
        float   *br;
        float   *ylm;
        float   rtop;
        float   rmid;
        float   rbot;
        PREM    *par;            /* pointor to the PREM model parameters   */
        char    *name1;
        char    *name2;
        char    degmask[132];
        char    layermask[132];
        float   **rknot;         /* radius at the knots for layered model  */
        float   *vavg;           /* average velocity for layered model     */
        float   *xmima;          /* min and max value for BSPLINE model    */
                                 /* or lat, lon, range for shell model     */
        int     *ixyz;           /* ix0, iy0 , iz0                         */
        int		**ajnode;
/* added for multiple split B-spline model */
	int     nsplit[2];	 /* # splits in LM(0) and UM(1)		   */
	int     nbsperlay[100];	 /* # splines in a layer		   */
	/* Note: rknot is used in multi-split BS model, not ar and br */
       } MODEL;

typedef struct  CMT {
        long  dum;
        char  devent[8];
        char  isour[4];
        long  year;
        long  month;
        long  day;
        long  hour;
        long  minute;
        float sec;
        float eplat;
        float eplong;
        float depth;
        float xmb;
        float xms;
        char  ireg[24];
        long  isb;
        long  icb;
        long  icutb;
        long  ism;
        long  icm;
        long  icutm;
        float torg;
        long  jh;        /* times from CMT        */
        long  jmin;
        float xsec;
        float errt;
        float epa;       /* CMT latitude          */
        float xlat;
        long  ins;
        float erra;
        float epo;       /* CMT longitude         */
        float xlon; 
        long  iew;
        float erro;
        float xd;        /* source depth from CMT */
        float errd;
        float durt;      /* source duration       */
        long  iexp;      /* scalar moment sc**iexp*/
        float xm[6];     /* moment tensor         */
        float xerr[6];
        float ev[3];
        long  ipl[3];
        long  iaz[3];
        float sc;        /* scalar moment sc**iexp*/
        long  istr[2];
        long  idip[2];
        long  islp[2];
        float dumm;           
      } CMT;

typedef struct  EQ {
        float  lon;
        float  lat;
        float  depth;
        float  mag;
      } EQ;

typedef struct ISCEQ{
     short  iymag1;   /* year and magnitude 1 (mb)        */
     short  mondep1;  /* month and depth1 (from P)        */
     short  daydep2;  /* day and depth2   (from pP)       */
     short  hournp;   /* hour and # of stations reporting */
     short  minmag2;  /* minute and magnitude 2           */
     short  jtsec;    /* second                           */
     short  lat;      /* lat                              */
     short  lon;      /* lon                              */
     long   rec;      /* record number                    */
     short  nobs;     /* no. of P observation for this eq */
     short  iflag;    /* flag indicator                   */
} ISCEQ;

typedef struct MARKER{ 
      int     linecolor;
      int     fillcolor;
      int     style;
      int     side;
      float   angle;
      float   linewidth;
      float   expansion;
      float   ratio;
      } MARKER;

typedef struct ISCST{
	char stname[4];	/* station names	*/
	float lat;		/* station latitude (in degree)	*/
	float lon;		/* station longitude (in degree)	*/
	int	elev;		/* station elevation (in meters)	*/
}ISCST;


typedef struct IPM{
	int numatd;
	float *ata;
	float *atd;
}IPM;
